/* -*- mode: C++; tab-width: 4 -*- */
/* ===================================================================== *\
	Copyright (c) 1998-2000 Palm, Inc. or its subsidiaries.
	All rights reserved.

	This file is part of the Palm OS Emulator.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
\* ===================================================================== */

#define PILOT_PRECOMPILED_HEADERS_OFF
#include "ROM Transfer.h"
#include <SysEvtMgr.h>
#include <SerialMgrOld.h>


#define kXmodemBlockSize	1024
#define kXmodemSoh			'\002'
#define kXmodemEof			'\004'
#define kXmodemAck			'\006'
#define kXmodemNak			'\025'
#define kXmodemCan			'\030'
#define kXmodemNakCrc		'C'

static char*	GetSmallROMStart (void);
static UInt32	GetSmallROMSize (char* smallROMStart);
static char*	GetBigROMStart (char* smallROMStart);
static UInt32	GetBigROMSize (char* bigROMStart);
static void		GetROMStartSize (char** romStart, Int32* romSize);

UInt32	gROMVersion;

// ---------------------------------------------------------------------------
//		 PilotMain
// ---------------------------------------------------------------------------
// The main entry point for ROM Transfer.

UInt32 PilotMain (	UInt16		iCommand,
					void*		iCommandParams,
					UInt16		iLaunchFlags)
{
	#pragma unused (iCommandParams)
	#pragma unused (iLaunchFlags)

	if (iCommand != sysAppLaunchCmdNormalLaunch)
		return 0;

	ErrTry
	{
		InitializeApplication ();
		ExecuteApplication ();
	}

	// Catch any unhandled exceptions.

	ErrCatch (iError)
	{
	}
	ErrEndCatch

	// Clean up.

	DisposeApplication ();

	return 0;
}


#pragma mark -


// ---------------------------------------------------------------------------
//		 InitializeApplication
// ---------------------------------------------------------------------------
// Initializes the application globals and prepares to execute the event loop.

void InitializeApplication (void)
{
	Char*			aLabel;
	ListPtr			aList;
	UInt16			aListItem;
	ControlPtr		aControl;
	FormPtr			aMainForm;

	FtrGet (sysFtrCreator, sysFtrNumROMVersion, &gROMVersion);

	// Initialize and draw the main form.

	aMainForm = FrmInitForm (MainForm);
	Throw_IfNil (aMainForm, 0);

	FrmSetActiveForm (aMainForm);
	FrmDrawForm	(aMainForm);

	// Set up the speed popup.

	aListItem = kItem_115200;
	aList = FrmGetObjectPtr (aMainForm, FrmGetObjectIndex (aMainForm, MainSpeedList));
	aLabel = LstGetSelectionText (aList, aListItem);
	aControl = FrmGetObjectPtr (aMainForm, FrmGetObjectIndex (aMainForm, MainSpeedPopTrigger));
	CtlSetLabel (aControl, aLabel);
	LstSetSelection (aList, aListItem);
}


// ---------------------------------------------------------------------------
//		 DisposeApplication
// ---------------------------------------------------------------------------
// Disposes the application globals and prepares to exit.

void DisposeApplication (void)
{
}


// ---------------------------------------------------------------------------
//		 ExecuteApplication
// ---------------------------------------------------------------------------
// Executes an event loop until the application quits.

void ExecuteApplication (void)
{
	EventType aEvent;

	aEvent.eType = nilEvent;

	// Repeat until the application quits.

	while (aEvent.eType != appStopEvent)
	{
		// Get the next available event.

		EvtGetEvent (&aEvent, evtWaitForever);

		// Give the system a chance to handle the event.

		if (!SysHandleEvent (&aEvent))

			// Try to handle the event ourselves.

			if (!ProcessEvent (&aEvent))

				// Let the form provide default handling of the event.

				FrmHandleEvent (FrmGetActiveForm (), &aEvent);
	}
}


// ---------------------------------------------------------------------------
//		 ProcessEvent
// ---------------------------------------------------------------------------
// Attempts to process the specified event.

/*
static void Display (FormPtr form, UInt32 value)
{
	UInt32	start = TimGetSeconds ();
	char	buffer[20];
	StrIToH (buffer, value);
	FrmCopyLabel (form, MainStatusTextLabel, buffer);

	while (TimGetSeconds() - start < 5)
		;
}
*/

Boolean ProcessEvent (EventPtr iEvent)
{
	Boolean		aWasHandled = false;

	if (iEvent->eType == ctlSelectEvent)
	{
		switch (iEvent->data.ctlEnter.controlID)
		{
			case MainBeginTransferButton:
			{
/*
				void*	smallROMStart	= GetSmallROMStart ();
				UInt32	smallROMSize	= GetSmallROMSize (smallROMStart);
				void*	bigROMStart		= GetBigROMStart (smallROMStart);
				UInt32	bigROMSize		= GetBigROMSize (bigROMStart);

				char*	romStart;
				Int32	romSize;
				GetROMStartSize (&romStart, &romSize);

				{
					FormPtr	form = FrmGetActiveForm();
					Display (form, (UInt32) smallROMStart);
					Display (form, (UInt32) smallROMSize);
					Display (form, (UInt32) bigROMStart);
					Display (form, (UInt32) bigROMSize);
					Display (form, (UInt32) romStart);
					Display (form, (UInt32) romSize);
				}
*/
				TransferROM ();

		   		aWasHandled = true;
		   		break;
		   	}
		}
	}

	return aWasHandled;
}


// ---------------------------------------------------------------------------
//		 TransferROM
// ---------------------------------------------------------------------------
// Downloads the ROM via the serial connection.

static Err MySerReceive(UInt16 refNum, void* bufP, UInt32 count, UInt32 timeout)
{
	Err	err;

	if (sysGetROMVerMajor(gROMVersion) < 2)
		err = SerReceive10 (refNum, bufP, count, timeout);
	else
		(void) SerReceive (refNum, bufP, count, timeout, &err);

	return err;
}

static Err MySerSend(UInt16 refNum, void* bufP, UInt32 count)
{	
	Err	err;

	if (sysGetROMVerMajor(gROMVersion) < 2)
		err = SerSend10 (refNum, bufP, count);
	else
		(void) SerSend (refNum, bufP, count, &err);

	return err;
}

static UInt32 MySysTicksPerSecond ()
{
	if (sysGetROMVerMajor(gROMVersion) < 2)
		return sysTicksPerSecond;

	return SysTicksPerSecond ();
}

void TransferROM (void)
{
	Int16	err;
	Int32	offset;
	char*	romStart = (char*) -1;
	Int32	romSize;

	Int16	speedChoice;
	UInt32	speed;
	FormPtr form;

	UInt8	c;
	UInt8	block;
	UInt32	i;
	Boolean	bXmodemCrc;
	UInt8	header[kXmodemBlockSize];
	Boolean bXmodemHeaderBlock;
	
	form = FrmGetActiveForm ();

	ErrTry
	{
		Throw_IfNil (form, 0);

		speedChoice = LstGetSelection (FrmGetObjectPtr (form, FrmGetObjectIndex (form, MainSpeedList)));
		switch (speedChoice)
		{
			case kItem_1200: speed = kBaud_1200; break;
			case kItem_2400: speed = kBaud_2400; break;
			case kItem_4800: speed = kBaud_4800; break;
			case kItem_9600: speed = kBaud_9600; break;
			case kItem_14400: speed = kBaud_14400; break;
			case kItem_19200: speed = kBaud_19200; break;
			case kItem_28800: speed = kBaud_28800; break;
			case kItem_38400: speed = kBaud_38400; break;
			case kItem_57600: speed = kBaud_57600; break;
			case kItem_115200: speed = kBaud_115200; break;
			default: speed = kBaud_14400; break;
		}

		err = Comm_Initialize (kPort_Pilot, speed);
		Throw_IfError (err, err);

		// Get the size and location of the ROM.

		GetROMStartSize (&romStart, &romSize);

		// Change the status text.

		FrmCopyLabel (form, MainStatusTextLabel, "Waiting...          ");

		// First, wait for the receiver to send us a nak. If it's
		// a real nak, then we'll send in checksum mode. If it's
		// a 'C', we'll send in CRC mode.

		Comm_ClearError();
		bXmodemCrc = false;
		while (true)
		{
			err = MySerReceive(gPortReference, &c, 1, 15 * MySysTicksPerSecond ());
			Throw_IfError(err, err);

			if (c == kXmodemNakCrc)
			{
				bXmodemCrc = true;
				break;
			}
			else if (c == kXmodemNak)
			{
				break;
			}
			else if (c == kXmodemCan)
			{
				ErrThrow(c);
			}
		}

		// Build the Ymodem header block.

		MemSet(header, kXmodemBlockSize, 0);
		StrCopy((char*) header, "PalmOS.ROM"); // What's in a name?
		StrIToA((char*) &header[StrLen((char*) header)+1], romSize);

		// Send the ROM one block at a time until it's done.

		offset	= 0;
		block	= 0;
		bXmodemHeaderBlock = true;

		while (offset < romSize)
		{
			UInt8 *blockStart;

			EvtResetAutoOffTimer ();

			// Change the status text.
			
			{
				UInt32	percentComplete = (offset * 100) / (romSize - kXmodemBlockSize);
				char	buffer1[100];
				char	buffer2[20];

				StrCopy (buffer1, "Sending...");
				StrIToA (buffer2, percentComplete);
				StrCat (buffer1, buffer2);
				StrCat (buffer1, "%          ");

				FrmCopyLabel (form, MainStatusTextLabel, buffer1);
			}

			// Send each part of the block individually.

			c = kXmodemSoh;
			err = MySerSend(gPortReference, &c, 1);
			err = MySerSend(gPortReference, &block, 1);
			c = block ^ 0xFF;
			err = MySerSend(gPortReference, &c, 1);

			if (bXmodemHeaderBlock)
			{
				blockStart = header;
			}
			else
			{
				blockStart = (UInt8*) (romStart + offset);
			}
			err = MySerSend (gPortReference, blockStart, kXmodemBlockSize);

			if (bXmodemCrc)
			{
				UInt16 crc = Crc16CalcBlock (blockStart, kXmodemBlockSize, 0);
				
				UInt8	zeros[2] = {0, 0};
				crc = Crc16CalcBlock (zeros, 2, crc);

				c = crc >> 8;
				err = MySerSend (gPortReference, &c, 1);
				c = crc;
				err = MySerSend (gPortReference, &c, 1);
			}
			else
			{
				c = 0;
				for (i = 0; i < kXmodemBlockSize; i++)
				{
					c += blockStart[i];
				}
				err = MySerSend (gPortReference, &c, 1);
			}
			Throw_IfError (err, err);

			// Wait for the other end to ack or nak us. Leave
			// plenty of timeout so the receiver has time to
			// decide whether it got a partial block.

			err = MySerReceive (gPortReference, &c, 1, 20 * MySysTicksPerSecond ());
			Throw_IfError (err, err);
			if (c == kXmodemNak || c == kXmodemNakCrc)
			{
				continue;
			}
			else if (c != kXmodemAck)
			{
				ErrThrow(c);
			}

			if (!bXmodemHeaderBlock)
			{
				// Only do this if we're into sending the file
				// (don't increment on the file header block).

				offset += kXmodemBlockSize;
			}
			block++;
			bXmodemHeaderBlock = false;
		}

		// Send an EOF and wait for an acknowledgement.

		while (true)
		{
			c = kXmodemEof;
			err = MySerSend(gPortReference, &c, 1);
			err = MySerReceive(gPortReference, &c, 1, 20 * MySysTicksPerSecond ());
			Throw_IfError (err, err);
			if (c == kXmodemNak)
			{
				continue;
			}
			else if (c == kXmodemAck)
			{
				break;
			}
			else
			{
				ErrThrow(c);
			}
		}

		// Send another one for good measure.

		c = kXmodemEof;
		err = MySerSend(gPortReference, &c, 1);

		// Change the status text.

		FrmCopyLabel (form, MainStatusTextLabel, "Finished               ");
	}

	// Catch any exceptions.

	ErrCatch (iError)
	{
		// Change the status text.

		if (form != NULL)
		{
			char	buffer1[100];
			char	buffer2[20];

			StrCopy (buffer1, "Error #");
			StrIToA (buffer2, iError);
			StrCat (buffer1, buffer2);
			StrCat (buffer1, "           ");

			FrmCopyLabel (form, MainStatusTextLabel, buffer1);
		}
	}
	ErrEndCatch

	Comm_Dispose ();
}


// #define NON_PORTABLE
// #include <MemoryPrvNew.h>	// CardHeaderType, StorageHeaderType

// Flags Field
#define memHeapFlagReadOnly	0x0001		// heap is read-only (ROM based)
#define memHeapFlagVers2		0x8000		// version 2 heap (> 64K)
#define memHeapFlagVers3		0x4000		// version 3 heap (has free list)
#define memHeapFlagVers4		0x2000		// version 4 heap (has free master pointer table entry list)

/************************************************************
 * This structure defines a section within the RAM storage header
 *  on Card#0 used to hold non-volatile System information. We store
 *  System information that can not be conveniently stored or accessed
 *  from a Database in this area because:
 *	1.) it can be accessed earlier during the boot-up process and 
 * 2.) It can be accessed from an interrupt routine.
 *************************************************************/ 
typedef struct SysNVParamsType {
	UInt32	rtcHours;									// Real-Time clock hours - add to value
																// in DragonBall RTC register to get
																// actual date & time.
	UInt32	rtcHourMinSecCopy;						// Copy of latest value in rtcHourMinSec reg of 
																// DBall. Used as default RTC value on Reset.
	UInt8		swrLCDContrastValue;						// Contrast Value for LCD on EZ-based products
																// that use the software contrast PWM (such as Sumo)
	UInt8 	swrLCDBrightnessValue;					// Brightness value for screens with adjustable brightness.
	
	// Note that in the ROM store, these next four fields contain the default
	// settings for card 0's RAM store, when it has to be initialized.
	void*		splashScreenPtr;							// MemPtr to splash screen bitmap
	void*		hardResetScreenPtr;						// MemPtr to hard reset screen bitmap.
	UInt16	localeLanguage;							// Language for locale.
	UInt16	localeCountry;								// Country for locale.
	
	// 11/15/99 SCL: New Globals added for Licensees, Partners, OEMs, etc.
	// These storage locations are reserved for the HAL running on a given device.
	UInt32	sysNVOEMStorage1;							// 4 bytes for Device OEM use only!
	UInt32	sysNVOEMStorage2;							// 4 bytes for Device OEM use only!

	} SysNVParamsType;
typedef SysNVParamsType *SysNVParamsPtr;


/************************************************************
 * Structure of a Card Header.
 * There is 1 card header for in every card that has ROM. The
 *  card header is stored at sysCardHeaderOffset into the card.
 *
 * RAM only cards will not have a card header
 *************************************************************/
#define	memMaxNameLen				32		// size of name and manuf fields including null
typedef struct CardHeaderType {
	UInt32		initStack;					// initial stack pointer (v4: offset to code in RomBoot)
	UInt32		resetVector;				// reset vector
	UInt32		signature;					// must be sysCardSignature
	UInt16		hdrVersion;					// header version
	UInt16		flags;						// card flags;
	UInt8			name[memMaxNameLen];		// card name
	UInt8			manuf[memMaxNameLen];	// card manufacturer's name
	UInt16		version;						// card version
	UInt32		creationDate;				// card creation date
	UInt16		numRAMBlocks;				// number of RAM blocks on card
	UInt32		blockListOffset;			// offset to RAM block list
	UInt32		readWriteParmsOffset;	// v2: offset from CardBase to r/w system data if any (in ROM)
	UInt32		readWriteParmsSize;		// v2: size of read/write system data if any (in ROM)
	UInt32		readOnlyParmsOffset;		// v2: offset from CardBase to read-only system data (in ROM)
	UInt32		bigROMOffset;				// v2: in SmallROM header: where SmallROM expects bigROM to live
													//			in BigROM header: where BigROM expects itself to live
	UInt32		checksumBytes;				// v2: size of card image in bytes (for checksum)
	UInt16		checksumValue;				// v2: checksum of card image (from Crc16CalcBlock)

	UInt32		readWriteWorkingOffset;	// v3: offset from CardBase to r/w working area if any (in ROM)
	UInt32		readWriteWorkingSize;	// v3: size of read/write working area if any (in ROM)

	UInt32		halCodeOffset;				// v4: offset from CardHeader to HAL code
	
	UInt8			reserved[130];				// to bring us to 0x100 alignment
	} CardHeaderType;
typedef CardHeaderType*		CardHeaderPtr;

/************************************************************
 * Structure of a Storage Header. 
 * There is 1 of these for every "store" on a memory card. A
 *  "store" can be all the RAM on a card or all the ROM on a card.
 *
 * The RAM storage header is stored at sysRAMHeader offset into the
 *	 card. and the ROM storage header is stored at sysROMHeader offset
 *  into the card.
 *************************************************************/
typedef struct {
	UInt32		signature;					// must be sysStoreSignature
	UInt16		version;						// version of header
	UInt16		flags;						// flags
	UInt8			name[memMaxNameLen];		// name of store
	UInt32		creationDate;				// creation date
	UInt32		backupDate;					// last backup date
	UInt32		heapListOffset;			// offset to heap list for store
	UInt32		initCodeOffset1;			// init code for store, if any
	UInt32		initCodeOffset2;			// second init code for store, if any
	LocalID		databaseDirID;				// local ID of database dir.
	UInt32		rsvSpace;					// where first heap starts.
	UInt32		dynHeapSpace;				// how big the dynamic heap area is
													//  (always 0 for ROM stores)
	UInt32		firstRAMBlockSize;		// Copy of firstRAMBlock size from cardinfo
													// Used to determine if we're rebooting
													// with a different amount of RAM.
													
	// The following fields are used to store non-volatile information that
	//  must be accessed by the system but that is not convenient to store
	//  in a database due to access time or frequency. It is only valid
	//  in the RAM STORE ON CARD #0! Though some initialization values for
	//	 the card 0 RAM store come from the ROM store's nvParams field.
	SysNVParamsType	nvParams;
	
	// Filler bytes - reserved for future use. Size adjusted to
	// keep total size of storage header at 0x100 bytes.
	UInt8			reserved[176-sizeof(SysNVParamsType)];		

	// CRC value
	UInt32		crc;							// crc to check validity	
	} StorageHeaderType;
typedef StorageHeaderType*		StorageHeaderPtr;


#ifdef __cplusplus
extern "C" {
#endif

void *			MemHeapPtr(UInt16 heapID)
							SYS_TRAP(sysTrapMemHeapPtr);
	
#ifdef __cplusplus 
}
#endif

// ---------------------------------------------------------------------------
//		 GetCardHeaderVersion
// ---------------------------------------------------------------------------

static UInt16 GetCardHeaderVersion (void* smallROMStart)
{
	return ((CardHeaderType*) smallROMStart)->hdrVersion;
}


// ---------------------------------------------------------------------------
//		 HandspringHack
// ---------------------------------------------------------------------------

static Boolean HandspringHack (UInt32 romSize, char* romStart)
{
	CardHeaderType*	card = (CardHeaderType*) romStart;

	if (romSize != 0x1000)
		return false;

	if (StrCompare ((char*) card->manuf, "Handspring, Inc.") != 0)
		return false;

	// small ROM - masked ROM build
	if (card->resetVector == 0x10C032F2 && card->checksumValue == 0xFB2C)
		return true;

	// big ROM - masked ROM build
	if (card->resetVector == 0x10C6B6C8 && card->checksumValue == 0x175C)
		return true;

	// small ROM - OTP ROM build
	if (card->resetVector == 0x10C032F2 && card->checksumValue == 0x2A87)
		return true;

	// big ROM - OTP ROM build
	if (card->resetVector == 0x10C6B6C8 && card->checksumValue == 0xA843)
		return true;

	// small ROM - Visor rev b
	if (card->resetVector == 0x10C032F2 && card->checksumValue == 0xC8B7)
		return true;

	// big ROM - Visor rev b
	if (card->resetVector == 0x10C6B6E2 && card->checksumValue == 0x831D)
		return true;

	// small ROM - Japanese
	if (card->resetVector == 0x100032F2 && card->checksumValue == 0x7F66)
		return true;

	// big ROM - Japanese
	if (card->resetVector == 0x100C2C06 && card->checksumValue == 0x497A)
		return true;

	return false;
}


// ---------------------------------------------------------------------------
//		 GetSmallROMStart
// ---------------------------------------------------------------------------

#define hwr68328Base			0xFFFFF000L		// Base address of 68328 
#define kChipSelectAOffset		0x0100			// Chip select A register.


static char* GetSmallROMStart1 (void)
{
	char*	smallROMStart;

	UInt32	processor;
	Err		err = FtrGet (sysFtrCreator, sysFtrNumProcessorID, &processor);

	if ((err == 0) &&
		((processor & sysFtrNumProcessorMask) == sysFtrNumProcessorEZ))
	{
		// Running on EZ--use chip select A to guess at the base.
		// Read the chip select A register, which has the base address of the "ROM".
		// in a compacted form (EZ man 4-4).
		UInt32	flashBaseCompact = (UInt32) *(UInt16*) (hwr68328Base + kChipSelectAOffset);
		smallROMStart = (char*) ((flashBaseCompact & 0x0FFFE) << 13);
	}
	else
	{
		// Try the old standard.
		smallROMStart = (char*) 0x10C00000;
	}

	return smallROMStart;
}

char* GetSmallROMStart (void)
{
	char*	smallROMStart1 = GetSmallROMStart1 ();

	return smallROMStart1;
}


// ---------------------------------------------------------------------------
//		 GetSmallROMSize
// ---------------------------------------------------------------------------

UInt32 GetSmallROMSize (char* smallROMStart)
{
	// For now, it's OK if this function is pretty gross.  It turns
	// out that we don't need it and don't call it.

	CardHeaderType*	card = (CardHeaderType*) smallROMStart;
	UInt32	smallROMSize = card->checksumBytes;

	// Special hack for patched Handspring header?  checksumBytes
	// contains 4K when it should really be 16K or 32K.

	if (HandspringHack (smallROMSize, smallROMStart))
	{
		// "For the small ROM size, you can basically take the
		//  bigROMOffset field of the card header and subtract
		//  the start of smallROM from that to deduce the size."
		//			-- ron

		// Keith sez: Make sure we account for the fact that
		// "bigROMOffset" is an offset from card 0 base.  Call
		// MemLocalIDToPtr to convert it to an address.

		char* bigROMStart = GetBigROMStart (smallROMStart);
		smallROMSize = bigROMStart - smallROMStart;
	}

	return smallROMSize;
}


// ---------------------------------------------------------------------------
//		 GetBigROMStart
// ---------------------------------------------------------------------------

char* GetBigROMStart (char* smallROMStart)
{
	CardHeaderType*	card		= (CardHeaderType*) smallROMStart;
	char*			bigROMStart	= MemLocalIDToPtr (card->bigROMOffset, 0);

	return bigROMStart;
}


// ---------------------------------------------------------------------------
//		 GetBigROMSize
// ---------------------------------------------------------------------------

static UInt32 GetBigROMSize1 (char* bigROMStart)
{
	UInt32	bigROMSize;
	Err		err = MemCardInfo (
				0,				// UInt16 cardNo
				NULL,			// Char * nameP
				NULL,			// Char * manufNameP
				NULL,			// UInt16 * versionP
				NULL,			// UInt32 * crDateP
				&bigROMSize,	// UInt32 * romSizeP
				NULL,			// UInt32 * ramSizeP
				NULL);			// UInt32 * freeBytesP

	return bigROMSize;
}

static UInt32 GetBigROMSize2 (char* bigROMStart)
{
	CardHeaderType*	card = (CardHeaderType*) bigROMStart;
	UInt32	bigROMSize = card->checksumBytes;

	return bigROMSize;
}

static UInt32 GetBigROMSize3 (char* bigROMStart)
{
	// "For the big ROM size, you can deduce it from the size of the ROM
	//  heap.  BigROMBase+0x20A always contains the start address of the
	//  ROM heap header.  The start of the ROM heap has a MemHeapHeaderType
	//  structure that contains a field which is the heap size so you can
	//  deduce the size of the ROM by adding the heap size to the start of
	//  the heap."
	//			-- ron

	// Keith sez: Let's use the API instead of "BigROMBase+0x20A".

	char*	bigROMEnd = NULL;

	// Iterate over all the heaps, looking for one that's read-only.
	// I'll assume that that's the ROM heap.

	UInt16	ii;
	UInt16	numHeaps = MemNumHeaps (0);
	for (ii = 0; ii < numHeaps; ++ii)
	{
		UInt16	heapID = MemHeapID (0, ii);
		UInt16	flags = MemHeapFlags (heapID);

		if ((flags & memHeapFlagReadOnly) != 0)
		{
			char*	romHeapStart = MemHeapPtr (heapID);
			UInt32	romHeapSize = MemHeapSize (heapID);
			bigROMEnd = romHeapStart + romHeapSize;
			break;
		}
	}

	return bigROMEnd - bigROMStart;
}

UInt32 GetBigROMSize (char* bigROMStart)
{
	// This technique appears to return a value 4 bytes too small.

	UInt32	bigROMSize1 = GetBigROMSize1 (bigROMStart);

	// The preferred method.  Based on checksumBytes in the card
	// header.  Since we validate the checksum, we want to make
	// sure we get all the bytes going into the checksum.

	UInt32	bigROMSize2 = GetBigROMSize2 (bigROMStart);

	// Backup method.  Based on size of ROM heap.  I'm a little
	// leary of using it all the time, however, since I don't know
	// if "checksumBytes" includes any padding bytes past the end
	// of the heap.

	UInt32	bigROMSize3 = GetBigROMSize3 (bigROMStart);

	UInt32	bigROMSize = bigROMSize2;

	// Special hack for patched Handspring header?  checksumBytes
	// contains 4K when it should really be bigger.

	if (HandspringHack (bigROMSize, bigROMStart))
	{
		bigROMSize = bigROMSize3;
	}

	return bigROMSize;
}


// ---------------------------------------------------------------------------
//		 GetROMStartSize
// ---------------------------------------------------------------------------

void GetROMStartSize (char** romStart, Int32* romSize)
{
	// Get a pointer to the beginning of ROM, which is formatted
	// as a CardHeader struct.

	char*	smallROMStart	= GetSmallROMStart ();
//	UInt32	smallROMSize;
	char*	bigROMStart;
	UInt32	bigROMSize;

	// If we have a version 1 card header, we don't have the
	// information we need to get the ROM size.  So fall back
	// on the original, well-known values.

	if (GetCardHeaderVersion (smallROMStart) < 2)
	{
		// Determine which ROM we're transferring.

		if (sysGetROMVerMajor(gROMVersion) == 1)
		{
			*romStart = (char*) 0x10C00000;
			*romSize = 512 * 1024L;
			return;
		}

		if (sysGetROMVerMajor(gROMVersion) == 2)
		{
			*romStart = (char*) 0x10C00000;
			*romSize = 1024 * 1024L;
			return;
		}
	}

//	smallROMSize	= GetSmallROMSize (smallROMStart);
	bigROMStart		= GetBigROMStart (smallROMStart);
	bigROMSize		= GetBigROMSize (bigROMStart);

	*romStart = smallROMStart;
	*romSize = bigROMStart + bigROMSize - smallROMStart;
}

