/* -*- mode: C++; tab-width: 4 -*- */
/* ===================================================================== *\
	Copyright (c) 1998-2000 Palm, Inc. or its subsidiaries.
	All rights reserved.

	This file is part of the Palm OS Emulator.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
\* ===================================================================== */

#define PILOT_PRECOMPILED_HEADERS_OFF
#include "Serial Port Tools.h"
#include <SysEvtMgr.h>
#include <SerialMgrOld.h>

// for Handspring API access
#define	sysTrapHsSelector				sysTrapOEMDispatch
#define hsSelExtKeyboardEnable			5
#define SYS_SEL_TRAP(trapNum, selector) \
	_SYSTEM_API(_CALL_WITH_16BIT_SELECTOR)(_SYSTEM_TABLE, trapNum, selector)

Err			HsExtKeyboardEnable (Boolean enable)
				SYS_SEL_TRAP (sysTrapHsSelector, hsSelExtKeyboardEnable);


// Globals.

UInt16		gPortReference		= 0;
Boolean		gCommEstablished	= false;


// ---------------------------------------------------------------------------
//		 Comm_Initialize
// ---------------------------------------------------------------------------
// Opens the communications port.

Int16 Comm_Initialize (Int16 iPort, UInt32 iBaud)
{
	Int16				aError;
	SerSettingsType		aSettings;
	UInt32 				ftrHSVersion;

	// Exit if communications have already been established.

	if (gCommEstablished)
		return (kError_CommOpen);

	// Clear the globals.

	gPortReference		= 0;
	gCommEstablished	= false;

	// <chg 02-01-2000 BP>  If on Handspring device, disable the keyboard 
	// thread before opening the serial library.  
	if (!FtrGet ('hsEx', 0, &ftrHSVersion))
		HsExtKeyboardEnable (false);

	// Find the serial library.

	Comm_ClearError ();

	aError = SysLibFind ("Serial Library", &gPortReference);
	if (aError != 0)
		return (kError_CantOpenPort);

	// Open the serial port.

	if (iPort != kPort_Pilot)
		return (kError_InvalidPort);

	Comm_ClearError ();

	aError = SerOpen (gPortReference, iPort, iBaud);

	if (aError == serErrAlreadyOpen)
	{
		SerClose (gPortReference);
		return (kError_PortOpen);
	}
	else if (aError != 0)
		return (kError_CantOpenPort);

	// Set the serial attributes.

	Comm_ClearError ();

	aSettings.baudRate		= iBaud;
	aSettings.flags			= serSettingsFlagBitsPerChar8 | serSettingsFlagStopBits1;		
	aSettings.ctsTimeout	= serDefaultCTSTimeout;

	aError = SerSetSettings (gPortReference, &aSettings);

	if (aError != 0)
	{
		SerClose (gPortReference);
		return (kError_CantOpenPort);
	}

	gCommEstablished = true;

	return (0);
}


// ---------------------------------------------------------------------------
//		 Comm_Dispose
// ---------------------------------------------------------------------------
// Closes the communications port.

void Comm_Dispose (void)
{
	// Exit if communications have not been established.

	if (!gCommEstablished)
		return;

	// Close the serial port.

	Comm_ClearError ();

	SerClose (gPortReference);

	gPortReference = 0;

	// Clear the globals.

	gCommEstablished = false;
}


// ---------------------------------------------------------------------------
//		 Comm_ClearError
// ---------------------------------------------------------------------------
// Clears the current serial errors.

void Comm_ClearError (void)
{
	// Exit if communications have not been established.

	if (!gCommEstablished)
		return;

	SerClearErr (gPortReference);
}
