# -*- mode: Perl; tab-width: 4 -*-

$started = 0;
$catenating = 0;
$id = "";
$text = "";
$language = "ENGLISH";

$rez_resource = 0;
$vcpp_resource = 0;
$cpp_source = 0;
$translate_high_ascii = 0;

$format = $ARGV[0];

if ($format eq "-format=rez_resource")
{
	$rez_resource = 1;
}

if ($format eq "-format=vcpp_resource")
{
	$vcpp_resource = 1;
	$translate_high_ascii = 1
}

if ($format eq "-format=cpp_source")
{
	$cpp_source = 1;
	$translate_high_ascii = 2
}


sub escape_quotes
{	
	if ($rez_resource or $cpp_source)
	{
		$text =~ s/\"/\\\"/g;		# Turn a " into \"
	}

	if ($vcpp_resource)
	{
		$text =~ s/\"/\"\"/g;		# Turn a " into ""
	}
}

sub translate_high_ascii
{
	if ($translate_high_ascii)
	{
		# Translate Macintosh High Ascii characters
		# into the equivalents for Windows and Unix.

		# (Note: I tried turning the curly quotes into
		#  the equivalent on Windows, but they just looked
		#  really bad in dialogs, so I now turn them into
		#  straight quotes.)

		$text =~ s//\"/g;			# Turn  into "
		$text =~ s//\"/g;			# Turn  into "
		$text =~ s//\'/g;			# Turn  into '
		$text =~ s//\'/g;			# Turn  into '
		$text =~ s//\.\.\./g;		# Turn  into ...
		$text =~ s//\(TM)/g;		# Turn  into (TM)

		if ($translate_high_ascii == 1)
		{
			$text =~ s//\/g;			# Turn  into 
		}
		else
		{
			$text =~ s//(R)/g;			# Turn  into (R)
			$text =~ s//(c)/g;			# Turn  into (c)
		}
	}
}

sub dump_if_catenating
{
	if ($catenating eq 1)
	{
		translate_high_ascii ();
		escape_quotes ();

		if ($rez_resource)
		{
			print "\t$id, \"$text\";";
		}

		if ($vcpp_resource)
		{
			print "\t$id\t\"$text\"";
		}

		if ($cpp_source)
		{
			print "\t{ $id, \"$text\" },";
		}

		$catenating = 0;
	}
}


while (chomp ($line = <STDIN>) )
{
	if ($line eq "START")
	{
		$started = 1;

		if ($rez_resource)
		{
			printf "type \'STRA\' {\n";
			printf "	integer = \$\$CountOf(SomeArray);\n";
			printf "	wide array SomeArray {\n";
			printf "		integer;\n";
			printf "		wstring;\n";
			printf "	};\n";
			printf "};\n";
			printf "\n";
			printf "resource \'STRA\' (1000) {{\n";
		}

		if ($vcpp_resource)
		{
			print "STRINGTABLE DISCARDABLE\n";
			print "BEGIN\n";
		}

		if ($cpp_source)
		{
			print <<EOF;
#include <map>

static map<int, const char*> _ResStrMap;

struct item
{
	int key;
	const char* value;
};

static const struct item _ResStrTable[] = {
EOF
		}
	}
	elsif ($line eq "END")
	{
		$started = 0;

		if ($rez_resource)
		{
			print "}};\n\n";
		}

		if ($vcpp_resource)
		{
			print "END\n\n";
		}

		if ($cpp_source)
		{
			print <<EOF;
	{ 0, NULL }
	};

static void _ResStrTableInit (void)
{
	for (const struct item* it = _ResStrTable; it->value != NULL; it++)
	{
		_ResStrMap[it->key] = it->value;
	}
}


EOF
			print "const char* _ResGetString(int idx)\n";
			print "{\n";
			print "\tstatic int inited;\n";
			print "\tif (!inited)\n";
			print "\t{\n";
			print "\t\tinited = 1;\n";
			print "\t\t_ResStrTableInit();\n";
			print "\t}\n";
			print "\n";
			print "\treturn _ResStrMap[idx];\n";
			print "}\n";
		}
	}
	elsif ($started && $line =~ /^([A-Z]+)=(.+)/)
	{
		dump_if_catenating ();

		if ($1 eq "ID")
		{
			$id = $2;
		}
		elsif ($1 eq $language)
		{
			$text = $2;
			$catenating = 1;
		}
	}
	else
	{
		if ($catenating && substr ($line, 0, 1) eq "\t")
		{
			$text .= substr ($line, 1);
		}
		else
		{
			dump_if_catenating ();

			print "$line\n";
		}
	}
}
