/* -*- mode: C++; tab-width: 4 -*- */
/* ===================================================================== *\
	Copyright (c) 1998-2000 Palm, Inc. or its subsidiaries.
	All rights reserved.

	This file is part of the Palm OS Emulator.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
\* ===================================================================== */

#ifndef _BANK_ROM_H_
#define _BANK_ROM_H_

class EmStream;
class SessionFile;

const emuptr	kDefaultROMMemoryStart	= 0x10C00000;

class ROMBank
{
	public:
		static void				Initialize			(EmStream& hROM);
		static void				Reset				(void);
		static void				Save				(SessionFile&);
		static void				Load				(SessionFile&);
		static void				Dispose				(void);

		static void				SetBankHandlers		(void);

		static uint32			GetLong				(emuptr iAddress);
		static uint32			GetWord				(emuptr iAddress);
		static uint32			GetByte				(emuptr iAddress);
		static void				SetLong				(emuptr iAddress, uint32 iLongValue);
		static void				SetWord				(emuptr iAddress, uint32 iWordValue);
		static void				SetByte				(emuptr iAddress, uint32 iByteValue);
		static int				ValidAddress		(emuptr iAddress, uint32 iSize);
		static uint8*			GetRealAddress		(emuptr iAddress);
		static uint8*			GetMetaAddress		(emuptr iAddress);
		static void				AddOpcodeCycles		(void);

		static emuptr			GetMemoryStart		(void) { return gROMMemoryStart; }
		static int				IsPCInRAM			(void) { return m68k_getpc () < GetMemoryStart (); }
		static int				IsPCInROM			(void) { return m68k_getpc () >= GetMemoryStart (); }

	private:
		static void				InvalidAccess		(emuptr iAddress, long size, Bool forRead);
		static void				LoadROM				(EmStream& hROM);

		static emuptr			gROMMemoryStart;
};


class FlashBank
{
	public:
		static void				Initialize		(void);
		static void				Reset			(void);
		static void				Save			(SessionFile&);
		static void				Load			(SessionFile&);
		static void				Dispose			(void);

		static void				SetBankHandlers	(void);

		static uint32			GetLong			(emuptr iAddress);
		static uint32			GetWord			(emuptr iAddress);
		static uint32			GetByte			(emuptr iAddress);
		static void				SetLong			(emuptr iAddress, uint32 iLongValue);
		static void				SetWord			(emuptr iAddress, uint32 iWordValue);
		static void				SetByte			(emuptr iAddress, uint32 iByteValue);
		static int				ValidAddress	(emuptr iAddress, uint32 iSize);
		static uint8*			GetRealAddress	(emuptr iAddress);
		static uint8*			GetMetaAddress	(emuptr iAddress);
		static void				AddOpcodeCycles	(void);

		static emuptr			GetMemoryStart	(void) { return kDefaultROMMemoryStart; }
};


#endif /* _BANK_ROM_H_ */

