/* -*- mode: C++; tab-width: 4 -*- */
/* ===================================================================== *\
	Copyright (c) 1998-2000 Palm, Inc. or its subsidiaries.
	All rights reserved.

	This file is part of the Palm OS Emulator.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
\* ===================================================================== */

#ifndef Bank_Regs_h
#define Bank_Regs_h

class SessionFile;
struct RGBType;

class RegsBank
{
	public:
		static void				Initialize				(void);
		static void				Reset					(void);
		static void				Save					(SessionFile&);
		static void				Load					(SessionFile&);
		static void				Dispose					(void);

		static void				SetBankHandlers			(void);

		static uint32			GetLong					(emuptr address);
		static uint32			GetWord					(emuptr address);
		static uint32			GetByte					(emuptr address);
		static void				SetLong					(emuptr address, uint32 value);
		static void				SetWord					(emuptr address, uint32 value);
		static void				SetByte					(emuptr address, uint32 value);
		static int				ValidAddress			(emuptr address, uint32 size);
		static uint8*			GetRealAddress			(emuptr address);

		static void				Cycle					(Bool sleeping);
		static void				CycleSlowly				(Bool sleeping);

		static void				ButtonEvent				(Bool buttonIsDown, int32 button);
		static void				HotSyncEvent			(Bool buttonIsDown);

		static void				TurnSoundOff			(void);
		static void				ResetTimer				(void);
		static void				ResetRTC				(void);

		static int32			GetInterruptLevel		(void);
		static int32			GetInterruptBase		(void);
		static int32			GetLCDDepth				(void);
		static int32			GetLCDRowBytes			(void);
		static int32			GetLCDWidth				(void);
		static int32			GetLCDHeight			(void);
		static emuptr			GetLCDStartAddr			(void);
		static int32			GetLCDPalette			(RGBType*);
		static int32			GetLCDOffset			(void);
		static int32			GetDynamicHeapSize		(void);
		static int32			GetROMSize				(void);
		static emuptr			GetROMBaseAddress		(void);
		static Bool				ChipSelectsConfigured	(void);
		static int32			GetSystemClockFrequency	(void);
		static Bool				GetLCDOn				(void);
		static Bool				GetBacklightOn			(void);
		static Bool				GetSerialPortOn			(void);
		static Bool				GetCanStop				(void);

		static uint8			GetPortInputValue		(int);
		static uint8			GetPortInternalValue	(int);
		static void				PortDataChanged			(int, uint8, uint8);

	private:
		static void				InvalidAccess			(emuptr address, long size, Bool forRead);
		static void				PreventedAccess			(emuptr address, long size, Bool forRead);
};

#endif	/* Bank_Regs */
