/* -*- mode: C++; tab-width: 4 -*- */
/* ===================================================================== *\
	Copyright (c) 1998-2000 Palm, Inc. or its subsidiaries.
	All rights reserved.

	This file is part of the Palm OS Emulator.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
\* ===================================================================== */

#include "EmCommon.h"
#include "CGremlinsStubs.h"

#include "Platform.h"			// Platform::ViewDrawLine

// ---------------------------------------------------------------------------
//		 StubEnqueuePt
// ---------------------------------------------------------------------------
// This is a stub routine that called the application object's method that
// enqueues a point.

void	StubAppEnqueuePt(PointType* pen)
{
	::EvtEnqueuePenPoint(pen);
}


// ---------------------------------------------------------------------------
//		 StubAppEnqueueKey
// ---------------------------------------------------------------------------
// This is a stub routine that called the Application object's method that
// enqueues a key.

void StubAppEnqueueKey (UInt16 ascii, UInt16 keycode, UInt16 modifiers)
{
	switch (ascii)
	{
		// This translates some control characters into system chars
		case 0x01:	// control-A
			ascii = vchrMenu;
			modifiers = commandKeyMask;
			break;
		case 0x02:	// control-B
			ascii = vchrLowBattery;
			modifiers = commandKeyMask;
			break;			
		case 0x03:	// control-C
			ascii = vchrCommand;
			modifiers = commandKeyMask;
			break;
		case 0x04:	// control-D
			ascii = vchrConfirm;
			modifiers = commandKeyMask;
			break;
		case 0x05:	// control-E
			ascii = vchrLaunch;
			modifiers = commandKeyMask;
			break;
		case 0x06:	// control-F
			ascii = vchrKeyboard;
			modifiers = commandKeyMask;
			break;
		case 0x0D:	// control-M
			ascii = chrLineFeed;
			break;
		case 0x0E:	// control-N
			ascii = vchrNextField;
			modifiers = commandKeyMask;
			break;			
		case 0x13:	// control-S
			ascii = vchrAutoOff;
			modifiers = commandKeyMask;
			break;			
		case 0x14:	// control-T
			ascii = vchrHardContrast;
			modifiers = commandKeyMask;
			break;			
		case 0x15:	// control-U
			ascii = vchrBacklight;
			modifiers = commandKeyMask;
			break;
	}


#if 0	// Turning this bit off for now.  It never worked because (a) the values
		// assigned to "status" were incorrect (they should be correct now) and
		// (b) they only worked for non-EZ devices.  EZ devices kind of ignore
		// the "status" parameter and probe the keyboard settings directly,
		// something we don't handle at this point of control.  So I'm turning
		// this off and just posting the ascii keys.  Since the old method
		// never worked, then there's no issue with the Key Manager no longer
		// doing the right thing.

	// If this is one of the hard keys, send it through the Key Manager so that
	//  it can do the right things concering double-taps, etc.
	if ((modifiers & commandKeyMask) &&
			(ascii > hardKeyMin || (ascii == pageUpChr) || (ascii == pageDownChr)))
	{

		DWord		status = 0;

		switch (ascii)
		{
			case hardPowerChr: 		status = hwr328IntLoInt0;		break;
			case pageUpChr: 		status = hwr328IntLoInt1;		break;
			case pageDownChr: 		status = hwr328IntLoInt2;		break;
			case hard1Chr: 			status = hwr328IntLoInt3;		break;
			case hard2Chr: 			status = hwr328IntLoInt4;		break;
			case hard3Chr: 			status = hwr328IntLoInt5;		break;
			case hard4Chr: 			status = hwr328IntLoInt6;		break;
			case hardCradleChr: 	status = hwr328IntHiIRQ1 << 16;	break;
			case hardAntennaChr: 	status = hwr328IntHiIRQ2 << 16;	break;
		}

		// I'm dubious that this is the correct thing to do in the emulator.
		// This is an interrupt routine that we're calling directly.  But
		// what if a "real" interrupt is triggered from some other source
		// while this call is executing (for instance, the user clicks on
		// one of the "keyboard buttons", thus generating a keyboard
		// interrupt)?  It's possible that the two will stomp on each other.
		// The best thing to do is to post events the emulator way.
		// However, I'm hard-pressed to do that.  That would require calling
		// Hardware_ButtonEvent twice (once to signal the keyboard
		// interrupt, and another to clear it), but I don't know how to give
		// the processor time in between to respond to the first call.

		if (status != 0)
			::KeyHandleInterrupt (false, status);
	}

	// For other keys, enqueue them directly.
	else
#endif
	{
		::EvtEnqueueKey (ascii, keycode, modifiers);
	}
}


// ---------------------------------------------------------------------------
//		 StubAppGremlinsOn
// ---------------------------------------------------------------------------
// Stub routine that update the Gremlins menu an the global variable the
// keeps track of the ????

void StubAppGremlinsOn (void)
{
	// Called from Gremlins::Initialize.
}


// ---------------------------------------------------------------------------
//		 StubAppGremlinsOff
// ---------------------------------------------------------------------------
// Stub routine that update the Gremlins menu an the global variable the
// keeps track of the ????

void StubAppGremlinsOff (void)
{
	// Called by Gremlins when counter > until.
}


// ---------------------------------------------------------------------------
//		 StubViewDrawLine
// ---------------------------------------------------------------------------
// This is a stub routine that called the View object's method that draw
// a line.

void StubViewDrawLine (int xStart, int yStart, int xEnd, int yEnd)
{
	Platform::ViewDrawLine (xStart, yStart, xEnd, yEnd);
}


// ---------------------------------------------------------------------------
//		 StubViewDrawPixel
// ---------------------------------------------------------------------------
// This is a stub routine that called the View object's method that draw
// a pixel.

void StubViewDrawPixel (int xPos, int yPos)
{
	Platform::ViewDrawPixel (xPos, yPos);
}


