/* -*- mode: C++; tab-width: 4 -*- */
/* ===================================================================== *\
	Copyright (c) 1998-2000 Palm, Inc. or its subsidiaries.
	All rights reserved.

	This file is part of the Palm OS Emulator.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
\* ===================================================================== */

#include "EmCommon.h"
#include "EmDevice.h"

#include "PalmPack.h"
#define NON_PORTABLE
	#include "HwrMiscFlags.h"		// hwrMiscFlagIDTouchdown, etc.

	#ifndef hwrMiscFlagIDPalmIIIc
	#define	hwrMiscFlagIDPalmIIIc			(hwrMiscFlagID4 | hwrMiscFlagID1)
	#endif


#undef NON_PORTABLE
#include "PalmPackPop.h"

#include "EmRegs328Pilot.h"
#include "EmRegs328PalmPilot.h"
#include "EmRegs328PalmIII.h"
#include "EmRegs328PalmVII.h"
#include "EmRegs328Symbol1700.h"

#include "EmRegsEZPalmIIIc.h"
#include "EmRegsEZPalmIIIe.h"
#include "EmRegsEZPalmIIIx.h"
#include "EmRegsEZPalmV.h"
#include "EmRegsEZPalmVx.h"
#include "EmRegsEZPalmVII.h"
#include "EmRegsEZVisor.h"
#include "EmRegsEZTemp.h"

struct DeviceInfo
{
	DeviceType	fDeviceType;
	uint32		fFlags;
	long		fCPUSpeed;
	long		fHardwareID;
	long		fHardwareSubID;
};

static const uint32	kSupports68328			= 0x00000001;
static const uint32	kSupports68EZ328		= 0x00000002;
static const uint32	kSupports68VZ328		= 0x00000004;

static const uint32	kHasFlash				= 0x00000010;
static const uint32	kHasSED1375				= 0x00000040;
static const uint32	kHasPLD					= 0x00000080;
static const uint32	kHasUSB					= 0x00000100;
static const uint32	kHasSymbolASIC			= 0x00000200;

static const long	k16MHz					= 16580608;
static const long	k20MHz					= 20000000;

#define	hwrMiscFlagExtSubIDNone				0xFF

static const DeviceInfo kDeviceInfo[] =
{
#define UNSUPPORTED	0, 0, 0, 0
	{
		kDeviceUnspecified,
		UNSUPPORTED
	},
	{
		kDevicePilot1000,
		kSupports68328,
		k16MHz, hwrMiscFlagIDTouchdown, hwrMiscFlagExtSubIDNone
	},
	{
		kDevicePilot5000,
		kSupports68328,
		k16MHz, hwrMiscFlagIDTouchdown, hwrMiscFlagExtSubIDNone
	},
	{
		kDevicePalmPilotPersonal,
		kSupports68328,
		k16MHz, hwrMiscFlagIDPalmPilot, hwrMiscFlagExtSubIDNone
	},
	{
		kDevicePalmPilotProfessional,
		kSupports68328,
		k16MHz, hwrMiscFlagIDPalmPilot, hwrMiscFlagExtSubIDNone
	},
	{
		kDevicePalmPilotWithUpgrade,
		UNSUPPORTED
	},
	{
		kDevicePalmIII,
		kSupports68328 + kHasFlash,
		k16MHz, hwrMiscFlagIDRocky, hwrMiscFlagExtSubIDNone
	},
	{
		kDevicePalmVII,
		kSupports68328 + kHasFlash,
		k16MHz, hwrMiscFlagIDJerry, hwrMiscFlagExtSubIDNone
	},
	{
		kDeviceUnused1,
		UNSUPPORTED
	},
	{
		kDeviceQualComm,
		UNSUPPORTED
	},
	{
		kDevicePalmV,
		kSupports68EZ328 + kHasFlash,
		k16MHz, hwrMiscFlagIDSumo, hwrMiscFlagExtSubIDSumo
	},
	{
		kDevicePalmIIIx,
		kSupports68EZ328 + kHasFlash,
		k16MHz, hwrMiscFlagIDBrad, hwrMiscFlagExtSubIDBrad
	},
	{
		kDeviceUnused2,
		UNSUPPORTED
	},
	{
		kDevicePalmIIIc,
		kSupports68EZ328 + kHasFlash + kHasSED1375,
		k16MHz, hwrMiscFlagIDPalmIIIc, hwrMiscFlagExtSubIDNone
	},
	{
		kDevicePalmVIIEZ,
		kSupports68EZ328 + kHasFlash + kHasPLD,
		k16MHz, hwrMiscFlagIDJerryEZ, hwrMiscFlagExtSubIDNone
	},
	{
		kDevicePalmIIIe,
		kSupports68EZ328,
		k16MHz, hwrMiscFlagIDBrad, hwrMiscFlagExtSubIDBrad
	},
	{
		kDevicePalmVx,
		kSupports68EZ328 + kHasFlash,
		k20MHz, hwrMiscFlagIDSumo, hwrMiscFlagExtSubIDCobra
	},
	{
		kDeviceSymbol1700,
		kSupports68328 + kHasFlash + kHasSymbolASIC,
		k16MHz, hwrMiscFlagIDRocky, hwrMiscFlagExtSubIDNone
	},
	{
		kDeviceTRGpro,
//		kSupports68EZ328 + kHasFlash,
//		k16MHz, hwrMiscFlagIDBrad, hwrMiscFlagExtSubIDBrad
		UNSUPPORTED
	},
	{
		kDeviceVisor,
		kSupports68EZ328 + kHasUSB,
		k16MHz, 0, 0	// Hardware ID for first Visor
	},
	{
		kDeviceLast,
		UNSUPPORTED
	}
};


/***********************************************************************
 *
 * FUNCTION:    EmDevice::EmDevice
 *
 * DESCRIPTION: Constructors
 *
 * PARAMETERS:  None
 *
 * RETURNED:    Nothing
 *
 ***********************************************************************/

EmDevice::EmDevice (void) :
	fDeviceType (kDeviceUnspecified)
{
}


EmDevice::EmDevice (DeviceType device) :
	fDeviceType (device)
{
	// Make sure our device information table is built correctly.

	assert (countof (kDeviceInfo) - 1 == kDeviceLast);

	for (DeviceType dt = kDeviceUnspecified; dt <= kDeviceLast; ++dt)
	{
		assert (kDeviceInfo[dt].fDeviceType == dt);
	}

	assert (Supported());
}


EmDevice::EmDevice (const EmDevice& other) :
	fDeviceType (other.fDeviceType)
{
}


/***********************************************************************
 *
 * FUNCTION:    EmDevice::~EmDevice
 *
 * DESCRIPTION: Destructor
 *
 * PARAMETERS:  None
 *
 * RETURNED:    Nothing
 *
 ***********************************************************************/

EmDevice::~EmDevice (void)
{
}


/***********************************************************************
 *
 * FUNCTION:    EmDevice::Supported
 *
 * DESCRIPTION: Returns whether or not this EmDevice represents a device
 *				we currently support emulating.
 *
 * PARAMETERS:  None
 *
 * RETURNED:    True if so.
 *
 ***********************************************************************/

Bool EmDevice::Supported (void) const
{
	return fDeviceType <= kDeviceLast && kDeviceInfo[fDeviceType].fCPUSpeed != 0;
}


/***********************************************************************
 *
 * FUNCTION:    EmDevice::Supports68328
 *
 * DESCRIPTION: Returns whether or not this device is based on the
 *				MC68328.
 *
 * PARAMETERS:  None
 *
 * RETURNED:    True if so.
 *
 ***********************************************************************/

Bool EmDevice::Supports68328 (void) const
{
	return (kDeviceInfo[fDeviceType].fFlags & kSupports68328) != 0;
}


/***********************************************************************
 *
 * FUNCTION:    EmDevice::Supports68EZ328
 *
 * DESCRIPTION: Returns whether or not this device is based on the
 *				MC68EZ328.
 *
 * PARAMETERS:  None
 *
 * RETURNED:    True if so.
 *
 ***********************************************************************/

Bool EmDevice::Supports68EZ328 (void) const
{
	return (kDeviceInfo[fDeviceType].fFlags & kSupports68EZ328) != 0;
}


/***********************************************************************
 *
 * FUNCTION:    EmDevice::Supports68VZ328
 *
 * DESCRIPTION: Returns whether or not this device is based on the
 *				MC68VZ328.
 *
 * PARAMETERS:  None
 *
 * RETURNED:    True if so.
 *
 ***********************************************************************/

Bool EmDevice::Supports68VZ328 (void) const
{
	return (kDeviceInfo[fDeviceType].fFlags & kSupports68VZ328) != 0;
}


/***********************************************************************
 *
 * FUNCTION:    EmDevice::CreateRegs
 *
 * DESCRIPTION: Create the register object that manages the Dragonball
 *				(or derivative) hardware registers.
 *
 * PARAMETERS:  None
 *
 * RETURNED:    Pointer to the created object.  Caller is responsible
 *				for destroying the object when it's done with it.
 *
 ***********************************************************************/

EmRegs* EmDevice::CreateRegs (void) const
{
	EmRegs*	regs = NULL;

	switch (fDeviceType)
	{
		case kDeviceUnspecified:			regs = NULL;					break;
		case kDevicePilot1000:				regs = new EmRegs328Pilot;		break;
		case kDevicePilot5000:				regs = new EmRegs328Pilot;		break;
		case kDevicePalmPilotPersonal:		regs = new EmRegs328PalmPilot;	break;
		case kDevicePalmPilotProfessional:	regs = new EmRegs328PalmPilot;	break;
		case kDevicePalmPilotWithUpgrade:	regs = NULL;					break;
		case kDevicePalmIII:				regs = new EmRegs328PalmIII;	break;
		case kDevicePalmVII:				regs = new EmRegs328PalmVII;	break;
		case kDeviceUnused1:				regs = NULL;					break;
		case kDeviceQualComm:				regs = NULL;					break;
		case kDevicePalmV:					regs = new EmRegsEZPalmV;		break;
		case kDevicePalmIIIx:				regs = new EmRegsEZPalmIIIx;	break;
		case kDeviceUnused2:				regs = NULL;					break;
		case kDevicePalmIIIc: 				regs = new EmRegsEZPalmIIIc;	break;
		case kDevicePalmVIIEZ: 				regs = new EmRegsEZPalmVII;		break;
		case kDevicePalmIIIe:				regs = new EmRegsEZPalmIIIe;	break;
		case kDevicePalmVx:					regs = new EmRegsEZPalmVx;		break;
		case kDeviceSymbol1700:				regs = new EmRegs328Symbol1700;	break;
		case kDeviceTRGpro:					regs = NULL;					break;
		case kDeviceVisor:					regs = new EmRegsEZVisor;		break;
		case kDeviceLast:					assert (false);					break;
	}

	return regs;
}


/***********************************************************************
 *
 * FUNCTION:    EmDevice::HasFlash
 *
 * DESCRIPTION: Returns whether or not this device uses Flash RAM.
 *
 * PARAMETERS:  None
 *
 * RETURNED:    True if so.
 *
 ***********************************************************************/

Bool EmDevice::HasFlash (void) const
{
	return (kDeviceInfo[fDeviceType].fFlags & kHasFlash) != 0;
}


/***********************************************************************
 *
 * FUNCTION:    EmDevice::HasSED1375
 *
 * DESCRIPTION: Returns whether or not this device uses the SED 1375
 *				display.
 *
 * PARAMETERS:  None
 *
 * RETURNED:    True if so.
 *
 ***********************************************************************/

Bool EmDevice::HasSED1375 (void) const
{
	return (kDeviceInfo[fDeviceType].fFlags & kHasSED1375) != 0;
}


/***********************************************************************
 *
 * FUNCTION:    EmDevice::HasPLD
 *
 * DESCRIPTION: Returns whether or not this device has a PLD
 *				(programmable logic device) to augment the lack of IO
 *				pins on the EZ.
 *
 * PARAMETERS:  None
 *
 * RETURNED:    True if so.
 *
 ***********************************************************************/

Bool EmDevice::HasPLD (void) const
{
	return (kDeviceInfo[fDeviceType].fFlags & kHasPLD) != 0;
}


/***********************************************************************
 *
 * FUNCTION:    EmDevice::HasUSB
 *
 * DESCRIPTION: Returns whether or not this device has a USB register
 *				bank.
 *
 * PARAMETERS:  None
 *
 * RETURNED:    True if so.
 *
 ***********************************************************************/

Bool EmDevice::HasUSB (void) const
{
	return (kDeviceInfo[fDeviceType].fFlags & kHasUSB) != 0;
}


/***********************************************************************
 *
 * FUNCTION:    EmDevice::HasSymbolASIC
 *
 * DESCRIPTION: Returns whether or not this device has a Symbol ASIC
 *				register bank.
 *
 * PARAMETERS:  None
 *
 * RETURNED:    True if so.
 *
 ***********************************************************************/

Bool EmDevice::HasSymbolASIC (void) const
{
	return (kDeviceInfo[fDeviceType].fFlags & kHasSymbolASIC) != 0;
}


/***********************************************************************
 *
 * FUNCTION:    EmDevice::CPUSpeed
 *
 * DESCRIPTION: Returns the CPU speed of this device.  Note that this is
 *				the CPU speed as shipped;  it does not attempt to adjust
 *				for hacks that alter the actual CPU speed.
 *
 * PARAMETERS:  None
 *
 * RETURNED:    CPU speed in Hz.
 *
 ***********************************************************************/

long EmDevice::CPUSpeed (void) const
{
	return kDeviceInfo[fDeviceType].fCPUSpeed;
}


/***********************************************************************
 *
 * FUNCTION:    EmDevice::HardwareID
 *
 * DESCRIPTION: Returns the hardware ID of this device.  This ID is the
 *				one that eventually gets put into GHwrMiscGFlags by
 *				HwrIdentifyFeatures.
 *
 * PARAMETERS:  None
 *
 * RETURNED:    The device's hardware ID.
 *
 ***********************************************************************/

long EmDevice::HardwareID (void) const
{
	return kDeviceInfo[fDeviceType].fHardwareID;
}


/***********************************************************************
 *
 * FUNCTION:    EmDevice::HardwareSubID
 *
 * DESCRIPTION: Returns the hardware ID of this device.  This ID is the
 *				one that eventually gets put into GHwrMiscGFlags by
 *				HwrIdentifyFeatures.
 *
 * PARAMETERS:  None
 *
 * RETURNED:    The device's hardware ID.
 *
 ***********************************************************************/

long EmDevice::HardwareSubID (void) const
{
	return kDeviceInfo[fDeviceType].fHardwareSubID;
}


/***********************************************************************
 *
 * FUNCTION:    EmDevice::GetMenuItemList
 *
 * DESCRIPTION: Return the list of items that should appear in the
 *				various listboxes and menus that allow the user to
 *				select a device.  The list is returned as a collection
 *				of pairs.  The first element of the pair is the
 *				DeviceType, and the second element of the pair is
 *				the name of the device in text form, suitable for
 *				displaying to the user.
 *
 * PARAMETERS:  None
 *
 * RETURNED:    Collection containing the results
 *
 ***********************************************************************/

EmDeviceMenuItemList EmDevice::GetMenuItemList (void)
{
	EmDeviceMenuItemList	result;

	result.push_back (make_pair (kDevicePilot1000,			string ("Pilot")));
	result.push_back (make_pair (kDevicePalmPilotPersonal,	string ("PalmPilot")));
	result.push_back (make_pair (kDevicePalmIII,			string ("Palm III")));
	result.push_back (make_pair (kDevicePalmIIIc,			string ("Palm IIIc")));
	result.push_back (make_pair (kDevicePalmIIIe,			string ("Palm IIIe")));
	result.push_back (make_pair (kDevicePalmIIIx,			string ("Palm IIIx")));
	result.push_back (make_pair (kDevicePalmV,				string ("Palm V")));
	result.push_back (make_pair (kDevicePalmVx,				string ("Palm Vx")));
	result.push_back (make_pair (kDevicePalmVII,			string ("Palm VII")));
	result.push_back (make_pair (kDevicePalmVIIEZ,			string ("Palm VII EZ")));

	result.push_back (make_pair (kDeviceSymbol1700,			string ("Symbol 1700")));
//	result.push_back (make_pair (kDeviceTRGpro,				string ("TRGpro")));
	result.push_back (make_pair (kDeviceVisor,				string ("Visor")));

	return result;
}


/***********************************************************************
 *
 * FUNCTION:    EmDevice::GetStartupItemList
 *
 * DESCRIPTION: Return a list of items used to help process the -device
 *				option from command-line options.  Each item in the
 *				list is a DeviceType along with a string that the user
 *				can type in to select that device.
 *
 * PARAMETERS:  None
 *
 * RETURNED:    Collection containing the results
 *
 ***********************************************************************/

EmDeviceStartupItemList EmDevice::GetStartupItemList (void)
{
	EmDeviceStartupItemList	result;

	// Add text entries for all the devices.  Do it in this goofy way
	// so that we can take advantages of compilers that report when
	// not all enumerated types are accounted for in the case statements.

	for (DeviceType device = kDeviceUnspecified; device < kDeviceLast; ++device)
	{
		const char* name = NULL;

		switch (device)
		{
			case kDeviceUnspecified:
				break;

			case kDevicePilot1000:
				result.push_back (make_pair (device, string ("Pilot")));
				result.push_back (make_pair (device, string ("Pilot1000")));
				result.push_back (make_pair (device, string ("Pilot5000")));
				break;

			case kDevicePilot5000:
				break;

			case kDevicePalmPilotPersonal:
				result.push_back (make_pair (device, string ("PalmPilot")));
				result.push_back (make_pair (device, string ("PalmPilotPersonal")));
				result.push_back (make_pair (device, string ("PalmPilotProfessional")));
				break;

			case kDevicePalmPilotProfessional:
				break;

			case kDevicePalmPilotWithUpgrade:
				break;

			case kDevicePalmIII:
				result.push_back (make_pair (device, string ("PalmIII")));
				break;

			case kDevicePalmVII:
				result.push_back (make_pair (device, string ("PalmVII")));
				break;

			case kDeviceUnused1:
				break;

			case kDeviceQualComm:
				break;

			case kDevicePalmV:
				result.push_back (make_pair (device, string ("PalmV")));
				break;

			case kDevicePalmIIIx:
				result.push_back (make_pair (device, string ("PalmIIIx")));
				break;

			case kDeviceUnused2:
				break;

			case kDevicePalmIIIc:
				result.push_back (make_pair (device, string ("PalmIIIc")));
				break;

			case kDevicePalmVIIEZ:
				result.push_back (make_pair (device, string ("PalmVIIEZ")));
				break;

			case kDevicePalmIIIe:
				result.push_back (make_pair (device, string ("PalmIIIe")));
				break;

			case kDevicePalmVx:
				result.push_back (make_pair (device, string ("PalmVx")));
				break;

			case kDeviceSymbol1700:
				result.push_back (make_pair (device, string ("Symbol1700")));
				break;

			case kDeviceTRGpro:
				result.push_back (make_pair (device, string ("TRGpro")));
				break;

			case kDeviceVisor:
				result.push_back (make_pair (device, string ("Visor")));
				break;

			case kDeviceLast:
				assert (false);
				break;
		}

		if (name)
			result.push_back (make_pair (device, string (name)));
	}

	return result;
}


/***********************************************************************
 *
 * FUNCTION:    EmDevice::GetPrefItemList
 *
 * DESCRIPTION: Return a list of items used to generate keys in the
 *				preferences file.
 *
 * PARAMETERS:  None
 *
 * RETURNED:    Collection containing the results
 *
 ***********************************************************************/

EmDevicePrefItemList EmDevice::GetPrefItemList (void)
{
	EmDevicePrefItemList	result;

	// Add text entries for all the devices.  Do it in this goofy way
	// so that we can take advantages of compilers that report when
	// not all enumerated types are accounted for in the case statements.

	for (DeviceType device = kDeviceUnspecified; device < kDeviceLast; ++device)
	{
		const char* name = NULL;

		switch (device)
		{
			case kDeviceUnspecified:			name = "Unspecified";			break;
			case kDevicePilot1000:				name = "Pilot1000";				break;
			case kDevicePilot5000:				name = "Pilot5000";				break;
			case kDevicePalmPilotPersonal:		name = "PalmPilotPersonal";		break;
			case kDevicePalmPilotProfessional:	name = "PalmPilotProfessional";	break;
			case kDevicePalmPilotWithUpgrade:	name = "PalmPilotWithUpgrade";	break;
			case kDevicePalmIII:				name = "PalmIII";				break;
			case kDevicePalmVII:				name = "PalmVII";				break;
			case kDeviceUnused1:												break;
			case kDeviceQualComm:				name = "QualComm";				break;
			case kDevicePalmV:					name = "PalmV";					break;
			case kDevicePalmIIIx:				name = "PalmIIIx";				break;
			case kDeviceUnused2:												break;
			case kDevicePalmIIIc: 				name = "PalmIIIc";				break;
			case kDevicePalmVIIEZ: 				name = "PalmVIIEZ";				break;
			case kDevicePalmIIIe:				name = "PalmIIIe";				break;
			case kDevicePalmVx:					name = "PalmVx";				break;
			case kDeviceSymbol1700:				name = "Symbol1700";			break;
			case kDeviceTRGpro:					name = "TRGpro";				break;
			case kDeviceVisor:					name = "Visor";					break;
			case kDeviceLast:					assert (false);					break;
		}

		if (name)
			result.push_back (make_pair (device, string (name)));
	}

	return result;
}
