/* -*- mode: C++; tab-width: 4 -*- */
/* ===================================================================== *\
	Copyright (c) 1998-2000 Palm, Inc. or its subsidiaries.
	All rights reserved.

	This file is part of the Palm OS Emulator.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
\* ===================================================================== */

#ifndef EmDevice_h
#define EmDevice_h

#include <vector>				// vector
#include <utility>				// pair
#include <string>				// string

class EmRegs;


enum DeviceType 
{
	kDeviceUnspecified	= 0,
	kDevicePilot1000,
	kDevicePilot5000,
	kDevicePalmPilotPersonal,
	kDevicePalmPilotProfessional,
	kDevicePalmPilotWithUpgrade,
	kDevicePalmIII,
	kDevicePalmVII,
	kDeviceUnused1,
	kDeviceQualComm,
	kDevicePalmV,
	kDevicePalmIIIx,
	kDeviceUnused2,
	kDevicePalmIIIc,
	kDevicePalmVIIEZ,
	kDevicePalmIIIe,
	kDevicePalmVx,
	kDeviceSymbol1700,
	kDeviceTRGpro,
	kDeviceVisor,
	kDeviceLast
};
typedef vector<DeviceType>	DeviceList;

	// Pre-increment operator
inline void	operator++(DeviceType& x)
{
	x = DeviceType (x + 1);
}

	// Post-increment operator
inline void	operator++(DeviceType& x, int)
{
	x = DeviceType (x + 1);
}

	// Pre-decrement operator
inline void	operator--(DeviceType& x)
{
	x = DeviceType (x - 1);
}

	// Post-decrement operator
inline void	operator--(DeviceType& x, int)
{
	x = DeviceType (x - 1);
}



// Used to generate/return a list of items to be added to a Device menu.

typedef pair<DeviceType, string>	EmDeviceMenuItem;
typedef vector<EmDeviceMenuItem>	EmDeviceMenuItemList;


// Used to generate/return a list of items that can be specified as
// the device option on the command line.

typedef pair<DeviceType, string>	EmDeviceStartupItem;
typedef vector<EmDeviceStartupItem>	EmDeviceStartupItemList;


// Used to generate/return a list of items that can be used for
// storing a device type to the preference file.

typedef pair<DeviceType, string>	EmDevicePrefItem;
typedef vector<EmDevicePrefItem>	EmDevicePrefItemList;


class EmDevice
{
	public:
								EmDevice			(void);
								EmDevice			(DeviceType);
								EmDevice			(const EmDevice&);
								~EmDevice			(void);

		Bool					Supported			(void) const;

		Bool					Supports68328		(void) const;
		Bool					Supports68EZ328		(void) const;
		Bool					Supports68VZ328		(void) const;

		EmRegs*					CreateRegs			(void) const;

		Bool					HasFlash			(void) const;
		Bool					HasSED1375			(void) const;
		Bool					HasPLD				(void) const;
		Bool					HasUSB				(void) const;
		Bool					HasSymbolASIC		(void) const;

		long					CPUSpeed			(void) const;

		long					HardwareID			(void) const;
		long					HardwareSubID		(void) const;

	public:
		static EmDeviceMenuItemList		GetMenuItemList		(void);
		static EmDeviceStartupItemList	GetStartupItemList	(void);
		static EmDevicePrefItemList		GetPrefItemList		(void);

	private:
		DeviceType				fDeviceType;
};

#endif	/* EmDevice_h */
