/* -*- mode: C++; tab-width: 4 -*- */
/* ===================================================================== *\
	Copyright (c) 1998-2000 Palm, Inc. or its subsidiaries.
	All rights reserved.

	This file is part of the Palm OS Emulator.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
\* ===================================================================== */

#include "EmCommon.h"
#include "EmDlg.h"

#include "EmROMTransfer.h"		// EmROMTransfer
#include "EmTransportSerial.h"	// PortNameList, GetSerialPortNameList, etc.
#include "Hordes.h"				// Hordes::New
#include "Logging.h"			// FOR_EACH_LOG_PREF
#include "Miscellaneous.h"		// MemoryTextList, GetMemoryTextList
#include "Platform.h"			// Platform
#include "PreferenceMgr.h"		// Preference
#include "Strings.r.h"			// kStr_AppName
#include "Skins.h"				// SkinNameList

#include "algorithm"			// find


static void PrvConvertBaudListToStrings (const EmTransportSerial::BaudList& baudList,
									StringList& baudStrList);

/***********************************************************************
 *
 * FUNCTION:	DoGetFile
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

EmDlgItemID EmDlg::DoGetFile (	EmFileRef& result,
								const string& prompt,
								const EmDirRef& defaultPath,
								const EmFileTypeList& filterList)
{
	return HostRunGetFile (result, prompt, defaultPath, filterList);
}


/***********************************************************************
 *
 * FUNCTION:	DoGetFileList
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

EmDlgItemID EmDlg::DoGetFileList (	EmFileRefList& results,
									const string& prompt,
									const EmDirRef& defaultPath,
									const EmFileTypeList& filterList)
{
	return HostRunGetFileList (results, prompt, defaultPath, filterList);
}


/***********************************************************************
 *
 * FUNCTION:	DoGetDirectory
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/


/***********************************************************************
 *
 * FUNCTION:	DoPutFile
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

EmDlgItemID EmDlg::DoPutFile (	EmFileRef& result,
								const string& prompt,
								const EmDirRef& defaultPath,
								const EmFileTypeList& filterList,
								const string& defName)
{
	return HostRunPutFile (result, prompt, defaultPath, filterList, defName);
}


/***********************************************************************
 *
 * FUNCTION:	DoAboutBox
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

EmDlgFnResult EmDlg::PrvAboutBox (EmDlgContext& context)
{
	EmDlgRef	dlg = context.fDlg;

	switch (context.fCommandID)
	{
		case kDlgCmdInit:
		{
			if (context.fPanelID == kDlgPanelAbtMain)
			{
				Errors::SetParameter ("%application", Platform::GetString (kStr_AppName));
				Errors::SetParameter ("%version", Platform::GetShortVersionString ());

				string	appAndVersion = Errors::ReplaceParameters (kStr_AppAndVers);

				SetItemText (dlg, kDlgItemAbtAppName, appAndVersion);
			}
			
			break;
		}

		case kDlgCmdIdle:
		{
			break;
		}

		case kDlgCmdItemSelected:
		{
			switch (context.fItemID)
			{
				case kDlgItemOK:
				case kDlgItemCancel:
				{
					return kDlgResultClose;
				}
				
				case kDlgItemAbtURLPalmWeb:
				case kDlgItemAbtURLPalmMail:
				case kDlgItemAbtURLWindowsWeb:
				case kDlgItemAbtURLWindowsMail:
				case kDlgItemAbtURLMacWeb:
				case kDlgItemAbtURLMacMail:
				case kDlgItemAbtURLUAEWeb:
				case kDlgItemAbtURLUAEMail:
				case kDlgItemAbtURLQNXWeb:
				case kDlgItemAbtURLQNXMail:
				{
					string	URL = GetItemText (dlg, context.fItemID);
#if 0
					::LaunchURL (URL);
#endif
					break;
				}
				
				default:
					assert (false);
			}

			break;
		}

		case kDlgCmdPanelEnter:
		case kDlgCmdPanelExit:
		case kDlgCmdNone:
			assert (false);
			break;
	}

	return kDlgResultContinue;
}

EmDlgItemID EmDlg::DoAboutBox (void)
{
	return HostRunDialog (PrvAboutBox, NULL, kDlgAboutBox);
}


/***********************************************************************
 *
 * FUNCTION:	DoSessionNew
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

struct SessionNewData
{
	Configuration*		cfg;
	Configuration		fWorkingCfg;
	SkinNameList		fSkinChoices;
};

EmDlgFnResult EmDlg::PrvSessionNew (EmDlgContext& context)
{
	SessionNewData*	data = (SessionNewData*) context.fUserData;
	EmDlgRef		dlg = context.fDlg;
	Configuration&	cfg = data->fWorkingCfg;

	switch (context.fCommandID)
	{
		case kDlgCmdInit:
		{
			// Make a copy of the configuration that we update as the
			// user makes dialog selections.

			cfg = *(data->cfg);

			// Get the list of skins that the user wants to use.

			Preference<SkinNameList>	prefSkins (kPrefKeySkins);
			data->fSkinChoices = *prefSkins;

			// Build our menus.

			PrvBuildDeviceMenu (context);
			PrvBuildSkinMenu (context);
			PrvBuildMemorySizeMenu (context);
			PrvSetROMName (context);
			break;
		}

		case kDlgCmdIdle:
		{
			break;
		}

		case kDlgCmdItemSelected:
		{
			switch (context.fItemID)
			{
				case kDlgItemOK:
				{
					*(data->cfg) = cfg;
					
					Preference<SkinNameList>	prefSkins (kPrefKeySkins);
					prefSkins = data->fSkinChoices;
					// Fall thru...
				}

				case kDlgItemCancel:
				{
					return kDlgResultClose;
				}
				
				case kDlgItemNewDevice:
				{
					EmDeviceMenuItemList			devices = EmDevice::GetMenuItemList ();
					EmDeviceMenuItemList::size_type	index = GetItemValue (dlg, kDlgItemNewDevice);

					cfg.fDeviceType = devices [index].first;

					PrvBuildSkinMenu (context);
					break;
				}

				case kDlgItemNewSkin:
				{
					SkinNameList	skins;
					::SkinGetSkinNames (cfg.fDeviceType, skins);

					DeviceType				device = cfg.fDeviceType;
					SkinNameList::size_type	index = GetItemValue (dlg, kDlgItemNewSkin);

					data->fSkinChoices [device] = skins [index];
					break;
				}

				case kDlgItemNewMemorySize:
				{
					MemoryTextList	sizes;
					::GetMemoryTextList (sizes);

					MemoryTextList::size_type	index = GetItemValue (dlg, kDlgItemNewMemorySize);

					cfg.fRAMSize = sizes [index].first;
					break;
				}

				case kDlgItemNewROMBrowse:
				{
					EmFileTypeList	typeList;
					typeList.push_back (kFileTypeROM);

					EmFileRef	romFileRef;

					if (DoGetFile (romFileRef, "", "", typeList) == kDlgItemOK)
					{
						cfg.fROMFile = romFileRef;
						PrvSetROMName (context);
					}
					break;
				}
				
				default:
					assert (false);
			}	// switch (context.fItemID)

			break;
		}	// case kDlgCmdItemSelected

		case kDlgCmdPanelEnter:
		case kDlgCmdPanelExit:
		case kDlgCmdNone:
			assert (false);
			break;
	}	// switch (command)

	return kDlgResultContinue;
}

EmDlgItemID EmDlg::DoSessionNew (Configuration& cfg)
{
	SessionNewData	data;
	data.cfg = &cfg;

	return HostRunDialog (PrvSessionNew, &data, kDlgSessionNew);
}


/***********************************************************************
 *
 * FUNCTION:	DoSessionSave
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/


/***********************************************************************
 *
 * FUNCTION:	DoHordeNew
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

struct HordeNewData
{
	HordeInfo		info;
	EmDatabaseList	appList;
};

EmDlgFnResult EmDlg::PrvHordeNew (EmDlgContext& context)
{
	HordeNewData*	data = (HordeNewData*) context.fUserData;
	EmDlgRef		dlg = context.fDlg;
	HordeInfo&		info = data->info;

	switch (context.fCommandID)
	{
		case kDlgCmdInit:
		{
			// Get Horde preference settings.

			Preference<HordeInfo>	pref (kPrefKeyHordeInfo);
			info = *pref;

			// Install the settings into the dialog box items.

			SetItemValue (dlg, kDlgItemHrdStart,		info.fStartNumber);
			SetItemValue (dlg, kDlgItemHrdStop,			info.fStopNumber);
			SetItemValue (dlg, kDlgItemHrdDepthSwitch,	info.fDepthSwitch);
			SetItemValue (dlg, kDlgItemHrdDepthSave,	info.fDepthSave);
			SetItemValue (dlg, kDlgItemHrdDepthStop,	info.fDepthStop);
			SetItemValue (dlg, kDlgItemHrdCheckSwitch,	info.fCanSwitch);
			SetItemValue (dlg, kDlgItemHrdCheckSave,	info.fCanSave);
			SetItemValue (dlg, kDlgItemHrdCheckStop,	info.fCanStop);

			// Get the list of applications.

			::GetDatabases (data->appList, kApplicationsOnly);

			// Insert the names of the applications into the list.

			Bool	selected = false;
			EmDatabaseList::iterator	iter = data->appList.begin();
			while (iter != data->appList.end())
			{
				DatabaseInfo&	thisDB = *iter;

				// Add the current item.

				AppendToList (dlg, kDlgItemHrdAppList, thisDB.name);

				// If the item we just added is in our "selected" list,
				// then select it.

				EmDatabaseList::iterator	begin = info.fAppList.begin ();
				EmDatabaseList::iterator	end = info.fAppList.end ();

				if (find (begin, end, thisDB) != end)
				{
					selected = true;
					int index = iter - data->appList.begin();
					SelectListItem (dlg, kDlgItemHrdAppList, index);
				}

				++iter;
			}

			// If nothing's selected, then select something (the first item).

			if (!selected)
			{
				SelectListItem (dlg, kDlgItemHrdAppList, 0);
			}

			break;
		}

		case kDlgCmdIdle:
		{
			break;
		}

		case kDlgCmdItemSelected:
		{
			switch (context.fItemID)
			{
				case kDlgItemOK:
				{
					info.fStartNumber	= GetItemValue (dlg, kDlgItemHrdStart);
					info.fStopNumber	= GetItemValue (dlg, kDlgItemHrdStop);
					info.fDepthSwitch	= GetItemValue (dlg, kDlgItemHrdDepthSwitch);
					info.fDepthSave		= GetItemValue (dlg, kDlgItemHrdDepthSave);
					info.fDepthStop		= GetItemValue (dlg, kDlgItemHrdDepthStop);
					info.fCanSwitch		= GetItemValue (dlg, kDlgItemHrdCheckSwitch);
					info.fCanSave		= GetItemValue (dlg, kDlgItemHrdCheckSave);
					info.fCanStop		= GetItemValue (dlg, kDlgItemHrdCheckStop);

					// Get the indexes for the selected items.

					EmDlgListIndexList	items;
					GetSelectedItems(dlg, kDlgItemHrdAppList, items);

					// Use the indexes to get the actual items, and add those
					// items to the "result" container.

					info.fAppList.clear ();
					EmDlgListIndexList::iterator	iter = items.begin();
					while (iter != items.end())
					{
						DatabaseInfo	dbInfo = data->appList[*iter];
						info.fAppList.push_back (dbInfo);
						++iter;
					}

					// Transfer the new fields to the old fields for Hordes to use.

					info.NewToOld ();

					// Save the preferences

					Preference<HordeInfo>	pref (kPrefKeyHordeInfo);
					pref = info;

					// Startup up the gremlin sub-system.

					Hordes::New (info);

					// Fall thru...
				}

				case kDlgItemCancel:
				{
					return kDlgResultClose;
				}

				case kDlgItemHrdAppList:
				case kDlgItemHrdStart:
				case kDlgItemHrdStop:
				case kDlgItemHrdCheckSwitch:
				case kDlgItemHrdCheckSave:
				case kDlgItemHrdCheckStop:
				case kDlgItemHrdDepthSwitch:
				case kDlgItemHrdDepthSave:
				case kDlgItemHrdDepthStop:
					break;

				case kDlgItemHrdLogging:
					DoEditLoggingOptions (kGremlinLogging);
					break;

				default:
					assert (false);
			}

			break;
		}

		case kDlgCmdPanelEnter:
		case kDlgCmdPanelExit:
		case kDlgCmdNone:
			assert (false);
			break;
	}

	return kDlgResultContinue;
}

EmDlgItemID EmDlg::DoHordeNew (void)
{
	HordeNewData	data;

	return HostRunDialog (PrvHordeNew, &data, kDlgHordeNew);
}


/***********************************************************************
 *
 * FUNCTION:	DatabaseImportProgressOpen
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

/***********************************************************************
 *
 * FUNCTION:	DatabaseImportProgressUpdate
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

/***********************************************************************
 *
 * FUNCTION:	DatabaseImportProgressClose
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

/***********************************************************************
 *
 * FUNCTION:	DoDatabaseExport
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

#if 0
struct ExportDatbaseData
{
	EmDatabaseList*	dbList;
	EmDatabaseList	allDatabases;
};
#endif

EmDlgFnResult EmDlg::PrvDatabaseExport (EmDlgContext&)
{
#if 0
	ExportDatbaseData*	data = (ExportDatbaseData*) context.fUserData;

	switch (context.fCommandID)
	{
		case kDlgCmdInit:
		{
			// Clear out the results container.

			data->selectedDatabases.clear();

			// Get the list of installed databases.

			::GetDatabases (data->allDatabases, kAllDatabases);

			// Add then names of the databases to the list.

			EmDatabaseList::iterator	iter = data->allDatabases.begin();
			while (iter != data->allDatabases.end())
			{
				AppendToList (dlg, kDlgItemExpDbList, iter->name);
				++iter;
			}
			break;
		}

		case kDlgCmdIdle:
		{
			break;
		}

		case kDlgCmdItemSelected:
		{
			switch (context.fItemID)
			{
				case kDlgItemOK;
				{
					// Get the indexes for the selected items.

					EmDlgSelectedItemsList	items;
					GetSelectedItems(dlg, kDlgItemExpDbList, items);

					// Use the indexes to get the actual items, and add those
					// items to the "result" container.

					EmDlgSelectedItemsList::iterator	iter = items.begin();
					while (iter != items.end())
					{
						data->dbList->push_back (data->allDatabases[*iter]);
						++iter;
					}

					// Fall thru...
				}

				case kDlgItemCancel:
				{
					return kDlgResultClose;
				}

				default:
					assert (false);
			}

			break;
		}
	}
#endif

	return kDlgResultContinue;
}

#if 0
EmDlgItemID EmDlg::DoDatabaseExport (EmDatabaseList& dbList)
{
	ExportDatbaseData	data;
	data.dbList = &dbList;

	return HostRunDialog (PrvDatabaseExport, &data, kDlgDatabaseExport);
}
#endif


/***********************************************************************
 *
 * FUNCTION:	DoROMTransferQuery
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

EmDlgFnResult EmDlg::PrvROMTransferQuery (EmDlgContext& context)
{
	EmDlgRef				dlg = context.fDlg;
	EmTransport::Config**	config = (EmTransport::Config**) context.fUserData;

	switch (context.fCommandID)
	{
		case kDlgCmdInit:
		{
			EmTransportSerial::PortNameList	portList;
			EmTransportSerial::GetSerialPortNameList (portList);

			EmTransportSerial::BaudList		baudList;
			EmTransportSerial::GetSerialBaudList (baudList);

			StringList		baudStrList;
			::PrvConvertBaudListToStrings (baudList, baudStrList);

			// On Unix, the port is specified in an edit text item.
			// On other platforms, we presend a list of ports in a menu.

#if PLATFORM_UNIX
			SetItemText (dlg, kDlgItemDlqPortList, portList[0]);
#else
			AppendToMenu (dlg, kDlgItemDlqPortList, portList);
#endif
			AppendToMenu (dlg, kDlgItemDlqBaudList, baudStrList);

			SetItemValue (dlg, kDlgItemDlqPortList, 0);		// First port
			SetItemValue (dlg, kDlgItemDlqBaudList, 0);		// 115K baud

			// Load the instructions string.  Load it in 9 parts,
			// as the entire string is greater than:
			//
			//	Rez can handle on the Mac.
			//	MSDev's string editor on Windows.
			//	The ANSI-allowed string length (for Unix).

			string	ins;
			for (int ii = 0; ii < 9; ++ii)
			{
				string	temp = Platform::GetString (kStr_ROMTransferInstructions + ii);
				ins += temp;
			}
			SetItemText (dlg, kDlgItemDlqInstructions, ins);

			break;
		}

		case kDlgCmdIdle:
		{
			break;
		}

		case kDlgCmdItemSelected:
		{
			switch (context.fItemID)
			{
				case kDlgItemOK:
				{
					// Get the selected settings.

					long	portNum = GetItemValue (dlg, kDlgItemDlqPortList);
					long	baudNum = GetItemValue (dlg, kDlgItemDlqBaudList);

					EmTransportSerial::PortNameList	portList;
					EmTransportSerial::GetSerialPortNameList (portList);

					EmTransportSerial::BaudList		baudList;
					EmTransportSerial::GetSerialBaudList (baudList);

					EmTransportSerial::ConfigSerial*	serConfig = new EmTransportSerial::ConfigSerial;

					serConfig->fPort			= portList [portNum];
					serConfig->fBaud			= baudList [baudNum];
					serConfig->fParity			= EmTransportSerial::kNoParity;
					serConfig->fStopBits		= 1;
					serConfig->fDataBits		= 8;
					serConfig->fHwrHandshake	= true;

					*config = serConfig;

					// Fall thru...
				}

				case kDlgItemCancel:
				{
					return kDlgResultClose;
				}

				case kDlgItemDlqInstructions:
				case kDlgItemDlqPortList:
				case kDlgItemDlqBaudList:
					break;

				default:
					assert (false);
			}

			break;
		}
		
		case kDlgCmdPanelEnter:
		case kDlgCmdPanelExit:
		case kDlgCmdNone:
			assert (false);
			break;
	}

	return kDlgResultContinue;
}

EmDlgItemID EmDlg::DoROMTransferQuery (EmTransport::Config*& config)
{
	config = NULL;
	return HostRunDialog (PrvROMTransferQuery, &config, kDlgROMTransferQuery);
}


/***********************************************************************
 *
 * FUNCTION:	DoROMTransferProgress
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

EmDlgFnResult EmDlg::PrvROMTransferProgress (EmDlgContext& context)
{
	EmDlgRef		dlg = context.fDlg;
	EmROMTransfer*	transferer = (EmROMTransfer*) context.fUserData;

	switch (context.fCommandID)
	{
		case kDlgCmdInit:
		{
			context.fNeedsIdle = true;
			break;
		}

		case kDlgCmdIdle:
		{
			if (!transferer->Continue (dlg))
				return kDlgResultClose;
			break;
		}

		case kDlgCmdItemSelected:
		{
			switch (context.fItemID)
			{
				case kDlgItemOK:
				{
					// Fall thru...
				}

				case kDlgItemCancel:
				{
					transferer->Abort (dlg);
					return kDlgResultClose;
				}

				default:
					assert (false);
			}

			break;
		}
		
		case kDlgCmdPanelEnter:
		case kDlgCmdPanelExit:
		case kDlgCmdNone:
			assert (false);
			break;
	}

	return kDlgResultContinue;
}

EmDlgItemID EmDlg::DoROMTransferProgress (EmROMTransfer& transferer)
{
	return HostRunDialog (PrvROMTransferProgress, &transferer, kDlgROMTransferProgress);
}


/***********************************************************************
 *
 * FUNCTION:	GremlinControlOpen
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/


/***********************************************************************
 *
 * FUNCTION:	GremlinControlUpdate
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/


/***********************************************************************
 *
 * FUNCTION:	GremlinControlClose
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/


/***********************************************************************
 *
 * FUNCTION:	DoEditPreferences
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/


/***********************************************************************
 *
 * FUNCTION:	DoEditLoggingOptions
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

struct EditLoggingOptionsData
{
	LoggingType		fActiveSet;

	// Define a "uint8" variable for each of the logging options we
	// support.  The name of each variable is of the form "f<name>",
	// where "<name>" is what appear in the FOR_EACH_LOG_PREF macro.

#define DEFINE_STORAGE(name)	uint8	f##name;
	FOR_EACH_LOG_PREF (DEFINE_STORAGE)
};

static void PrvFetchPrefs (EmDlgContext& context)
{
	// Transfer all of the logging values from the preferences
	// to our local storage.

	EditLoggingOptionsData&	data = *(EditLoggingOptionsData*) context.fUserData;

	#define GET_PREF(name)	\
		{	\
			Preference<uint8> pref(kPrefKey##name);	\
			data.f##name = *pref;	\
		}

	FOR_EACH_LOG_PREF (GET_PREF)
}

static void PrvInstallPrefs (EmDlgContext& context)
{
	// Transfer all of the logging values from our local
	// storages to the preferences system.

	EditLoggingOptionsData&	data = *(EditLoggingOptionsData*) context.fUserData;

	#define SET_PREF(name)	\
		{	\
			Preference<uint8> pref(kPrefKey##name);	\
			pref = data.f##name;	\
		}

	FOR_EACH_LOG_PREF (SET_PREF)
}

static void PrvPrefsToButtons (EmDlgContext& context)
{
	// Set the buttons in in the current panel.

	EditLoggingOptionsData&	data = *(EditLoggingOptionsData*) context.fUserData;
	int	bitmask = data.fActiveSet;

	#define TO_BUTTON(name)	\
		EmDlg::SetItemValue (context.fDlg, kDlgItemLog##name, (data.f##name & bitmask) != 0);

	FOR_EACH_LOG_PREF (TO_BUTTON)
}

static void PrvPrefsFromButtons (EmDlgContext& context)
{
	// Update our local preference values from the buttons in the current panel.

	EditLoggingOptionsData&	data = *(EditLoggingOptionsData*) context.fUserData;
	int	bitmask = data.fActiveSet;

	#define FROM_BUTTON(name)	\
		if (EmDlg::GetItemValue (context.fDlg, kDlgItemLog##name) != 0)	\
			data.f##name |= bitmask;	\
		else	\
			data.f##name &= ~bitmask;

	FOR_EACH_LOG_PREF (FROM_BUTTON)
}

EmDlgFnResult EmDlg::PrvEditLoggingOptions (EmDlgContext& context)
{
	EmDlgRef				dlg = context.fDlg;
	EditLoggingOptionsData&	data = *(EditLoggingOptionsData*) context.fUserData;

	switch (context.fCommandID)
	{
		case kDlgCmdInit:
		{
			::PrvFetchPrefs (context);
			::PrvPrefsToButtons (context);

			if (data.fActiveSet == kNormalLogging)
				EmDlg::SetItemValue (dlg, kDlgItemLogNormal, 1);
			else
				EmDlg::SetItemValue (dlg, kDlgItemLogGremlins, 1);

			// Disable unsupported options.

			EmDlg::DisableItem (dlg, kDlgItemLogLogErrorMessages);
			EmDlg::DisableItem (dlg, kDlgItemLogLogWarningMessages);
			EmDlg::DisableItem (dlg, kDlgItemLogLogCPUOpcodes);
			EmDlg::DisableItem (dlg, kDlgItemLogLogApplicationCalls);
			EmDlg::DisableItem (dlg, kDlgItemLogLogExgMgr);
			EmDlg::DisableItem (dlg, kDlgItemLogLogExgMgrData);
			EmDlg::DisableItem (dlg, kDlgItemLogLogRPC);
			EmDlg::DisableItem (dlg, kDlgItemLogLogRPCData);

			break;
		}

		case kDlgCmdIdle:
		{
			break;
		}

		case kDlgCmdItemSelected:
		{
			switch (context.fItemID)
			{
				case kDlgItemOK:
				{
					::PrvPrefsFromButtons (context);
					::PrvInstallPrefs (context);

					// Fall thru...
				}

				case kDlgItemCancel:
				{
					return kDlgResultClose;
				}

				case kDlgItemLogNormal:
				case kDlgItemLogGremlins:
					::PrvPrefsFromButtons (context);
					if (context.fItemID == kDlgItemLogNormal)
						data.fActiveSet = kNormalLogging;
					else
						data.fActiveSet = kGremlinLogging;
					::PrvPrefsToButtons (context);
					break;

				#define DUMMY_CASE(name)	case kDlgItemLog##name:
				FOR_EACH_LOG_PREF (DUMMY_CASE)
					break;

				default:
					assert (false);
			}

			break;
		}

		case kDlgCmdPanelEnter:
			break;

		case kDlgCmdPanelExit:
			break;

		case kDlgCmdNone:
			assert (false);
			break;
	}

	return kDlgResultContinue;
}


EmDlgItemID EmDlg::DoEditLoggingOptions (LoggingType initialSet)
{
	EditLoggingOptionsData	data;

	data.fActiveSet	= initialSet;

	return HostRunDialog (PrvEditLoggingOptions, &data, kDlgEditLogging);
}


/***********************************************************************
 *
 * FUNCTION:	DoEditDebuggingOptions
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/


/***********************************************************************
 *
 * FUNCTION:	DoEditSkins
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

EmDlgFnResult EmDlg::PrvEditSkins (EmDlgContext& context)
{
	EmDlgRef	dlg = context.fDlg;

	switch (context.fCommandID)
	{
		case kDlgCmdInit:
		{
			// Get the current configuration so that we know what
			// device is being emulated.

			Preference<Configuration>	prefConfiguration (kPrefKeyLastConfiguration);
			Configuration				cfg = *prefConfiguration;

			// Get the list of selected skins so that we know which
			// one to select in the listbox.

			Preference<SkinNameList>	prefSkins (kPrefKeySkins);
			SkinNameList				skinChoices = *prefSkins;

			// Get the list of skins for the selected device.

			SkinNameList	skins;
			::SkinGetSkinNames (cfg.fDeviceType, skins);

			// Add each skin in the list to the UI element.

			Bool					selected = false;
			int						index = 0;
			SkinNameList::iterator	skinIter = skins.begin();
			while (skinIter != skins.end())
			{
				string&	skinName = *skinIter;
				AppendToList (dlg, kDlgItemSknSkinList, skinName);

				// Select the currently-used skin.

				if (skinIter->c_str() == skinChoices[cfg.fDeviceType])
				{
					SelectListItem (dlg, kDlgItemSknSkinList, index);
					selected = true;
				}

				++index;
				++skinIter;
			}

			// Ensure that *something* was selected.

			if (!selected)
			{
				SelectListItem (dlg, kDlgItemSknSkinList, 0);
			}

			// Set up the checkboxes

			Preference<ScaleType>	prefScale (kPrefKeyScale);
			SetItemValue (dlg, kDlgItemSknDoubleScale, (*prefScale == 2) ? 1 : 0);

			Preference<RGBType>		prefBackgroundColor (kPrefKeyBackgroundColor);
			SetItemValue (dlg, kDlgItemSknWhiteBackground, prefBackgroundColor.Loaded () ? 1 : 0);

			break;
		}

		case kDlgCmdIdle:
		{
			break;
		}

		case kDlgCmdItemSelected:
		{
			switch (context.fItemID)
			{
				case kDlgItemOK:
				{
					// Update the skin choices.
					
					// Get the current configuration so that we know what
					// device is being emulated.

					Preference<Configuration>	prefConfiguration (kPrefKeyLastConfiguration);
					Configuration				cfg = *prefConfiguration;

					// Get the selected skin.

					EmDlgListIndex	index = GetSelectedItem (dlg, kDlgItemSknSkinList);
					assert (index != kDlgItemListNone);

					// Get the skins for this device.

					SkinNameList	deviceSkins;
					::SkinGetSkinNames (cfg.fDeviceType, deviceSkins);

					// Get the skins preferences.

					Preference<SkinNameList>	prefSkins (kPrefKeySkins);

					// Update the preferences.

					SkinNameList	skins;
					skins = *prefSkins;
					skins[cfg.fDeviceType] = deviceSkins[index];
					prefSkins = skins;

					// Update the scale.

					Preference<ScaleType>	prefScale (kPrefKeyScale);
					prefScale = (GetItemValue (dlg, kDlgItemSknDoubleScale) != 0) ? 2 : 1;

					// Update the background color.

					if (GetItemValue (dlg, kDlgItemSknWhiteBackground) != 0)
					{
						RGBType				color (0xFF, 0xFF, 0xFF);
						Preference<RGBType>	prefBackgroundColor (kPrefKeyBackgroundColor);
						prefBackgroundColor = color;
					}
					else
					{
						gPrefs->DeletePref (kPrefKeyBackgroundColor);
					}
					// Fall thru...
				}

				case kDlgItemCancel:
				{
					return kDlgResultClose;
				}

				case kDlgItemSknSkinList:
				case kDlgItemSknDoubleScale:
				case kDlgItemSknWhiteBackground:
					break;

				default:
					assert (false);
			}

			break;
		}
		
		case kDlgCmdPanelEnter:
		case kDlgCmdPanelExit:
		case kDlgCmdNone:
			assert (false);
			break;
	}

	return kDlgResultContinue;
}

EmDlgItemID EmDlg::DoEditSkins (void)
{
	return HostRunDialog (PrvEditSkins, NULL, kDlgEditSkins);
}


/***********************************************************************
 *
 * FUNCTION:	DoDialog
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/


/***********************************************************************
 *
 * FUNCTION:	DoSaveBound
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

EmDlgFnResult EmDlg::PrvSaveBound (EmDlgContext& context)
{
	EmDlgRef	dlg = context.fDlg;

	switch (context.fCommandID)
	{
		case kDlgCmdInit:
		{
			DisableItem (dlg, kDlgItemOK);
			break;
		}

		case kDlgCmdIdle:
		{
			break;
		}

		case kDlgCmdItemSelected:
		{
			switch (context.fItemID)
			{
				case kDlgItemOK:
				{
					Bool			fullSave = GetItemValue (dlg, kDlgItemBmdSaveRAM) != 0;
					EmFileRef		destFile;
					EmFileTypeList	filterList (1, kFileTypeApplication);

#if PLATFORM_MAC
					string			newName ("Emulator_Bound");
#elif PLATFORM_UNIX
					string			newName ("pose_bound");
#elif PLATFORM_WINDOWS
					string			newName ("Emulator_Bound.exe");
#else
#error "Unsupported platform"
#endif

					if (DoPutFile (destFile, "Save new emulator",
						EmDirRef::GetEmulatorDirectory (), filterList, newName) == kDlgItemOK)
					{
						Platform::BindPoser (fullSave, destFile);
					}
					// Fall thru...
				}

				case kDlgItemCancel:
				{
					return kDlgResultClose;
				}

				case kDlgItemBndSaveROM:
				case kDlgItemBmdSaveRAM:
					EnableItem (dlg, kDlgItemOK);
					break;

				default:
					assert (false);
			}

			break;
		}
		
		case kDlgCmdPanelEnter:
		case kDlgCmdPanelExit:
		case kDlgCmdNone:
			assert (false);
			break;
	}

	return kDlgResultContinue;
}

EmDlgItemID EmDlg::DoSaveBound (void)
{
	return HostRunDialog (PrvSaveBound, NULL, kDlgSaveBound);
}


/***********************************************************************
 *
 * FUNCTION:	EmDlg::SetItemText
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

void EmDlg::SetItemText (EmDlgRef dlg, EmDlgItemID item, StrCode strCode)
{
	string	str = Platform::GetString (strCode);
	SetItemText (dlg, item, str);
}


/***********************************************************************
 *
 * FUNCTION:	EmDlg::SetItemText
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

void EmDlg::SetItemText (EmDlgRef dlg, EmDlgItemID item, const char* str)
{
	SetItemText (dlg, item, string (str));
}


/***********************************************************************
 *
 * FUNCTION:	EmDlg::AppendToMenu
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

void EmDlg::AppendToMenu (EmDlgRef dlg, EmDlgItemID item, const string& text)
{
	StringList	strList;
	strList.push_back (text);

	AppendToMenu (dlg, item, strList);
}


/***********************************************************************
 *
 * FUNCTION:	EmDlg::AppendToList
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

void EmDlg::AppendToList (EmDlgRef dlg, EmDlgItemID item, const string& text)
{
	StringList	strList;
	strList.push_back (text);

	AppendToList (dlg, item, strList);
}


/***********************************************************************
 *
 * FUNCTION:	EmDlg::SelectListItem
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

void EmDlg::SelectListItem (EmDlgRef dlg, EmDlgItemID item, EmDlgListIndex listItem)
{
	EmDlgListIndexList	itemList;
	itemList.push_back (listItem);

	SelectListItems (dlg, item, itemList);
}


/***********************************************************************
 *
 * FUNCTION:	EmDlg::GetSelectedItem
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

EmDlgListIndex EmDlg::GetSelectedItem (EmDlgRef dlg, EmDlgItemID item)
{
	EmDlgListIndexList	itemList;
	GetSelectedItems (dlg, item, itemList);
	
	if (itemList.size () > 0)
	{
		return itemList[0];
	}
	
	return kDlgItemListNone;
}


/***********************************************************************
 *
 * FUNCTION:	EmDlgContext c'tor
 *
 * DESCRIPTION:	Constructor.  Initialize our data members.
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

EmDlgContext::EmDlgContext (void) :
	fFn (NULL),
	fFnResult (kDlgResultNone),
	fDlg (NULL),
	fDlgID (kDlgNone),
	fPanelID (kDlgPanelNone),
	fItemID (kDlgItemNone),
	fCommandID (kDlgCmdNone),
	fNeedsIdle (false),
	fUserData (NULL)
{
}


/***********************************************************************
 *
 * FUNCTION:	EmDlgContext c'tor
 *
 * DESCRIPTION:	Constructor.  Initialize our data members.
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

EmDlgContext::EmDlgContext (const EmDlgContext& other) :
	fFn (other.fFn),
	fFnResult (other.fFnResult),
	fDlg (other.fDlg),
	fDlgID (other.fDlgID),
	fPanelID (other.fPanelID),
	fItemID (other.fItemID),
	fCommandID (other.fCommandID),
	fNeedsIdle (other.fNeedsIdle),
	fUserData (other.fUserData)
{
}


/***********************************************************************
 *
 * FUNCTION:	EmDlgContext::Init
 *
 * DESCRIPTION:	Call the custom dialog handler to initialize the dialog.
 *				If the dialog says that it wants idle time, then start
 *				idleing it.
 *
 * PARAMETERS:	None
 *
 * RETURNED:	The result returned by the dialog handler.
 *
 ***********************************************************************/

EmDlgFnResult EmDlgContext::Init (void)
{
	fItemID		= kDlgItemNone;
	fCommandID	= kDlgCmdInit;

	fFnResult	= fFn (*this);

	if (fNeedsIdle)
		EmDlg::HostStartIdling (*this);

	return fFnResult;
}


/***********************************************************************
 *
 * FUNCTION:	EmDlgContext::Event
 *
 * DESCRIPTION:	Call the custom dialog handler to handle an event.
 *
 * PARAMETERS:	item - dialog item that was selected.
 *
 * RETURNED:	The result returned by the dialog handler.
 *
 ***********************************************************************/

EmDlgFnResult EmDlgContext::Event (EmDlgItemID itemID)
{	
	fItemID		= itemID;
	fCommandID	= kDlgCmdItemSelected;

	fFnResult	= fFn (*this);
	return fFnResult;
}


/***********************************************************************
 *
 * FUNCTION:	EmDlgContext::Idle
 *
 * DESCRIPTION:	Call the custom dialog handler to idle.
 *
 * PARAMETERS:	None
 *
 * RETURNED:	The result returned by the dialog handler.
 *
 ***********************************************************************/

EmDlgFnResult EmDlgContext::Idle (void)
{	
	fItemID		= kDlgItemNone;
	fCommandID	= kDlgCmdIdle;

	fFnResult	= fFn (*this);
	return fFnResult;
}


/***********************************************************************
 *
 * FUNCTION:	EmDlgContext::PanelEnter
 *
 * DESCRIPTION:	Call the custom dialog handler to idle.
 *
 * PARAMETERS:	None
 *
 * RETURNED:	The result returned by the dialog handler.
 *
 ***********************************************************************/

EmDlgFnResult EmDlgContext::PanelEnter (void)
{	
	fItemID		= kDlgItemNone;
	fCommandID	= kDlgCmdPanelEnter;

	fFnResult	= fFn (*this);
	return fFnResult;
}


/***********************************************************************
 *
 * FUNCTION:	EmDlgContext::PanelExit
 *
 * DESCRIPTION:	Call the custom dialog handler to idle.
 *
 * PARAMETERS:	None
 *
 * RETURNED:	The result returned by the dialog handler.
 *
 ***********************************************************************/

EmDlgFnResult EmDlgContext::PanelExit (void)
{	
	fItemID		= kDlgItemNone;
	fCommandID	= kDlgCmdPanelExit;

	fFnResult	= fFn (*this);
	return fFnResult;
}


void EmDlg::PrvBuildDeviceMenu (EmDlgContext&)
{
}

void EmDlg::PrvBuildSkinMenu (EmDlgContext&)
{
}

void EmDlg::PrvBuildMemorySizeMenu (EmDlgContext&)
{
}

void EmDlg::PrvSetROMName (EmDlgContext& context)
{
	SessionNewData*	data = (SessionNewData*) context.fUserData;
	EmDlgRef&		dlg = context.fDlg;
	Configuration&	cfg = data->fWorkingCfg;
	EmFileRef&		rom = cfg.fROMFile;

#if PLATFORM_WINDOWS

	if (rom.IsSpecified ())
	{
		SetItemText (dlg, kDlgItemNewROM, rom.GetName());
		EnableItem (dlg, kDlgItemOK);
	}
	else
	{
		DisableItem (dlg, kDlgItemOK);
	}

#else

	if (rom.IsSpecified ())
	{
		SetItemText (dlg, kDlgItemNewROMBrowse, rom.GetName());
		EnableItem (dlg, kDlgItemOK);
	}
	else
	{
		string	browse = Platform::GetString (kStr_Browse);
		SetItemText (dlg, kDlgItemNewROMBrowse, browse);
		DisableItem (dlg, kDlgItemOK);
	}

#endif
}


void PrvConvertBaudListToStrings (const EmTransportSerial::BaudList& baudList,
									StringList& baudStrList)
{
	EmTransportSerial::BaudList::const_iterator	iter = baudList.begin();
	while (iter != baudList.end())
	{
		char	buffer[20];
		::FormatInteger (buffer, *iter);
		strcat (buffer, " bps");
		baudStrList.push_back (buffer);
		++iter;
	}
}


#if 0
	// Template for callback function

EmDlgFnResult EmDlg::PrvCallback (EmDlgContext& context)
{
	EmDlgRef				dlg = context.fDlg;
	<type>*	data = (<type>*) context.fUserData;

	switch (context.fCommandID)
	{
		case kDlgCmdInit:
		{
			break;
		}

		case kDlgCmdIdle:
		{
			break;
		}

		case kDlgCmdItemSelected:
		{
			switch (context.fItemID)
			{
				case kDlgItemOK:
				{
					// Fall thru...
				}

				case kDlgItemCancel:
				{
					return kDlgResultClose;
				}

				default:
					assert (false);
			}

			break;
		}
		
		case kDlgCmdPanelEnter:
		case kDlgCmdPanelExit:
		case kDlgCmdNone:
			assert (false);
			break;
	}

	return kDlgResultContinue;
}
#endif
