/* -*- mode: C++; tab-width: 4 -*- */
/* ===================================================================== *\
	Copyright (c) 1998-2000 Palm, Inc. or its subsidiaries.
	All rights reserved.

	This file is part of the Palm OS Emulator.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
\* ===================================================================== */

#ifndef EmDlg_h
#define EmDlg_h

#include "EmFileRef.h"			// EmFileRefList
#include "EmTransport.h"		// EmTransport
#include "EmulatorTypes.h"		// DatabaseInfoList

#include "Logging.h"			// FOR_EACH_LOG_PREF

class Fl_Widget;
class EmROMTransfer;


enum EmDlgCmdID
{
	kDlgCmdNone					= 0,

	kDlgCmdInit					= 1,
	kDlgCmdIdle					= 2,
	kDlgCmdItemSelected			= 3,
	kDlgCmdPanelEnter			= 4,
	kDlgCmdPanelExit			= 5
};

enum EmDlgFnResult
{
	kDlgResultNone				= 0,

	kDlgResultContinue			= 1,
	kDlgResultClose				= 2
};

enum EmDlgID
{
	kDlgNone					= 0,

	kDlgAboutBox				= 1,
	kDlgSessionNew				= 2,
	kDlgSessionSave				= 3,
	kDlgHordeNew				= 4,
	kDlgDatabaseImportProgress	= 5,
	kDlgDatabaseExport			= 6,
	kDlgROMTransferQuery		= 7,
	kDlgROMTransferProgress		= 8,
	kDlgGremlinControl			= 9,
	kDlgEditPreferences			= 10,
	kDlgEditLogging				= 11,
	kDlgEditDebugging			= 12,
	kDlgEditSkins				= 13,
	kDlgCommonDialog			= 14,
	kDlgSaveBound				= 15
};

enum EmDlgPanelID
{
	kDlgPanelNone				= 0,

	kDlgPanelAbtMain			= 1,
	kDlgPanelAbtWindows			= 2,
	kDlgPanelAbtMac				= 3,
	kDlgPanelAbtUAE				= 4,
	kDlgPanelAbtQNX				= 5
};

#define DLG_BASE(dlgID)	((dlgID) * 100)
enum EmDlgItemID
{
	kDlgItemNone				= 0,

	kDlgItemOK					= 1,
	kDlgItemCancel				= 2,

	kDlgItemAbtAppName			= DLG_BASE(kDlgAboutBox) + 0,
	kDlgItemAbtURLPalmWeb		= DLG_BASE(kDlgAboutBox) + 1,
	kDlgItemAbtURLPalmMail		= DLG_BASE(kDlgAboutBox) + 2,
	kDlgItemAbtURLWindowsWeb	= DLG_BASE(kDlgAboutBox) + 3,
	kDlgItemAbtURLWindowsMail	= DLG_BASE(kDlgAboutBox) + 4,
	kDlgItemAbtURLMacWeb		= DLG_BASE(kDlgAboutBox) + 5,
	kDlgItemAbtURLMacMail		= DLG_BASE(kDlgAboutBox) + 6,
	kDlgItemAbtURLUAEWeb		= DLG_BASE(kDlgAboutBox) + 7,
	kDlgItemAbtURLUAEMail		= DLG_BASE(kDlgAboutBox) + 8,
	kDlgItemAbtURLQNXWeb		= DLG_BASE(kDlgAboutBox) + 0,
	kDlgItemAbtURLQNXMail		= DLG_BASE(kDlgAboutBox) + 10,

	kDlgItemNewDevice			= DLG_BASE(kDlgSessionNew) + 0,
	kDlgItemNewSkin				= DLG_BASE(kDlgSessionNew) + 1,
	kDlgItemNewMemorySize		= DLG_BASE(kDlgSessionNew) + 2,
	kDlgItemNewROM				= DLG_BASE(kDlgSessionNew) + 3,
	kDlgItemNewROMBrowse		= DLG_BASE(kDlgSessionNew) + 4,

	kDlgItemHrdAppList			= DLG_BASE(kDlgHordeNew) + 0,
	kDlgItemHrdStart			= DLG_BASE(kDlgHordeNew) + 1,
	kDlgItemHrdStop				= DLG_BASE(kDlgHordeNew) + 2,
	kDlgItemHrdCheckSwitch		= DLG_BASE(kDlgHordeNew) + 3,
	kDlgItemHrdCheckSave		= DLG_BASE(kDlgHordeNew) + 4,
	kDlgItemHrdCheckStop		= DLG_BASE(kDlgHordeNew) + 5,
	kDlgItemHrdDepthSwitch		= DLG_BASE(kDlgHordeNew) + 6,
	kDlgItemHrdDepthSave		= DLG_BASE(kDlgHordeNew) + 7,
	kDlgItemHrdDepthStop		= DLG_BASE(kDlgHordeNew) + 8,
	kDlgItemHrdLogging			= DLG_BASE(kDlgHordeNew) + 9,

	kDlgItemExpDbList			= 400,

	kDlgItemDlqInstructions		= 500,
	kDlgItemDlqPortList			= 501,
	kDlgItemDlqBaudList			= 502,

	kDlgItemDlpMessage			= 600,
	kDlgItemDlpProgress			= 601,

	kDlgItemLogNormal			= DLG_BASE(kDlgEditLogging) + 0,
	kDlgItemLogGremlins			= DLG_BASE(kDlgEditLogging) + 1,
	kDlgItemLogRadioBase		= DLG_BASE(kDlgEditLogging) + 10,
#define DEFINE_BUTTON_ID(name)	kDlgItemLog##name,
	FOR_EACH_LOG_PREF(DEFINE_BUTTON_ID)
	
	kDlgItemSknSkinList			= DLG_BASE(kDlgEditSkins) + 0,
	kDlgItemSknDoubleScale		= DLG_BASE(kDlgEditSkins) + 1,
	kDlgItemSknWhiteBackground	= DLG_BASE(kDlgEditSkins) + 2,

	kDlgItemBndSaveROM			= DLG_BASE(kDlgSaveBound) + 0,
	kDlgItemBmdSaveRAM			= DLG_BASE(kDlgSaveBound) + 1,

	kDlgItemLast
};

typedef vector<EmDlgItemID>		EmDlgItemIDList;

typedef long					EmDlgItemIndex;
typedef vector<EmDlgItemIndex>	EmDlgItemIndexList;

typedef long					EmDlgListIndex;	// Zero-based
typedef vector<EmDlgListIndex>	EmDlgListIndexList;
const EmDlgListIndex			kDlgItemListNone = -1;

struct EmDlgContext;

typedef EmDlgFnResult			(*EmDlgFn)(EmDlgContext&);
typedef void*					EmDlgRef;

struct EmDlgContext
{
						EmDlgContext (void);
						EmDlgContext (const EmDlgContext&);

	EmDlgFnResult		Init ();
	EmDlgFnResult		Event (EmDlgItemID);
	EmDlgFnResult		Idle ();
	EmDlgFnResult		PanelEnter ();
	EmDlgFnResult		PanelExit ();

	EmDlgFn				fFn;
	EmDlgFnResult		fFnResult;
	EmDlgRef			fDlg;
	EmDlgID				fDlgID;
	EmDlgPanelID		fPanelID;
	EmDlgItemID			fItemID;
	EmDlgCmdID			fCommandID;
	bool				fNeedsIdle;
	void*				fUserData;
};

// !!! Needs to be moved elsewhere
typedef DatabaseInfoList	EmDatabaseList;

class EmDlg
{
	public:
		static EmDlgItemID		DoGetFile					(EmFileRef& result,
															 const string& prompt,
															 const EmDirRef& defaultPath,
															 const EmFileTypeList& filterList);
		static EmDlgItemID		DoGetFileList				(EmFileRefList& results,
															 const string& prompt,
															 const EmDirRef& defaultPath,
															 const EmFileTypeList& filterList);
		static EmDlgItemID		DoPutFile					(EmFileRef& result,
															 const string& prompt,
															 const EmDirRef& defaultPath,
															 const EmFileTypeList& filterList,
															 const string& defName);

		static EmDlgItemID		DoAboutBox					(void);

		static EmDlgItemID		DoSessionNew				(Configuration&);
		static EmDlgItemID		DoSessionSave				(void);

		static EmDlgItemID		DoHordeNew					(void);

#if 0
		static EmDlgItemID		DatabaseImportProgressOpen	(EmDatabaseList&);
		static EmDlgItemID		DatabaseImportProgressUpdate(EmDatabaseList&);
		static EmDlgItemID		DatabaseImportProgressClose	(EmDatabaseList&);

		static EmDlgItemID		DoDatabaseExport			(EmDatabaseList&);
#endif

		static EmDlgItemID		DoROMTransferQuery			(EmTransport::Config*&);
		static EmDlgItemID		DoROMTransferProgress		(EmROMTransfer&);

		static EmDlgItemID		GremlinControlOpen			(void);
		static EmDlgItemID		GremlinControlUpdate		(void);
		static EmDlgItemID		GremlinControlClose			(void);

		static EmDlgItemID		DoEditPreferences			(void);
		static EmDlgItemID		DoEditLoggingOptions		(LoggingType);
		static EmDlgItemID		DoEditDebuggingOptions		(void);
		static EmDlgItemID		DoEditSkins					(void);

		static EmDlgItemID		DoDialog					(void);

		static EmDlgItemID		DoSaveBound					(void);

	private:
		static EmDlgItemID		HostRunGetFile				(EmFileRef& result,
															 const string& prompt,
															 const EmDirRef& defaultPath,
															 const EmFileTypeList& filterList);
		static EmDlgItemID		HostRunGetFileList			(EmFileRefList& results,
															 const string& prompt,
															 const EmDirRef& defaultPath,
															 const EmFileTypeList& filterList);
		static EmDlgItemID		HostRunPutFile				(EmFileRef& result,
															 const string& prompt,
															 const EmDirRef& defaultPath,
															 const EmFileTypeList& filterList,
															 const string& defName);

		static EmDlgItemID		HostRunDialog				(EmDlgFn, void*, EmDlgID);

	public:		// public so that PrvIdleCallback can get to it (hack?)
		static void				HostStartIdling				(EmDlgContext&);
		static void				HostStopIdling				(EmDlgContext&);

	private:
		static EmDlgFnResult	PrvAboutBox					(EmDlgContext&);
		static EmDlgFnResult	PrvSessionNew				(EmDlgContext&);
		static EmDlgFnResult	PrvHordeNew					(EmDlgContext&);
		static EmDlgFnResult	PrvDatabaseExport			(EmDlgContext&);
		static EmDlgFnResult	PrvROMTransferQuery			(EmDlgContext&);
		static EmDlgFnResult	PrvROMTransferProgress		(EmDlgContext&);
		static EmDlgFnResult	PrvEditLoggingOptions		(EmDlgContext&);
		static EmDlgFnResult	PrvEditSkins				(EmDlgContext&);
		static EmDlgFnResult	PrvSaveBound				(EmDlgContext&);

		static void				PrvBuildDeviceMenu			(EmDlgContext&);
		static void				PrvBuildSkinMenu			(EmDlgContext&);
		static void				PrvBuildMemorySizeMenu		(EmDlgContext&);
		static void				PrvSetROMName				(EmDlgContext&);

	public:	// Most of these are really "Host" functions
		static void				SetItemMin					(EmDlgRef, EmDlgItemID, long max);
		static void				SetItemValue				(EmDlgRef, EmDlgItemID, long value);
		static void				SetItemMax					(EmDlgRef, EmDlgItemID, long max);

		static void				SetItemText					(EmDlgRef, EmDlgItemID, StrCode);
		static void				SetItemText					(EmDlgRef, EmDlgItemID, const char*);
		static void				SetItemText					(EmDlgRef, EmDlgItemID, string);

		static long				GetItemMin					(EmDlgRef, EmDlgItemID);
		static long				GetItemValue				(EmDlgRef, EmDlgItemID);
		static long				GetItemMax					(EmDlgRef, EmDlgItemID);
		static string			GetItemText					(EmDlgRef, EmDlgItemID);

		static void				EnableItem					(EmDlgRef, EmDlgItemID);
		static void				DisableItem					(EmDlgRef, EmDlgItemID);

		static void				AppendToMenu				(EmDlgRef, EmDlgItemID, const string&);
		static void				AppendToMenu				(EmDlgRef, EmDlgItemID, const StringList&);

		static void				AppendToList				(EmDlgRef, EmDlgItemID, const string&);
		static void				AppendToList				(EmDlgRef, EmDlgItemID, const StringList&);
		static void				SelectListItem				(EmDlgRef, EmDlgItemID, EmDlgListIndex);
		static void				SelectListItems				(EmDlgRef, EmDlgItemID, const EmDlgListIndexList&);
		static EmDlgListIndex	GetSelectedItem				(EmDlgRef, EmDlgItemID);
		static void				GetSelectedItems			(EmDlgRef, EmDlgItemID, EmDlgListIndexList&);
};

void PrvSetWidgetID (Fl_Widget* o, EmDlgItemID id);		// !!! What's this doing here?  Should be in EmDlgFltk?


#endif	/* EmDlg_h */
