/* -*- mode: C++; tab-width: 4 -*- */
/* ===================================================================== *\
	Copyright (c) 1998-2000 Palm, Inc. or its subsidiaries.
	All rights reserved.

	This file is part of the Palm OS Emulator.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
\* ===================================================================== */

#include "EmCommon.h"
#include "EmPalmFunction.h"

#include "CPU_REG.h"			// LowMem_GetGlobal
#include "Miscellaneous.h"		// FindFunctionName
#include "RAM_ROM.h"			// CEnableFullAccess
#include "Strings.r.h"			// kStr_INetLibTrapBase
#include "UAE_Utils.h"			// uae_strcpy

const UInt16	kMagicRefNum	= 0x666;	// See comments in HtalLibSendReply.

const uint16	kOpcode_ADD 	= 0x0697;	// ADD.L X, (A7)
const uint16	kOpcode_LINK	= 0x4E50;
const uint16	kOpcode_RTE 	= 0x4E73;
const uint16	kOpcode_RTD 	= 0x4E74;
const uint16	kOpcode_RTS 	= 0x4E75;
const uint16	kOpcode_JMP 	= 0x4ED0;

const uint16	kOpcode_CMPW	= 0xB47C;		// CMP.W #$xxxx,D2
const uint16	kOpcode_ADDW	= 0xD442;		// ADD.W D2,D2
const uint32	kOpcode_JMPPC	= 0x4EFB2002;	// JMP *+$0004(D2.W)
const uint16	kOpcode_JMPREL	= 0x4EFA;		// JMP *+$xxxx

// This table should match up with gResourceBases in ErrorHandling.cpp.
static const char* gPalmOSLibraries [] =
{
	"INet.lib",
	"IrDA Library", // Also includes Exchange Lib
	"Security.lib",
	"Web.lib",
//	serIrCommLibNameP,	// ???SerIrCommLib.h doesn't declare the functions as SYSTRAPs!
	"Debugger Comm",
	"IrSerial Library",
	"Net.lib",
	"PADHTAL.lib",
	"Rail I/O Library",
	"RMP NetPlugIn.lib",
	"Serial Lib68681",
	"Serial Library",
	"TCPHTAL.lib",

	NULL
};



class EmFunctionRange
{
	public:
								EmFunctionRange	(const char* functionName);

		Bool					InRange			(emuptr = m68k_getpc ());
		void					Reset			(void);

		Bool					HasRange		(void) { return fBegin != UAE_NULL; }

	private:
		void					GetRange		(emuptr addr);

		const char*				fName;
		emuptr					fBegin;
		emuptr					fEnd;

		EmFunctionRange*		fNext;
		static EmFunctionRange*	fgRoot;
};


	// Note: can't use the full name of PrvMisAlignedBackwardInnerBitBlt, as
	// Debug::GetRoutineName returns only the first 31 characters.

EmFunctionRange*	EmFunctionRange::fgRoot;

static EmFunctionRange	gcj_kptkdelete						("cj_kptkdelete");
static EmFunctionRange	gBackspaceChar						("BackspaceChar");
static EmFunctionRange	gCrc16CalcBlock						("Crc16CalcBlock");
static EmFunctionRange	gDmWrite							("DmWrite");
static EmFunctionRange	gEggOfInfiniteWisdom				("EggOfInfiniteWisdom");
static EmFunctionRange	gFindShowResults					("FindShowResults");
static EmFunctionRange	gFindSaveFindStr					("FindSaveFindStr");
static EmFunctionRange	gFldDelete							("FldDelete");
static EmFunctionRange	gFntDefineFont						("FntDefineFont");
static EmFunctionRange	gGrfProcessStroke					("GrfProcessStroke");
static EmFunctionRange	gMemMove							("MemMove");
static EmFunctionRange	gMenuHandleEvent					("MenuHandleEvent");
static EmFunctionRange	gNetPrvSettingSet					("NetPrvSettingSet");
static EmFunctionRange	gNetPrvTaskMain						("NetPrvTaskMain");
static EmFunctionRange	gPrvCompressedInnerBitBlt			("PrvCompressedInnerBitBlt");
static EmFunctionRange	gPrvConvertDepth1To2BW				("PrvConvertDepth1To2BW");
static EmFunctionRange	gPrvFindMemoryLeaks					("PrvFindMemoryLeaks");
static EmFunctionRange	gPrvMisAlignedForwardInnerBitBlt	("PrvMisAlignedForwardInnerBitBlt");
static EmFunctionRange	gPrvMisAlignedBackwardInnerBitBlt	("PrvMisAlignedBackwardInnerBitBl");
static EmFunctionRange	gPrvSystemTimerProc					("PrvSystemTimerProc");
static EmFunctionRange	gSecPrvRandomSeed					("SecPrvRandomSeed");
static EmFunctionRange	gSysAppExit							("SysAppExit");
static EmFunctionRange	gTsmGlueGetFepMode					("TsmGlueGetFepMode");
static EmFunctionRange	gTsmGlueSetFepMode					("TsmGlueSetFepMode");

static EmFunctionRange	gHsPrvInit							("HsPrvInit");

Bool	Incj_kptkdelete						(emuptr addr)	{ return gcj_kptkdelete.InRange (addr); }
Bool	InBackspaceChar						(emuptr addr)	{ return gBackspaceChar.InRange (addr); }
Bool	InCrc16CalcBlock					(emuptr addr)	{ return gCrc16CalcBlock.InRange (addr); }
Bool	InDmWrite							(emuptr addr)	{ return gDmWrite.InRange (addr); }
Bool	InEggOfInfiniteWisdom				(emuptr addr)	{ return gEggOfInfiniteWisdom.InRange (addr); }
Bool	InFindShowResults					(emuptr addr)	{ return gFindShowResults.InRange (addr); }
Bool	InFindSaveFindStr					(emuptr addr)	{ return gFindSaveFindStr.InRange (addr); }
Bool	InFldDelete							(emuptr addr)	{ return gFldDelete.InRange (addr); }
Bool	InFntDefineFont						(emuptr addr)	{ return gFntDefineFont.InRange (addr); }
Bool	InGrfProcessStroke					(emuptr addr)	{ return gGrfProcessStroke.InRange (addr); }
Bool	InMemMove							(emuptr addr)	{ return gMemMove.InRange (addr); }
Bool	InMenuHandleEvent					(emuptr addr)	{ return gMenuHandleEvent.InRange (addr); }
Bool	InNetPrvSettingSet					(emuptr addr)	{ return gNetPrvSettingSet.InRange (addr); }
Bool	InNetPrvTaskMain					(emuptr addr)	{ return gNetPrvTaskMain.InRange (addr); }
Bool	InPrvCompressedInnerBitBlt			(emuptr addr)	{ return gPrvCompressedInnerBitBlt.InRange (addr); }
Bool	InPrvConvertDepth1To2BW				(emuptr addr)	{ return gPrvConvertDepth1To2BW.InRange (addr); }
Bool	InPrvFindMemoryLeaks				(emuptr addr)	{ return gPrvFindMemoryLeaks.InRange (addr); }
Bool	InPrvMisAlignedForwardInnerBitBlt	(emuptr addr)	{ return gPrvMisAlignedForwardInnerBitBlt.InRange (addr); }
Bool	InPrvMisAlignedBackwardInnerBitBlt	(emuptr addr)	{ return gPrvMisAlignedBackwardInnerBitBlt.InRange (addr); }
Bool	InPrvSystemTimerProc				(emuptr addr)	{ return gPrvSystemTimerProc.InRange (addr); }
Bool	InSecPrvRandomSeed					(emuptr addr)	{ return gSecPrvRandomSeed.InRange (addr); }
Bool	InSysAppExit						(emuptr addr)	{ return gSysAppExit.InRange (addr); }
Bool	InTsmGlueGetFepMode					(emuptr addr)	{ return gTsmGlueGetFepMode.InRange (addr); }
Bool	InTsmGlueSetFepMode					(emuptr addr)	{ return gTsmGlueSetFepMode.InRange (addr); }

Bool	InHsPrvInit							(emuptr addr)	{ return gHsPrvInit.InRange (addr); }


/***********************************************************************
 *
 * FUNCTION:	EmFunctionRangeInit
 *
 * DESCRIPTION:	Reset all our objects that track where certain Palm
 *				functions exist in memory.
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

void EmPalmFunctionInit (void)
{
	gcj_kptkdelete						.Reset ();
	gBackspaceChar						.Reset ();
	gCrc16CalcBlock						.Reset ();
	gDmWrite							.Reset ();
	gEggOfInfiniteWisdom				.Reset ();
	gFindShowResults					.Reset ();
	gFindSaveFindStr					.Reset ();
	gFldDelete							.Reset ();
	gFntDefineFont						.Reset ();
	gGrfProcessStroke					.Reset ();
	gMemMove							.Reset ();
	gMenuHandleEvent					.Reset ();
	gNetPrvSettingSet					.Reset ();
	gNetPrvTaskMain						.Reset ();
	gPrvCompressedInnerBitBlt			.Reset ();
	gPrvConvertDepth1To2BW				.Reset ();
	gPrvFindMemoryLeaks					.Reset ();
	gPrvMisAlignedForwardInnerBitBlt	.Reset ();
	gPrvMisAlignedBackwardInnerBitBlt	.Reset ();
	gPrvSystemTimerProc					.Reset ();
	gSecPrvRandomSeed					.Reset ();
	gSysAppExit							.Reset ();
	gTsmGlueGetFepMode					.Reset ();
	gTsmGlueSetFepMode					.Reset ();

	gHsPrvInit							.Reset ();
}


/***********************************************************************
 *
 * FUNCTION:	EmFunctionRange::EmFunctionRange
 *
 * DESCRIPTION:	Create the object.  Initialize all data members.
 *
 * PARAMETERS:	functionName - name of function to track.  Storage is
 *					not owned by this object, so client should make
 *					sure it exists for the life of this object.  The
 *					function name is typically provided as a string
 *					constant in global space.
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

EmFunctionRange::EmFunctionRange (const char* functionName) :
	fName (functionName),
	fBegin (UAE_NULL),
	fEnd (UAE_NULL),
	fNext (fgRoot)
{
	fgRoot = this;
}


/***********************************************************************
 *
 * FUNCTION:	EmFunctionRange::InRange
 *
 * DESCRIPTION:	Determine whether or not the given address is within
 *				the function managed by this object.
 *
 * PARAMETERS:	addr - address to test.
 *
 * RETURNED:	True if so.
 *
 ***********************************************************************/

Bool EmFunctionRange::InRange (emuptr addr)
{
	// It's not in our range if it's in someone else's.
	// This is an optimization.

	static Bool	walkingChain;

	if (!walkingChain)
	{
		walkingChain = true;

		EmFunctionRange*	range = fgRoot;
		while (range)
		{
			if (range != this && range->HasRange () && range->InRange (addr))
			{
				walkingChain = false;
				return false;
			}

			range = range->fNext;
		}

		walkingChain = false;
	}

	// If we haven't determine the range of the function for which we're
	// responsible, see if given address is in that function.  If so,
	// remember the range of that function.

	if (!this->HasRange ())
	{
		this->GetRange (addr);
	}

	// If we know the range of the function for which we are responsible,
	// test the address against that range.

	if (this->HasRange () && (addr >= fBegin) && (addr < fEnd))
	{
		return true;
	}

	return false;
}


/***********************************************************************
 *
 * FUNCTION:	EmFunctionRange::Reset
 *
 * DESCRIPTION:	If we've determined and remembered where the managed
 *				function exists, forget that information.  This method
 *				is typically called so that this object can be reused
 *				after a new ROM image has been loaded.
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

void EmFunctionRange::Reset (void)
{
	fBegin = UAE_NULL;
	fEnd = UAE_NULL;
}


/***********************************************************************
 *
 * FUNCTION:	EmFunctionRange::GetRange
 *
 * DESCRIPTION:	Find the name of the function containing the given
 *				address.  If it's our managed function, remember the
 *				range of the function.
 *
 * PARAMETERS:	addr - probe address.
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

void EmFunctionRange::GetRange (emuptr addr)
{
	char		name[80];
	emuptr		startAddr;
	emuptr		endAddr;

	::FindFunctionName (addr, name, &startAddr, &endAddr);

	if (strcmp (name, fName) == 0)
	{
		fBegin = startAddr;
		fEnd = endAddr;
	}
}


/***********************************************************************
 *
 * FUNCTION:	GetFunctionAddress
 *
 * DESCRIPTION: Determines the address of a system function,
 *
 * PARAMETERS:	trapWord - the dispatch number used to invoke the
 *					function(the number after the TRAP $F opocde)
 *
 *				extra - an optional extra parameter.  This parameter can
 *					be something like a library reference number, or a
 *					register to be used in a sub-dispatcher.
 *
 *				digDeep - a boolean saying how hard we should look for
 *					the function address.  If false, we just return the
 *					address generated by the ROM TrapDispatcher
 *					function.  Otherwise, for certain special ROM
 *					routines (like the Floating Point Manager entry
 *					point), we look deeper.
 *
 * RETURNED:	The function's address, or UAE_NULL if it could not
 *				be determined.
 *
 ***********************************************************************/

emuptr GetFunctionAddress (uint16 trapWord, uint32 extra, Bool digDeep)
{
	// Ensure that it's in Canonical format.

	trapWord = SysTrapIndex (trapWord) | sysTrapBase;

	// If it's a regular system function (0xA000-0xA7FFF), handle it.

	if (IsSystemTrap (trapWord))
		return GetSysFunctionAddress (trapWord, extra, digDeep);

	// Assume it's a library function call.

	return GetLibFunctionAddress (trapWord, (UInt16) extra, digDeep);
}


/***********************************************************************
 *
 * FUNCTION:	GetLibFunctionAddress
 *
 * DESCRIPTION: Determines the address of a library function,
 *
 * PARAMETERS:	trapWord - the dispatch number used to invoke the
 *					function(the number after the TRAP $F opocde)
 *
 *				extra - an optional extra parameter.  This parameter can
 *					be something like a library reference number, or a
 *					register to be used in a sub-dispatcher.
 *
 *				digDeep - a boolean saying how hard we should look for
 *					the function address.  If false, we just return the
 *					address generated by the ROM TrapDispatcher
 *					function.  Otherwise, for certain special ROM
 *					routines (like the Floating Point Manager entry
 *					point), we look deeper.
 *
 * RETURNED:	The function's address, or UAE_NULL if it could not
 *				be determined.
 *
 ***********************************************************************/

emuptr GetLibFunctionAddress (uint16 trapWord, UInt16 refNum, Bool digDeep)
{
	if (refNum == sysInvalidRefNum || refNum == 0)
		throw EmInvalidRefNumException (0);

	CEnableFullAccess	munge;	// Remove blocks on memory access.

	/*
		The System Library Table (sysLibTableP) is an array of
		sysLibTableEntries entries.  Each entry has the following
		format:

			Ptr*		dispatchTblP;	// pointer to library dispatch table
			void*		globalsP;		// Library globals
			LocalID 	dbID;			// database id of the library
			VoidPtr 	codeRscH;		// library code resource handle for RAM-based libraries

		The latter two fields are present only in Palm OS 2.0 and
		later.	So our first steps are to (a) get the pointer to 
		the array, (b) make sure that the index into the array (the
		refNum passed as the first parameter to all library calls)
		is within range, (c) get a pointer to the right entry,
		taking into account the Palm OS version, and (d) getting the
		dispatchTblP field.

		The "library dispatch table" is an array of 16-bit offsets.  The
		values are all relative to the beginning of the table (dispatchTblP).
		The first entry in the array corresponds to the library name.  All
		subsequent entries are offsets to the various library functions,
		starting with the required four: sysLibTrapOpen, sysLibTrapClose,
		sysLibTrapSleep, and sysLibTrapWake.
	*/

	uint32	sysLibTableP		= LowMem_GetGlobal (sysLibTableP);
	UInt16	sysLibTableEntries	= LowMem_GetGlobal (sysLibTableEntries);

	if (sysLibTableP == UAE_NULL)
	{
		// !!! No library table!
		assert (false);
		return UAE_NULL;
	}

	if (refNum >= sysLibTableEntries)
	{
		// See comments in HtalLibHeadpatch::HtalLibSendReply.
		if (refNum == kMagicRefNum)
		{
			return 1;
		}

		// !!! RefNum out of range!
		assert (false);
		throw EmInvalidRefNumException (0, sysLibTableEntries);
	}

	emuptr libEntry;
	emuptr dispatchTblP;

	if (Patches::OSMajorVersion () > 1)
	{
		libEntry		= sysLibTableP + refNum * sizeof (SysLibTblEntryType);
		dispatchTblP	= get_long (libEntry + offsetof (SysLibTblEntryType, dispatchTblP));
	}
	else
	{
		libEntry		= sysLibTableP + refNum * sizeof (SysLibTblEntryTypeV10);
		dispatchTblP	= get_long (libEntry + offsetof (SysLibTblEntryTypeV10, dispatchTblP));
	}

	// Validate the dispatch number.  See if the library is one that
	// we know about.  If so, compare the dispatch number against
	// the list of known dispatch number limits.
	
	// The first entry in the table is always the offset from the
	// start of the table to the library name.	Use this information
	// get the library name.

	int16 	offset = get_word (dispatchTblP + LibTrapIndex (sysLibTrapName) * 2);
	emuptr 	libNameP = dispatchTblP + offset;
	char	libName[256];
	uae_strcpy (libName, libNameP);

	// Iterate over our list of Palm OS-supplied libraries to see
	// if this is one of them.	If so, the library is known to follow
	// a format that allows us to determine the size of the table.

	const char**	knownLibNameP = gPalmOSLibraries;

	while (*knownLibNameP)
	{
		if (strcmp (*knownLibNameP, libName) != 0)
		{
			knownLibNameP++;
			continue;
		}

		// OK, it's one of our libraries.  Now get the *second*
		// entry in the table.	This is the offset from the start
		// of the table to the entry point for the standard
		// "library open" function.  All Palm OS libraries follow
		// the convention where this offset points to a JMP
		// instruction immediately following the function table.
		// We can use this information to calculate the size of
		// the table.

		offset = get_word (dispatchTblP + LibTrapIndex (sysLibTrapOpen) * 2);

		if (LibTrapIndex (trapWord) < (offset / 2))
		{
			// It's a known library, and the offset is in range,
			// so break out of here so that we can get one with
			// actually calling the function.
			break;
		}

		// It's a known library, but the offset is out of range,
		// so report a problem.

		throw EmUnimplementedFunctionException (knownLibNameP - gPalmOSLibraries);
	}

	// Either we didn't know about that library, or we did and the
	// version test passed.  So go ahead and jump.

	offset = get_word (dispatchTblP + LibTrapIndex (trapWord) * 2);
	emuptr result = dispatchTblP + offset;

	if (digDeep && get_word (result) == kOpcode_JMPREL)
	{
		result += 2;
		result += (int16) get_word (result);
	}

	return result;
}


/***********************************************************************
 *
 * FUNCTION:	GetSysFunctionAddress
 *
 * DESCRIPTION: Determines the address of a non-library function,
 *
 * PARAMETERS:	trapWord - the dispatch number used to invoke the
 *					function(the number after the TRAP $F opocde)
 *
 *				extra - an optional extra parameter.  This parameter can
 *					be something like a library reference number, or a
 *					register to be used in a sub-dispatcher.
 *
 *				digDeep - a boolean saying how hard we should look for
 *					the function address.  If false, we just return the
 *					address generated by the ROM TrapDispatcher
 *					function.  Otherwise, for certain special ROM
 *					routines (like the Floating Point Manager entry
 *					point), we look deeper.
 *
 * RETURNED:	The function's address, or UAE_NULL if it could not
 *				be determined.
 *
 ***********************************************************************/

emuptr GetSysFunctionAddress (uint16 trapWord, uint32 extra, Bool digDeep)
{
	emuptr unimplementedAddress	= LowMem::GetTrapAddress (sysTrapSysUnimplemented);
	emuptr result				= LowMem::GetTrapAddress (trapWord);

	if (result == UAE_NULL ||
		((result == unimplementedAddress) && (trapWord != sysTrapSysUnimplemented)))
	{
		// sysTrapHostControl is always implemented, so make sure we
		// don't return a NULL address.

		if (SysTrapIndex (trapWord) == SysTrapIndex (sysTrapHostControl))
		{
			return 0x12345678;
		}

		throw EmUnimplementedFunctionException ();
	}

	if (digDeep)
	{
		emuptr result2 = UAE_NULL;

		if (trapWord == sysTrapIntlDispatch 	||
			trapWord == sysTrapOmDispatch		||
			trapWord == sysTrapTsmDispatch		||
			trapWord == sysTrapFlpDispatch 		||
			trapWord == sysTrapSerialDispatch)
		{
			result2 = GetStdDispatchAddress (result, extra);
		}

		else if (trapWord == sysTrapFlpEmDispatch)
		{
			result2 = GetFlpEmDispatchAddress (result, extra);
		}

		if (result2)
			result = result2;
	}

	if (!result)
		throw EmUnimplementedFunctionException ();

	return result;
}


/***********************************************************************
 *
 * FUNCTION:	GetStdDispatchAddress
 *
 * DESCRIPTION: Determine the address of a function that is reached
 *				via a sub-dispatcher.
 *
 * PARAMETERS:	entryPt - the entry point of the sub-dispatch function.
 *
 *				regD2 - sub-dispatch selector.
 *
 * RETURNED:	The function's address, or UAE_NULL if it could not
 *				be determined.
 *
 ***********************************************************************/

emuptr GetStdDispatchAddress (emuptr entryPt, uint32 regD2)
{
	CEnableFullAccess	munge;	// Remove blocks on memory access.

	/*
		The standard dispatch routine code looks like this...

			+$0000	10D5D7FE  *CMP.W	 #$0018,D2		| B47C 0018
			+$0004	10D5D802   BHI.S	 @error 		| 626C
			+$0006	10D5D804   ADD.W	 D2,D2			| D442
			+$0008	10D5D806   ADD.W	 D2,D2			| D442
			+$000A	10D5D808   JMP		 *+$0004(D2.W)	| 4EFB 2002

			+$000E	10D5D80C   JMP		 function1		| 4EFA F732
			+$0012	10D5D810   JMP		 function2		| 4EFA 0AB8
			...
	*/

	// check for expected opcodes

	if (get_word (entryPt + 0x00) != kOpcode_CMPW ||
		get_word (entryPt + 0x06) != kOpcode_ADDW ||
		get_word (entryPt + 0x08) != kOpcode_ADDW ||
		get_long (entryPt + 0x0A) != kOpcode_JMPPC)
	{
		return UAE_NULL;
	}

	// load the max selector value, and compare to D2.W

	uint16 maxSelector = get_word (entryPt + 2);

	if ((regD2 & 0x0000FFFF) > maxSelector)
	{
		return UAE_NULL;
	}

	// point to appropriate JMP xxx instruction.

	emuptr result = entryPt + 14 + (regD2 * 4);

	if (get_word (result) != kOpcode_JMPREL)
	{
		return UAE_NULL;
	}

	result += 2;
	result += (int16) get_word (result);

	return result;
}


/***********************************************************************
 *
 * FUNCTION:	GetFlpEmDispatchAddress
 *
 * DESCRIPTION: Determine the address of a function that is reached
 *				via a sub-dispatcher.
 *
 * PARAMETERS:	entryPt - the entry point of the sub-dispatch function.
 *
 *				regD2 - sub-dispatch selector.
 *
 * RETURNED:	The function's address, or UAE_NULL if it could not
 *				be determined.
 *
 ***********************************************************************/

emuptr GetFlpEmDispatchAddress (emuptr entryPt, uint32 regD2)
{
	CEnableFullAccess	munge;	// Remove blocks on memory access.

	/*
		The FlpEm dispatch routine code looks like this...

			+$0000	10D5D804   ADD.W	 D2,D2				| D442
			+$0002	10D5D806   ADD.W	 D2,D2				| D442
			+$0004	10D5D808   JMP		 *+$0004(D2.W)		| 4EFB 2002

			+$0008	10D5D80C   JMP		 _fp_round			| 4EFA F732
			+$000A	10D5D810   JMP		 _fp_get_fpscr		| 4EFA 0AB8
			...
	*/

	// check for expected opcodes

	if (get_word (entryPt + 0x00) != kOpcode_ADDW ||
		get_word (entryPt + 0x02) != kOpcode_ADDW ||
		get_long (entryPt + 0x04) != kOpcode_JMPPC)
	{
		return UAE_NULL;
	}

	// point to appropriate JMP xxx instruction.

	emuptr result = entryPt + 8 + (regD2 * 4);

	if (get_word (result) != kOpcode_JMPREL)
	{
		return UAE_NULL;
	}

	result += 2;
	result += (int16) get_word (result);

	return result;
}


/***********************************************************************
 *
 * FUNCTION:	GetTrapName
 *
 * DESCRIPTION: Returns a pointer to a string appropriate for
 *				identifying a function.  If the function has a Macsbug
 *				name, that name is returned.  If the function is a
 *				known Palm OS function, that name is looked up in a
 *				string resource list.  Otherwise, a default "unknown"
 *				string is returned.
 *
 * PARAMETERS:	context - structure describing the function.
 *
 *				trapWord - the dispatch number used to invoke the
 *					function(the number after the TRAP $F opocde).
 *
 *				extra - an optional extra parameter.  This parameter can
 *					be something like a library reference number, or a
 *					register to be used in a sub-dispatcher.
 *
 *				digDeep - a boolean saying how hard we should look for
 *					the function address.  If false, we just return the
 *					address generated by the ROM TrapDispatcher
 *					function.  Otherwise, for certain special ROM
 *					routines (like the Floating Point Manager entry
 *					point), we look deeper.
 *
 * RETURNED:	nothing
 *
 ***********************************************************************/

char* GetTrapName (const SystemCallContext& context, Bool digDeep)
{
	return GetTrapName (context.fTrapWord, context.fExtra, digDeep);
}


char* GetTrapName (uint16 trapWord, uint32 extra, Bool digDeep)
{
	static char name[sysPktMaxNameLen];

	try
	{
		::FindTrapName (trapWord, name, extra, digDeep);
	}
	catch (...)
	{
		strcpy (name, ">>> Unknown function name <<<");

		if (IsSystemTrap (trapWord))
		{
			string	knownName = Platform::GetString (kStr_SysTrapBase + SysTrapIndex (trapWord));
			if (knownName[0] != '<')
			{
				strcpy (name, knownName.c_str ());
			}
		}
	}

	return name;
}


/***********************************************************************
 *
 * FUNCTION:	FindTrapName
 *
 * DESCRIPTION: Finds the Macsbug name for the given function.	If the
 *				name cannot be found, an empty string is returned.
 *
 * PARAMETERS:	trapWord - the dispatch number used to invoke the
 *					function(the number after the TRAP $F opocde)
 *
 *				nameP - buffer to receive the function name.
 *
 *				extra - an optional extra parameter.  This parameter can
 *					be something like a library reference number, or a
 *					register to be used in a sub-dispatcher.
 *
 *				digDeep - a boolean saying how hard we should look for
 *					the function address.  If false, we just return the
 *					address generated by the ROM TrapDispatcher
 *					function.  Otherwise, for certain special ROM
 *					routines (like the Floating Point Manager entry
 *					point), we look deeper.
 *
 * RETURNED:	nothing
 *
 ***********************************************************************/

void FindTrapName (uint16 trapWord, char* nameP, uint32 extra, Bool digDeep)
{
	emuptr addr = ::GetFunctionAddress (trapWord, extra, digDeep);

	if (addr)
		::FindFunctionName (addr, nameP);
	else
		nameP[0] = '\0';
}


/***********************************************************************
 *
 * FUNCTION:	FindFunctionName
 *
 * DESCRIPTION: Returns information about the function containing the
 *				the given memory address, including the function's
 *				start (the LINK instruction, usually), the function's
 *				end (just after the RTS, usually), and the function's
 *				name (that follows the function).
 *
 * PARAMETERS:	addr - address contained within the function.
 *
 *				nameP - storage for the returned function name.  Must
 *					be at least 32 characters.
 *
 *				startAddrP - storage for the returned function start.
 *
 *				endAddrP - storage for the returned function end.
 *
 *				nameCapacity - bytes of storage available at nameP
 *
 * RETURNED:	nothing
 *
 ***********************************************************************/
 
// Llamagraphics, Inc:	Added nameCapacity argument so that callers
// can retrieve more than 31 character function names.	The default
// capacity is 32, so callers that don't provide the extra argument
// will get the same result as before.

void FindFunctionName (emuptr addr, char* nameP, 
			emuptr* startAddrP, emuptr* endAddrP,
			long nameCapacity)
{
	// Get the start address only if requested.

	if (startAddrP)
		*startAddrP = ::FindFunctionStart (addr);

	// Get the end address if requested or if we need it to
	// get the Macsbug name.

	if (nameP || endAddrP)
	{
		emuptr endAddr = ::FindFunctionEnd (addr);

		// Return the end address if requested.

		if (endAddrP)
			*endAddrP = endAddr;

		// Return the Macsbug name if requested.

		if (nameP)
			::CopyMacsbugName (endAddr, nameP, nameCapacity);
	}
}


/***********************************************************************
 *
 * FUNCTION:	FindFunctionStart
 *
 * DESCRIPTION: Find the start of the function containing the given
 *				address.  The start of the function is determined
 *				by looking backwards for the end of the previous
 *				function and then scooting forward to the beginning
 *				of this function.
 *
 * PARAMETERS:	addr - the probe address.
 *
 * RETURNED:	Start of the function.  UAE_NULL if not found.
 *
 ***********************************************************************/

emuptr FindFunctionStart (emuptr addr)
{
	long	maxLength = 0x02000;	// !!! TBD, get length of contaning chunk.

	while (--maxLength)
	{
		addr -= 2;

		// Make sure the address is valid
		if (!valid_address (addr, 2))
		{
			return UAE_NULL;
		}

		if (EndOfFunctionSequence (addr))
		{
			addr += 2;	// skip past the final RTS (or whatever).

			// Skip over the Macsbug name.
			Bool	isFixed;
			uint8	len = MacsbugNameLength (addr, &isFixed);

			addr += len;

			// Make sure the final address is word-aligned.
			if ((addr & 1) != 0)
				addr += 1;

			// If in variable format, the name is followed by
			// a word indicating the size of a chunk of "local data".
			if (!isFixed)
			{
				len = get_word (addr);
				addr += 2 + len;
			}

			return addr;
		}
	}

	return UAE_NULL;
}


/***********************************************************************
 *
 * FUNCTION:	FindFunctionEnd
 *
 * DESCRIPTION: Find the end of the function containing the given
 *				address.  The end of the function is determined by
 *				testing the given address for an end-of-function
 *				sequence, and then scooting forward by two bytes
 *				if the test fails.
 *
 * PARAMETERS:	addr - the probe address.
 *
 * RETURNED:	End of the function.  UAE_NULL if not found.
 *
 ***********************************************************************/

emuptr FindFunctionEnd (emuptr addr)
{
	long	maxLength = 0x02000;	// !!! TBD, get length of contaning chunk.

	while (--maxLength)
	{
		// Make sure the address is valid
		if (!valid_address (addr, 2))
		{
			return UAE_NULL;
		}

		if (EndOfFunctionSequence (addr))
		{
			return addr + 2;
		}

		addr += 2;
	}

	return UAE_NULL;
}


/***********************************************************************
 *
 * FUNCTION:	EndOfFunctionSequence
 *
 * DESCRIPTION: Test the given memory location to see if it contains
 *				a valid-looking end of function sequence.  A valid
 *				end of function sequence is an RTE, JMP (A0), RTD, or
 *				RTS (as long as it doesn't look like the sequnce
 *				CodeWarrior uses for 32-bit jumps).
 *
 * PARAMETERS:	addr - memory location to test.
 *
 * RETURNED:	True if it looks like this is an end of function
 *				sequence.
 *
 ***********************************************************************/

Bool EndOfFunctionSequence (emuptr addr)
{
	uint16 opcode = get_word (addr);

	// Look for the special 32-bit relative jumps which
	// CodeWarrior puts in. These are of the following form:
	//
	// +$0022  10CD3C2A   PEA		*+$0010 		; 10CD3C3A		| 487A 000E
	// +$0026  10CD3C2E   PEA		*+$0006 		; 10CD3C34		| 487A 0004
	// +$002A  10CD3C32   ADDI.L	#$00000CD2,(A7) ; '....'		| 0697 0000 0CD2
	// +$0030  10CD3C38   RTS										| 4E75

	if (opcode == kOpcode_RTS &&
		get_word (addr - 6) != kOpcode_ADD)
	{
		return true;
	}

	if (opcode == kOpcode_RTE ||
		opcode == kOpcode_JMP ||	// JMP (A0)
		opcode == kOpcode_RTD)
	{
		return true;
	}

	return false;
}


/***********************************************************************
 *
 * FUNCTION:	CopyMacsbugName
 *
 * DESCRIPTION: Return the Macsbug name at the given address.
 *
 * PARAMETERS:	eof - pointer to the end of the function (that is, the
 *					byte after the eof sequence).
 *				name - pointer to buffer to receive the name.
 *				nameCapacity - size of buffer pointed to by "name".
 *
 * RETURNED:	nothing
 *
 ***********************************************************************/

// Llamagraphics, Inc:	Added nameCapacity argument so that callers
// can retrieve more than 31 character function names.	The default
// capacity is 32, so callers that don't provide the extra argument
// will get the same result as before.

void CopyMacsbugName (emuptr eof, char* name, long nameCapacity)
{
	// Macsbug names follow the end of the function.

	// Get the length of the macsbug symbol.

	uint8	length = MacsbugNameLength (eof, NULL);

	if (length > nameCapacity - 1)
		length = nameCapacity - 1;

	// copy the string

	char*	dest = name;
	while (length--)
	{
		uint8	ch = get_byte (eof++);

		if (!ValidMacsbugChar (ch))
		{
			// All bets are off.
			*name = 0;
			return;
		}

		*dest++ = (char) ch;
	}

	// Terminate the string.

	*dest = 0;
}


/***********************************************************************
 *
 * FUNCTION:	MascsbugNameLength
 *
 * DESCRIPTION: Returns the length of the Macsbug symbol starting at
 *				the given memory location.	Also bumps "addr" to skip
 *				over the length byte or bytes.
 *
 * PARAMETERS:	addr - start of the Macsbug symbol.  Should contain
 *					the symbols length byte or bytes.
 *
 *				isFixed - receives a boolean indicating if this name
 *					is in "fixed" or "variable" format.  If the former,
 *					the name is padded with spaces which need to be
 *					accounted for.	If the latter, spaces are not
 *					allowed at all.  Also, variable-length names are
 *					followed by a word containing the length of a
 *					chunk of "local data".
 *
 * RETURNED:	The symbol length (zero on error).
 *
 ***********************************************************************/

uint8 MacsbugNameLength (emuptr& addr, Bool* isFixed)
{
	// The Macsbug name can be in one of three forms:
	//
	// Variable length: The first byte is in the range $80 to $9F and is a length in the
	//					range 0 to $1F. The high order bit must be set. A length of 0
	//					implies the second byte is the actual length in the range $01 
	//					thru $FF. The length byte(s) and name may be an odd number of
	//					bytes. However, the data after the name is word aligned.
	//
	// Fixed length 8:	The first byte is in the range $20 to $7F and is an ASCII character.
	//					The high order bit may be set but is not required to be.
	//
	// Fixed length 16: The first byte is in the range $20 to $7F and is an ASCII character.
	//					The high order bit may be set but is not required to be.
	//					The high order bit in the second byte is required to be set.
	//					This distinguishes the two types of fixed names.

	uint8	length = get_byte (addr);

	if (length < 0x20)
	{
		length = 0;

		if (isFixed)
			*isFixed = true;
	}
	else if (length >= 0x80 && length < 0xA0)
	{
		++addr;

		length &= 0x7F;

		if (length == 0)
			length = get_byte (addr++);

		if (isFixed)
			*isFixed = false;
	}
	else
	{
		length &= 0x7F;

		if (get_byte (addr + 1) < 0x80)
			length = 8;
		else
			length = 16;

		if (isFixed)
			*isFixed = true;
	}

	return length;
}


/***********************************************************************
 *
 * FUNCTION:	ValidMacsbugChar
 *
 * DESCRIPTION: Returns whether or not the given character is a
 *				character in a valid Macsbug symbol.  Valid characters
 *				are: [a-zA-Z0-9_%.]
 *
 * PARAMETERS:	ch - the character to test
 *
 * RETURNED:	True if the character is valid.
 *
 ***********************************************************************/

Bool ValidMacsbugChar (uint8 ch)
{
	static Bool initialized = false;
	static Bool validChar[256];

	if (!initialized)
	{
		initialized = true;

		memset (validChar, false, sizeof (validChar));

		validChar ['_'] = true;
		validChar ['%'] = true;
		validChar ['.'] = true;

		uint8	ii;

		for (ii = 'a'; ii <= 'z'; ++ii)
			validChar [ii] = true;

		for (ii = 'A'; ii <= 'Z'; ++ii)
			validChar [ii] = true;

		for (ii = '0'; ii <= '9'; ++ii)
			validChar [ii] = true;
	}

	return validChar [ch];
}


