/* -*- mode: C++; tab-width: 4 -*- */
/* ===================================================================== *\
	Copyright (c) 1998-2000 Palm, Inc. or its subsidiaries.
	All rights reserved.

	This file is part of the Palm OS Emulator.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
\* ===================================================================== */

// This file contains the macros that define the fields of structs in a
// way that allows us to create wrapper classes that access those fields
// in a binary-compatible, cross-platform fashion.
//
// This file is #included by EmPalmStructs.h.  The macros are stored in
// this separate .i file merely because of problems trying to edit them
// if they're stored in EmPalmStructs.h.  Visual Studio's editor seems
// to get bogged down when trying to edit a source file containing many
// large macros.  It frequently "goes to lunch" for a second or so,
// making editing, selecting, scrolling, etc., very frustrating.  Placing
// the macros in a .i file -- which the editor doesn't seem to try to
// parse -- eliminates this problem.


// Define macros that allow us to iterate over all the fields in a struct.
// The various columns are:
//
//		1:	field offset
//		2:	struct type
//		3:	field type
//		4:	field name (for offsetof macro)
//		5:	field name (for generation of C++ symbols -- functions, vars, etc.)


// ========================================
// ===== BreakpointType =====
// ========================================

#define FOR_BreakpointType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(BreakpointType, 6, FOR_BreakpointType_FIELDS)

#define FOR_BreakpointType_FIELDS(DO_TO_FIELD)												\
	DO_TO_FIELD (0,		BreakpointType,			MemPtr,			addr,			addr)		\
	DO_TO_FIELD (4,		BreakpointType,			Boolean,		enabled,		enabled)	\
	DO_TO_FIELD (5,		BreakpointType,			Boolean,		installed,		installed)	\


// ========================================
// ===== ctlEnter =====
// ========================================

#define FOR_ctlEnter_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(ctlEnter, 6, FOR_ctlEnter_FIELDS)

#define FOR_ctlEnter_FIELDS(DO_TO_FIELD)																\
	DO_TO_FIELD (8,		EventType,				Int16,			data.ctlEnter.controlID,	controlID)	\
	DO_TO_FIELD (10,	EventType,				voidPtr,		data.ctlEnter.pControl,		pControl)	\

typedef char	ctlEnter[6];


// ========================================
// ===== EventTypeData =====
// ========================================

#define FOR_EventTypeData_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(EventTypeData, 6, FOR_EventTypeData_FIELDS)

#define FOR_EventTypeData_FIELDS(DO_TO_FIELD)												\
	DO_TO_FIELD (8,		EventType,				ctlEnter,		data.ctlEnter,	ctlEnter)	\

typedef char	EventTypeData[6];	


// ========================================
// ===== EventType =====
// ========================================

#define FOR_EventType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(EventType, 24, FOR_EventType_FIELDS)

#define FOR_EventType_FIELDS(DO_TO_FIELD)													\
	DO_TO_FIELD (0,		EventType,				Int16,			eType,			eType)		\
	DO_TO_FIELD (2,		EventType,				Boolean,		penDown,		penDown)	\
	DO_TO_FIELD (3,		EventType,				UInt8,			tapCount,		tapCount)	\
	DO_TO_FIELD (4,		EventType,				Int16,			screenX,		screenX)	\
	DO_TO_FIELD (6,		EventType,				Int16,			screenY,		screenY)	\
	DO_TO_FIELD (8,		EventType,				EventTypeData,	data,			data)		\


// ========================================
// ===== FixedGlobalsType =====
// ========================================

#undef memCardInfoP

#define FOR_FixedGlobalsType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(FixedGlobalsType, 572, FOR_FixedGlobalsType_FIELDS)

#define FOR_FixedGlobalsType_FIELDS(DO_TO_FIELD)																		\
	DO_TO_FIELD (0,		FixedGlobalsType,		UInt8,			memCardSlots,			memCardSlots)			\
	DO_TO_FIELD (1,		FixedGlobalsType,		UInt8,			dbgWasEntered,			dbgWasEntered)			\
	DO_TO_FIELD (2,		FixedGlobalsType,		voidPtr,		memCardInfoP,			memCardInfoP)			\
	DO_TO_FIELD (6,		FixedGlobalsType,		UInt32,			memSemaphoreID,			memSemaphoreID)			\
	DO_TO_FIELD (10,	FixedGlobalsType,		UInt16,			memDebugMode,			memDebugMode)			\
	DO_TO_FIELD (12,	FixedGlobalsType,		voidPtr,		dmOpenList,				dmOpenList)				\
	DO_TO_FIELD (16,	FixedGlobalsType,		UInt8,			dbgInDebugger,			dbgInDebugger)			\
	DO_TO_FIELD (17,	FixedGlobalsType,		UInt8,			dbgTracing,				dbgTracing)				\
	DO_TO_FIELD (18,	FixedGlobalsType,		voidPtr,		dbgGlobalsP,			dbgGlobalsP)			\
	DO_TO_FIELD (22,	FixedGlobalsType,		voidPtr,		dbgSerGlobalsP,			dbgSerGlobalsP)			\
	DO_TO_FIELD (26,	FixedGlobalsType,		voidPtr,		sysAppInfoP,			sysAppInfoP)			\
	DO_TO_FIELD (30,	FixedGlobalsType,		voidPtr,		sysKernelDataP,			sysKernelDataP)			\
	DO_TO_FIELD (34,	FixedGlobalsType,		voidPtr,		sysDispatchTableP,		sysDispatchTableP)		\
	DO_TO_FIELD (38,	FixedGlobalsType,		UInt32,			sysOwnerIDsInUse,		sysOwnerIDsInUse)		\
	DO_TO_FIELD (42,	FixedGlobalsType,		voidPtr,		sysAMXAppInfoP,			sysAMXAppInfoP)			\
	DO_TO_FIELD (46,	FixedGlobalsType,		UInt32,			sysClockFreq,			sysClockFreq)			\
	DO_TO_FIELD (50,	FixedGlobalsType,		UInt32Ptr,		sysHardKeyCreators,		sysHardKeyCreators)		\
	DO_TO_FIELD (54,	FixedGlobalsType,		Int16,			sysBatteryCheckTimer,	sysBatteryCheckTimer)	\
	DO_TO_FIELD (56,	FixedGlobalsType,		UInt8,			sysBatteryMinThreshold,	sysBatteryMinThreshold)	\
	DO_TO_FIELD (57,	FixedGlobalsType,		UInt8,			hwrBatteryLevel,		hwrBatteryLevel)		\
	DO_TO_FIELD (58,	FixedGlobalsType,		UInt16,			sysNextBatteryAlertTimer,	sysNextBatteryAlertTimer)	\
	DO_TO_FIELD (60,	FixedGlobalsType,		UInt8,			sysBatteryWarnThreshold,	sysBatteryWarnThreshold)	\
	DO_TO_FIELD (61,	FixedGlobalsType,		UInt8,			sysDispatchTableRev,	sysDispatchTableRev)	\
	DO_TO_FIELD (62,	FixedGlobalsType,		UInt16,			sysDispatchTableSize,	sysDispatchTableSize)	\
	DO_TO_FIELD (64,	FixedGlobalsType,		voidPtr,		sysLibTableP,			sysLibTableP)			\
	DO_TO_FIELD (68,	FixedGlobalsType,		UInt16,			sysLibTableEntries,		sysLibTableEntries)		\
	DO_TO_FIELD (70,	FixedGlobalsType,		voidPtr,		sysConsoleStackChunkP,	sysConsoleStackChunkP)	\
	DO_TO_FIELD (74,	FixedGlobalsType,		voidPtr,		sysUIShellAppInfoP,		sysUIShellAppInfoP)		\
	DO_TO_FIELD (78,	FixedGlobalsType,		UInt32,			sysTimerID,				sysTimerID)				\
	DO_TO_FIELD (82,	FixedGlobalsType,		UInt32,			sysAutoOffEvtTicks,		sysAutoOffEvtTicks)		\
	DO_TO_FIELD (86,	FixedGlobalsType,		UInt16,			sysAutoOffSeconds,		sysAutoOffSeconds)		\
	DO_TO_FIELD (88,	FixedGlobalsType,		Int32,			sysRandomSeed,			sysRandomSeed)			\
	DO_TO_FIELD (92,	FixedGlobalsType,		voidPtr,		slkGlobalsP,			slkGlobalsP)			\
	DO_TO_FIELD (96,	FixedGlobalsType,		voidPtr,		serGlobalsP,			serGlobalsP)			\
	DO_TO_FIELD (100,	FixedGlobalsType,		voidPtr,		scrGlobalsP,			scrGlobalsP)			\
	DO_TO_FIELD (104,	FixedGlobalsType,		voidPtr,		fplGlobalsP,			fplGlobalsP)			\
	DO_TO_FIELD (108,	FixedGlobalsType,		voidPtr,		penGlobalsP,			penGlobalsP)			\
	DO_TO_FIELD (112,	FixedGlobalsType,		voidPtr,		sysEvtMgrGlobalsP,		sysEvtMgrGlobalsP)		\
	DO_TO_FIELD (116,	FixedGlobalsType,		voidPtr,		sndGlobalsP,			sndGlobalsP)			\
	DO_TO_FIELD (120,	FixedGlobalsType,		voidPtr,		timGlobalsP,			timGlobalsP)			\
	DO_TO_FIELD (124,	FixedGlobalsType,		voidPtr,		almGlobalsP,			almGlobalsP)			\
	DO_TO_FIELD (128,	FixedGlobalsType,		voidPtr,		ftrGlobalsP,			ftrGlobalsP)			\
	DO_TO_FIELD (132,	FixedGlobalsType,		voidPtr,		grfGlobalsP,			grfGlobalsP)			\
	DO_TO_FIELD (136,	FixedGlobalsType,		voidPtr,		keyGlobalsP,			keyGlobalsP)			\
	DO_TO_FIELD (140,	FixedGlobalsType,		UInt8,			uiGlobals,				uiGlobals)				\
	DO_TO_FIELD (316,	FixedGlobalsType,		voidPtr,		uiExtensionsP,			uiExtensionsP)			\
	DO_TO_FIELD (320,	FixedGlobalsType,		voidPtr,		curUIAppInfoP,			curUIAppInfoP)			\
	DO_TO_FIELD (324,	FixedGlobalsType,		UInt32,			nextUIAppDBID,			nextUIAppDBID)			\
	DO_TO_FIELD (328,	FixedGlobalsType,		UInt16,			nextUIAppCmd,			nextUIAppCmd)			\
	DO_TO_FIELD (330,	FixedGlobalsType,		voidPtr,		nextUIAppCmdPBP,		nextUIAppCmdPBP)		\
	DO_TO_FIELD (334,	FixedGlobalsType,		UInt8,			nextUIAppCardNo,		nextUIAppCardNo)		\
	DO_TO_FIELD (335,	FixedGlobalsType,		Int8,			memDataWELevel,			memDataWELevel)			\
	DO_TO_FIELD (336,	FixedGlobalsType,		UInt16,			hwrWakeUp,				hwrWakeUp)				\
	DO_TO_FIELD (338,	FixedGlobalsType,		UInt8,			hwrCPUDutyCycle,		hwrCPUDutyCycle)		\
	DO_TO_FIELD (339,	FixedGlobalsType,		UInt8,			hwrPenDown,				hwrPenDown)				\
	DO_TO_FIELD (340,	FixedGlobalsType,		UInt32,			hwrCurTicks,			hwrCurTicks)			\
	DO_TO_FIELD (344,	FixedGlobalsType,		UInt32,			hwrTotalRAMSize,		hwrTotalRAMSize)		\
	DO_TO_FIELD (348,	FixedGlobalsType,		UInt32,			hwrDozeSubTicks,		hwrDozeSubTicks)		\
	DO_TO_FIELD (352,	FixedGlobalsType,		UInt32,			sndOffTicks,			sndOffTicks)			\
	DO_TO_FIELD (356,	FixedGlobalsType,		UInt8,			sysResetFlags,			sysResetFlags)			\
	DO_TO_FIELD (357,	FixedGlobalsType,		UInt8,			sysBatteryKind,			sysBatteryKind)			\
	DO_TO_FIELD (358,	FixedGlobalsType,		UInt32,			memMinDynHeapFree,		memMinDynHeapFree)		\
	DO_TO_FIELD (362,	FixedGlobalsType,		UInt16,			sysPrefFlags,			sysPrefFlags)			\
	DO_TO_FIELD (364,	FixedGlobalsType,		voidPtr,		sysGlobalsP,			sysGlobalsP)			\
	DO_TO_FIELD (368,	FixedGlobalsType,		UInt16,			sysMiscFlags,			sysMiscFlags)			\
	DO_TO_FIELD (370,	FixedGlobalsType,		UInt16,			sysLibNet,				sysLibNet)				\
	DO_TO_FIELD (372,	FixedGlobalsType,		voidPtr,		netPktFreeQ,			netPktFreeQ)			\
	DO_TO_FIELD (376,	FixedGlobalsType,		UInt32,			sysEvGroupID,			sysEvGroupID)			\
	DO_TO_FIELD (380,	FixedGlobalsType,		voidPtr,		irq3GlobalsP,			irq3GlobalsP)			\
	DO_TO_FIELD (384,	FixedGlobalsType,		UInt32,			sysLastBatteryWarning,	sysLastBatteryWarning)	\
	DO_TO_FIELD (388,	FixedGlobalsType,		UInt32,			sysLowMemChecksum,		sysLowMemChecksum)		\
	DO_TO_FIELD (392,	FixedGlobalsType,		UInt16,			hwrHardwareRev,			hwrHardwareRev)			\
	DO_TO_FIELD (394,	FixedGlobalsType,		MemHandle,		dmProtectListH,			dmProtectListH)			\
	DO_TO_FIELD (398,	FixedGlobalsType,		UInt16,			hwrMiscFlags,			hwrMiscFlags)			\
	DO_TO_FIELD (400,	FixedGlobalsType,		UInt32,			sysProfileRefcon,		sysProfileRefcon)		\
	DO_TO_FIELD (404,	FixedGlobalsType,		voidPtr,		sysProfileProcP,		sysProfileProcP)		\
	DO_TO_FIELD (408,	FixedGlobalsType,		UInt32,			flpSoftFPSCR,			flpSoftFPSCR)			\
	DO_TO_FIELD (412,	FixedGlobalsType,		voidPtr,		irLibGlobalsP,			irLibGlobalsP)			\
	DO_TO_FIELD (416,	FixedGlobalsType,		UInt16,			exgActiveLib,			exgActiveLib)			\
	DO_TO_FIELD (418,	FixedGlobalsType,		UInt16,			uiBusyCount,			uiBusyCount)			\
	DO_TO_FIELD (420,	FixedGlobalsType,		voidPtr,		irq1GlobalsP,			irq1GlobalsP)			\
	DO_TO_FIELD (424,	FixedGlobalsType,		voidPtr,		irq2GlobalsP,			irq2GlobalsP)			\
	DO_TO_FIELD (428,	FixedGlobalsType,		voidPtr,		irq6GlobalsP,			irq6GlobalsP)			\
	DO_TO_FIELD (432,	FixedGlobalsType,		voidPtr,		intlMgrGlobalsP,		intlMgrGlobalsP)		\
	DO_TO_FIELD (436,	FixedGlobalsType,		UInt16,			inlineStart,			inlineStart)			\
	DO_TO_FIELD (438,	FixedGlobalsType,		UInt16,			inlineEnd,				inlineEnd)				\
	DO_TO_FIELD (440,	FixedGlobalsType,		Boolean,		inlineActive,			inlineActive)			\
	DO_TO_FIELD (441,	FixedGlobalsType,		UInt8,			inlineDefTerm,			inlineDefTerm)			\
	DO_TO_FIELD (442,	FixedGlobalsType,		voidPtr,		tsmFepLibStatusP,		tsmFepLibStatusP)		\
	DO_TO_FIELD (446,	FixedGlobalsType,		UInt16,			tsmFepLibRefNum,		tsmFepLibRefNum)		\
	DO_TO_FIELD (448,	FixedGlobalsType,		UInt16,			inlineHighlightLen,		inlineHighlightLen)		\
	DO_TO_FIELD (450,	FixedGlobalsType,		voidPtr,		sysBatteryDataP,		sysBatteryDataP)		\
	DO_TO_FIELD (454,	FixedGlobalsType,		voidPtr,		sysBatteryValidKindsP,	sysBatteryValidKindsP)	\
	DO_TO_FIELD (458,	FixedGlobalsType,		Boolean,		sysInShutdown,			sysInShutdown)			\
	DO_TO_FIELD (459,	FixedGlobalsType,		UInt8,			sysMinWakeupThreshold,	sysMinWakeupThreshold)	\
	DO_TO_FIELD (460,	FixedGlobalsType,		UInt32,			hwrMiscFlagsExt,		hwrMiscFlagsExt)		\
	DO_TO_FIELD (464,	FixedGlobalsType,		Boolean,		sysUseCritWarnTimer,	sysUseCritWarnTimer)	\
	DO_TO_FIELD (465,	FixedGlobalsType,		Boolean,		sysCritWarnTimerRunning,	sysCritWarnTimerRunning)	\
	DO_TO_FIELD (466,	FixedGlobalsType,		Int16,			sysCritWarnRTCDayStarted,	sysCritWarnRTCDayStarted)	\
	DO_TO_FIELD (468,	FixedGlobalsType,		Int32,			sysCritWarnTickTimer,	sysCritWarnTickTimer)	\
	DO_TO_FIELD (472,	FixedGlobalsType,		UInt32,			sysCritWarnLastTick,	sysCritWarnLastTick)	\
	DO_TO_FIELD (476,	FixedGlobalsType,		Boolean,		sysCharging,			sysCharging)			\
	DO_TO_FIELD (477,	FixedGlobalsType,		Boolean,		sysUpdateVoltageLevel,	sysUpdateVoltageLevel)	\
	DO_TO_FIELD (478,	FixedGlobalsType,		MemHandle,		notifyGlobalsH,			notifyGlobalsH)			\
	DO_TO_FIELD (482,	FixedGlobalsType,		UInt32,			sysReserved31DWord1,	sysReserved31DWord1)	\
	DO_TO_FIELD (486,	FixedGlobalsType,		UInt32,			sysReserved31DWord2,	sysReserved31DWord2)	\
	DO_TO_FIELD (490,	FixedGlobalsType,		UInt32,			sysReserved31DWord3,	sysReserved31DWord3)	\
	DO_TO_FIELD (494,	FixedGlobalsType,		UInt32,			sysReserved31DWord4,	sysReserved31DWord4)	\
	DO_TO_FIELD (498,	FixedGlobalsType,		voidPtr,		ramGlobalsP,			ramGlobalsP)			\
	DO_TO_FIELD (502,	FixedGlobalsType,		voidPtr,		hwrBatteryGlobalsP,		hwrBatteryGlobalsP)		\
	DO_TO_FIELD (506,	FixedGlobalsType,		UInt8,			hwrBatteryPercent,		hwrBatteryPercent)		\
	DO_TO_FIELD (507,	FixedGlobalsType,		UInt8,			commActivityFlags,		commActivityFlags)		\
	DO_TO_FIELD (508,	FixedGlobalsType,		voidPtr,		uart328GlobalsP,		uart328GlobalsP)		\
	DO_TO_FIELD (512,	FixedGlobalsType,		UInt32,			sysOEMGlobal1,			sysOEMGlobal1)			\
	DO_TO_FIELD (516,	FixedGlobalsType,		UInt32,			sysOEMGlobal2,			sysOEMGlobal2)			\
	DO_TO_FIELD (520,	FixedGlobalsType,		UInt16,			hwrBatteryStepsPerVolt,	hwrBatteryStepsPerVolt)	\
	DO_TO_FIELD (522,	FixedGlobalsType,		voidPtr,		sysPendingStackFrees,	sysPendingStackFrees)	\
	DO_TO_FIELD (526,	FixedGlobalsType,		UInt16,			sysDayCounter,			sysDayCounter)			\
	DO_TO_FIELD (528,	FixedGlobalsType,		Int16,			sysBatteryCheckPeriod,	sysBatteryCheckPeriod)	\
	DO_TO_FIELD (530,	FixedGlobalsType,		UInt16,			hwrJerryPldShadow,		hwrJerryPldShadow)		\
	DO_TO_FIELD (532,	FixedGlobalsType,		voidPtr,		radioGlobalsP,			radioGlobalsP)			\
	DO_TO_FIELD (536,	FixedGlobalsType,		voidPtr,		fatalAlertTemplateP,	fatalAlertTemplateP)	\
	DO_TO_FIELD (540,	FixedGlobalsType,		UInt16,			fatalAlertBtnWidth,		fatalAlertBtnWidth)		\
	DO_TO_FIELD (542,	FixedGlobalsType,		voidPtr,		silkscreenInfoP,		silkscreenInfoP)		\
	DO_TO_FIELD (546,	FixedGlobalsType,		UInt16,			inlineDefChecksum,		inlineDefChecksum)		\
	DO_TO_FIELD (548,	FixedGlobalsType,		voidPtr,		inlineDefField,			inlineDefField)			\
	DO_TO_FIELD (552,	FixedGlobalsType,		Boolean,		notifyBroadcastPending,	notifyBroadcastPending)	\
	DO_TO_FIELD (553,	FixedGlobalsType,		UInt8,			sysReserved35Byte1,		sysReserved35Byte1)		\
	DO_TO_FIELD (554,	FixedGlobalsType,		UInt32,			hwrOEMCompanyID,		hwrOEMCompanyID)		\
	DO_TO_FIELD (558,	FixedGlobalsType,		UInt32,			hwrOEMDeviceID,			hwrOEMDeviceID)			\
	DO_TO_FIELD (562,	FixedGlobalsType,		UInt32,			hwrOEMHALID,			hwrOEMHALID)			\
	DO_TO_FIELD (566,	FixedGlobalsType,		voidPtr,		sysUpdateGlobalsP,		sysUpdateGlobalsP)		\
	DO_TO_FIELD (570,	FixedGlobalsType,		UInt16,			smallROMChecksum,		smallROMChecksum)		\


// ========================================
// ===== LowMemHdrType =====
// ========================================

#define FOR_LowMemHdrType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(LowMemHdrType, 828, FOR_LowMemHdrType_FIELDS)

#define FOR_LowMemHdrType_FIELDS(DO_TO_FIELD)												\
	DO_TO_FIELD (0,		LowMemHdrType,			M68KExcTableType,	vectors,	vectors)	\
	DO_TO_FIELD (256,	LowMemHdrType,			FixedGlobalsType,	globals,	globals)	\


// ========================================
// ===== M68KExcTableType =====
// ========================================

#define FOR_M68KExcTableType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(M68KExcTableType, 256, FOR_M68KExcTableType_FIELDS)

#define FOR_M68KExcTableType_FIELDS(DO_TO_FIELD)											\
	DO_TO_FIELD (0,		M68KExcTableType,		UInt32,			initStack,		initStack)	\
	DO_TO_FIELD (4,		M68KExcTableType,		UInt32,			initPC,			initPC)		\
	DO_TO_FIELD (8,		M68KExcTableType,		UInt32,			busErr,			busErr)		\
	DO_TO_FIELD (12,	M68KExcTableType,		UInt32,			addressErr,		addressErr)	\
	DO_TO_FIELD (16,	M68KExcTableType,		UInt32,			illegalInstr,	illegalInstr)\
	DO_TO_FIELD (20,	M68KExcTableType,		UInt32,			divideByZero,	divideByZero)\
	DO_TO_FIELD (24,	M68KExcTableType,		UInt32,			chk,			chk)		\
	DO_TO_FIELD (28,	M68KExcTableType,		UInt32,			trap,			trap)		\
	DO_TO_FIELD (32,	M68KExcTableType,		UInt32,			privilege,		privilege)	\
	DO_TO_FIELD (36,	M68KExcTableType,		UInt32,			trace,			trace)		\
	DO_TO_FIELD (40,	M68KExcTableType,		UInt32,			aTrap,			aTrap)		\
	DO_TO_FIELD (44,	M68KExcTableType,		UInt32,			fTrap,			fTrap)		\
	DO_TO_FIELD (48,	M68KExcTableType,		UInt32,			reserved12,		reserved12)	\
	DO_TO_FIELD (52,	M68KExcTableType,		UInt32,			coproc,			coproc)		\
	DO_TO_FIELD (56,	M68KExcTableType,		UInt32,			formatErr,		formatErr)	\
	DO_TO_FIELD (60,	M68KExcTableType,		UInt32,			unitializedInt,	unitializedInt)\
	DO_TO_FIELD (96,	M68KExcTableType,		UInt32,			spuriousInt,	spuriousInt)\
	DO_TO_FIELD (100,	M68KExcTableType,		UInt32,			autoVec1,		autoVec1)	\
	DO_TO_FIELD (104,	M68KExcTableType,		UInt32,			autoVec2,		autoVec2)	\
	DO_TO_FIELD (108,	M68KExcTableType,		UInt32,			autoVec3,		autoVec3)	\
	DO_TO_FIELD (112,	M68KExcTableType,		UInt32,			autoVec4,		autoVec4)	\
	DO_TO_FIELD (116,	M68KExcTableType,		UInt32,			autoVec5,		autoVec5)	\
	DO_TO_FIELD (120,	M68KExcTableType,		UInt32,			autoVec6,		autoVec6)	\
	DO_TO_FIELD (124,	M68KExcTableType,		UInt32,			autoVec7,		autoVec7)	\


// ========================================
// ===== M68KRegsType =====
// ========================================

#define FOR_M68KRegsType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(M68KRegsType, 74, FOR_M68KRegsType_FIELDS)

#define FOR_M68KRegsType_FIELDS(DO_TO_FIELD)												\
	DO_TO_FIELD (0,		M68KRegsType,			UInt32,			d,				d)			\
	DO_TO_FIELD (32,	M68KRegsType,			UInt32,			a,				a)			\
	DO_TO_FIELD (60,	M68KRegsType,			UInt32,			usp,			usp)		\
	DO_TO_FIELD (64,	M68KRegsType,			UInt32,			ssp,			ssp)		\
	DO_TO_FIELD (68,	M68KRegsType,			UInt32,			pc,				pc)			\
	DO_TO_FIELD (72,	M68KRegsType,			UInt16,			sr,				sr)			\


// ========================================
// ===== SlkPktFooterType =====
// ========================================

#define FOR_SlkPktFooterType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SlkPktFooterType, 2, FOR_SlkPktFooterType_FIELDS)

#define FOR_SlkPktFooterType_FIELDS(DO_TO_FIELD)											\
	DO_TO_FIELD (0,		SlkPktFooterType,		UInt16,			crc16,			crc16)		\


// ========================================
// ===== SlkPktHeaderType =====
// ========================================

#define FOR_SlkPktHeaderType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SlkPktHeaderType, 10, FOR_SlkPktHeaderType_FIELDS)

#define FOR_SlkPktHeaderType_FIELDS(DO_TO_FIELD)											\
	DO_TO_FIELD (0,		SlkPktHeaderType,		UInt16,			signature1,		signature1)	\
	DO_TO_FIELD (2,		SlkPktHeaderType,		UInt8,			signature2,		signature2)	\
	DO_TO_FIELD (3,		SlkPktHeaderType,		UInt8,			dest,			dest)		\
	DO_TO_FIELD (4,		SlkPktHeaderType,		UInt8,			src,			src)		\
	DO_TO_FIELD (5,		SlkPktHeaderType,		UInt8,			type,			type)		\
	DO_TO_FIELD (6,		SlkPktHeaderType,		UInt16,			bodySize,		bodySize)	\
	DO_TO_FIELD (8,		SlkPktHeaderType,		UInt8,			transId,		transId)	\
	DO_TO_FIELD (9,		SlkPktHeaderType,		SlkPktHeaderChecksum,	checksum,	checksum)	\


// ========================================
// ===== SysPktBodyType =====
// ========================================

#define FOR_SysPktBodyType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktBodyType, sysPktMaxBodySize, FOR_SysPktBodyType_FIELDS)

#define FOR_SysPktBodyType_FIELDS(DO_TO_FIELD)												\
	DO_TO_FIELD (0,		SysPktBodyType,			UInt8,			command,		command)	\
	DO_TO_FIELD (1,		SysPktBodyType,			UInt8,			_filler,		_filler)	\
	DO_TO_FIELD (2,		SysPktBodyType,			UInt8,			data,			data)		\


// ========================================
// ===== SysPktContinueCmdType =====
// ========================================

#define FOR_SysPktContinueCmdType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktContinueCmdType, 90, FOR_SysPktContinueCmdType_FIELDS)

#define FOR_SysPktContinueCmdType_FIELDS(DO_TO_FIELD)										\
	DO_TO_FIELD (0,		SysPktContinueCmdType,	UInt8,			command,		command)	\
	DO_TO_FIELD (1,		SysPktContinueCmdType,	UInt8,			_filler,		_filler)	\
	DO_TO_FIELD (2,		SysPktContinueCmdType,	M68KRegsType,	regs,			reg)		\
	DO_TO_FIELD (76,	SysPktContinueCmdType,	Boolean,		stepSpy,		stepSpy)	\
	DO_TO_FIELD (77,	SysPktContinueCmdType,	UInt8,			reserved,		reserved)	\
	DO_TO_FIELD (78,	SysPktContinueCmdType,	UInt32,			ssAddr,			ssAddr)		\
	DO_TO_FIELD (82,	SysPktContinueCmdType,	UInt32,			ssCount,		ssCount)	\
	DO_TO_FIELD (86,	SysPktContinueCmdType,	UInt32,			ssCheckSum,		ssCheckSum)	\


// ========================================
// ===== SysPktDbgBreakToggleCmdType =====
// ========================================

#define FOR_SysPktDbgBreakToggleCmdType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktDbgBreakToggleCmdType, 2, FOR_SysPktDbgBreakToggleCmdType_FIELDS)

#define FOR_SysPktDbgBreakToggleCmdType_FIELDS(DO_TO_FIELD)									\
	DO_TO_FIELD (0,		SysPktDbgBreakToggleCmdType,	UInt8,	command,		command)	\
	DO_TO_FIELD (1,		SysPktDbgBreakToggleCmdType,	UInt8,	_filler,		_filler)	\


// ========================================
// ===== SysPktDbgBreakToggleRspType =====
// ========================================

#define FOR_SysPktDbgBreakToggleRspType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktDbgBreakToggleRspType, 4, FOR_SysPktDbgBreakToggleRspType_FIELDS)

#define FOR_SysPktDbgBreakToggleRspType_FIELDS(DO_TO_FIELD)									\
	DO_TO_FIELD (0,		SysPktDbgBreakToggleRspType,	UInt8,		command,	command)	\
	DO_TO_FIELD (1,		SysPktDbgBreakToggleRspType,	UInt8,		_filler,	_filler)	\
	DO_TO_FIELD (2,		SysPktDbgBreakToggleRspType,	Boolean,	newState,	newState)	\
	DO_TO_FIELD (3,		SysPktDbgBreakToggleRspType,	UInt8,		reserved,	reserved)	\


// ========================================
// ===== SysPktFindCmdType =====
// ========================================

#define FOR_SysPktFindCmdType2_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktFindCmdType2, sysPktMaxBodySize, FOR_SysPktFindCmdType2_FIELDS)

#define FOR_SysPktFindCmdType2_FIELDS(DO_TO_FIELD)											\
	DO_TO_FIELD (0,		SysPktFindCmdType2,		UInt8,			command,		command)	\
	DO_TO_FIELD (1,		SysPktFindCmdType2,		UInt8,			_filler,		_filler)	\
	DO_TO_FIELD (2,		SysPktFindCmdType2,		UInt32,			firstAddr,		firstAddr)	\
	DO_TO_FIELD (6,		SysPktFindCmdType2,		UInt32,			lastAddr,		lastAddr)	\
	DO_TO_FIELD (10,	SysPktFindCmdType2,		UInt16,			numBytes,		numBytes)	\
	DO_TO_FIELD (12,	SysPktFindCmdType2,		Boolean,		caseInsensitive,caseInsensitive)\
	DO_TO_FIELD (13,	SysPktFindCmdType2,		UInt8,			padding,		padding)	\
	DO_TO_FIELD (14,	SysPktFindCmdType2,		UInt8,			data,			data)		\


// ========================================
// ===== SysPktFindRspType =====
// ========================================

#define FOR_SysPktFindRspType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktFindRspType, 8, FOR_SysPktFindRspType_FIELDS)

#define FOR_SysPktFindRspType_FIELDS(DO_TO_FIELD)											\
	DO_TO_FIELD (0,		SysPktFindRspType,		UInt8,			command,		command)	\
	DO_TO_FIELD (1,		SysPktFindRspType,		UInt8,			_filler,		_filler)	\
	DO_TO_FIELD (2,		SysPktFindRspType,		UInt8,			addr,			addr)		\
	DO_TO_FIELD (6,		SysPktFindRspType,		UInt8,			found,			found)		\
	DO_TO_FIELD (7,		SysPktFindRspType,		UInt8,			padding,		padding)	\


// ========================================
// ===== SysPktGetBreakpointsCmdType =====
// ========================================

#define FOR_SysPktGetBreakpointsCmdType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktGetBreakpointsCmdType, 2, FOR_SysPktGetBreakpointsCmdType_FIELDS)

#define FOR_SysPktGetBreakpointsCmdType_FIELDS(DO_TO_FIELD)									\
	DO_TO_FIELD (0,		SysPktGetBreakpointsCmdType,	UInt8,	command,		command)	\
	DO_TO_FIELD (1,		SysPktGetBreakpointsCmdType,	UInt8,	_filler,		_filler)	\


// ========================================
// ===== SysPktEmptyRspType =====
// ========================================

#define FOR_SysPktEmptyRspType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktEmptyRspType, 2, FOR_SysPktEmptyRspType_FIELDS)

#define FOR_SysPktEmptyRspType_FIELDS(DO_TO_FIELD)											\
	DO_TO_FIELD (0,		SysPktEmptyRspType,		UInt8,			command,		command)	\
	DO_TO_FIELD (1,		SysPktEmptyRspType,		UInt8,			_filler,		_filler)	\


// ========================================
// ===== SysPktGetBreakpointsRspType2 =====
// ========================================

#define FOR_SysPktGetBreakpointsRspType2_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktGetBreakpointsRspType2, 38, FOR_SysPktGetBreakpointsRspType2_FIELDS)

#define FOR_SysPktGetBreakpointsRspType2_FIELDS(DO_TO_FIELD)									\
	DO_TO_FIELD (0,		SysPktGetBreakpointsRspType2,	UInt8,			command,	command)	\
	DO_TO_FIELD (1,		SysPktGetBreakpointsRspType2,	UInt8,			_filler,	_filler)	\
	DO_TO_FIELD (2,		SysPktGetBreakpointsRspType2,	BreakpointType,	bp,			bp)			\


// ========================================
// ===== SysPktGetTrapBreaksCmdType =====
// ========================================

#define FOR_SysPktGetTrapBreaksCmdType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktGetTrapBreaksCmdType, 2, FOR_SysPktGetTrapBreaksCmdType_FIELDS)

#define FOR_SysPktGetTrapBreaksCmdType_FIELDS(DO_TO_FIELD)									\
	DO_TO_FIELD (0,		SysPktGetTrapBreaksCmdType,	UInt8,		command,		command)	\
	DO_TO_FIELD (1,		SysPktGetTrapBreaksCmdType,	UInt8,		_filler,		_filler)	\


// ========================================
// ===== SysPktGetTrapBreaksRspType2 =====
// ========================================

#define FOR_SysPktGetTrapBreaksRspType2_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktGetTrapBreaksRspType2, 12, FOR_SysPktGetTrapBreaksRspType2_FIELDS)

#define FOR_SysPktGetTrapBreaksRspType2_FIELDS(DO_TO_FIELD)									\
	DO_TO_FIELD (0,		SysPktGetTrapBreaksRspType2,	UInt8,	command,		command)	\
	DO_TO_FIELD (1,		SysPktGetTrapBreaksRspType2,	UInt8,	_filler,		_filler)	\
	DO_TO_FIELD (2,		SysPktGetTrapBreaksRspType2,	UInt16,	trapBP,			trapBP)		\

// ========================================
// ===== SysPktGetTrapConditionsCmdType =====
// ========================================

#define FOR_SysPktGetTrapConditionsCmdType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktGetTrapConditionsCmdType, 2, FOR_SysPktGetTrapConditionsCmdType_FIELDS)

#define FOR_SysPktGetTrapConditionsCmdType_FIELDS(DO_TO_FIELD)									\
	DO_TO_FIELD (0,		SysPktGetTrapConditionsCmdType,	UInt8,	command,		command)	\
	DO_TO_FIELD (1,		SysPktGetTrapConditionsCmdType,	UInt8,	_filler,		_filler)	\


// ========================================
// ===== SysPktGetTrapConditionsRspType2 =====
// ========================================

#define FOR_SysPktGetTrapConditionsRspType2_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktGetTrapConditionsRspType2, 12, FOR_SysPktGetTrapConditionsRspType2_FIELDS)

#define FOR_SysPktGetTrapConditionsRspType2_FIELDS(DO_TO_FIELD)									\
	DO_TO_FIELD (0,	 	SysPktGetTrapConditionsRspType2,	UInt8,	command,		command)	\
	DO_TO_FIELD (1,		SysPktGetTrapConditionsRspType2,	UInt8,	_filler,		_filler)	\
	DO_TO_FIELD (2,		SysPktGetTrapConditionsRspType2,	UInt16,	trapParam,		trapParam)	\


// ========================================
// ===== SysPktReadMemCmdType =====
// ========================================

#define FOR_SysPktReadMemCmdType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktReadMemCmdType, 8, FOR_SysPktReadMemCmdType_FIELDS)

#define FOR_SysPktReadMemCmdType_FIELDS(DO_TO_FIELD)										\
	DO_TO_FIELD (0,		SysPktReadMemCmdType,	UInt8,			command,		command)	\
	DO_TO_FIELD (1,		SysPktReadMemCmdType,	UInt8,			_filler,		_filler)	\
	DO_TO_FIELD (2,		SysPktReadMemCmdType,	voidPtr,		address,		address)	\
	DO_TO_FIELD (6,		SysPktReadMemCmdType,	UInt16,			numBytes,		numBytes)	\


// ========================================
// ===== SysPktReadMemRspType2 =====
// ========================================

#define FOR_SysPktReadMemRspType2_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktReadMemRspType2, sysPktMaxBodySize, FOR_SysPktReadMemRspType2_FIELDS)

#define FOR_SysPktReadMemRspType2_FIELDS(DO_TO_FIELD)										\
	DO_TO_FIELD (0,		SysPktReadMemRspType2,	UInt8,			command,		command)	\
	DO_TO_FIELD (1,		SysPktReadMemRspType2,	UInt8,			_filler,		_filler)	\
	DO_TO_FIELD (2,		SysPktReadMemRspType2,	char,			data,			data)		\


// ========================================
// ===== SysPktReadRegsCmdType =====
// ========================================

#define FOR_SysPktReadRegsCmdType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktReadRegsCmdType, 2, FOR_SysPktReadRegsCmdType_FIELDS)

#define FOR_SysPktReadRegsCmdType_FIELDS(DO_TO_FIELD)										\
	DO_TO_FIELD (0,		SysPktReadRegsCmdType,	UInt8,			command,		command)	\
	DO_TO_FIELD (1,		SysPktReadRegsCmdType,	UInt8,			_filler,		_filler)	\


// ========================================
// ===== SysPktReadRegsRspType2 =====
// ========================================

#define FOR_SysPktReadRegsRspType2_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktReadRegsRspType2, 76, FOR_SysPktReadRegsRspType2_FIELDS)

#define FOR_SysPktReadRegsRspType2_FIELDS(DO_TO_FIELD)										\
	DO_TO_FIELD (0,		SysPktReadRegsRspType2,	UInt8,			command,		command)	\
	DO_TO_FIELD (1,		SysPktReadRegsRspType2,	UInt8,			_filler,		_filler)	\
	DO_TO_FIELD (2,		SysPktReadRegsRspType2,	M68KRegsType,	reg,			reg)		\


// ========================================
// ===== SysPktRPC2Type =====
// ========================================

#define FOR_SysPktRPC2Type_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktRPC2Type, sysPktMaxBodySize, FOR_SysPktRPC2Type_FIELDS)

#define FOR_SysPktRPC2Type_FIELDS(DO_TO_FIELD)												\
	DO_TO_FIELD (0,		SysPktRPC2Type,			UInt8,			command,		command)	\
	DO_TO_FIELD (1,		SysPktRPC2Type,			UInt8,			_filler,		_filler)	\
	DO_TO_FIELD (2,		SysPktRPC2Type,			UInt16,			trapWord,		trapWord)	\
	DO_TO_FIELD (4,		SysPktRPC2Type,			UInt32,			resultD0,		resultD0)	\
	DO_TO_FIELD (8,		SysPktRPC2Type,			UInt32,			resultA0,		resultA0)	\
	DO_TO_FIELD (12,	SysPktRPC2Type,			UInt16,			resultException,resultException)\
	DO_TO_FIELD (14,	SysPktRPC2Type,			UInt8,			DRegMask,		DRegMask)	\
	DO_TO_FIELD (15,	SysPktRPC2Type,			UInt8,			ARegMask,		ARegMask)	\
	DO_TO_FIELD (16,	SysPktRPC2Type,			UInt32,			Regs,			Regs)		\
/*	DO_TO_FIELD (1,		SysPktRPC2Type,			UInt16,			numParams,		numParams)*/\
/*	DO_TO_FIELD (1,		SysPktRPC2Type,			char,			param,			param) */	\


// ========================================
// ===== SysPktRPCParamType2 =====
// ========================================

#define FOR_SysPktRPCParamType2_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktRPCParamType2, 6, FOR_SysPktRPCParamType2_FIELDS)

#define FOR_SysPktRPCParamType2_FIELDS(DO_TO_FIELD)											\
	DO_TO_FIELD (0,		SysPktRPCParamType2,	UInt8,			byRef,			byRef)		\
	DO_TO_FIELD (1,		SysPktRPCParamType2,	UInt8,			size,			size)		\
	DO_TO_FIELD (2,		SysPktRPCParamType2,	UInt8,			asByte,			asByte)		\
	DO_TO_FIELD (2,		SysPktRPCParamType2,	UInt16,			asShort,		asShort)	\
	DO_TO_FIELD (2,		SysPktRPCParamType2,	UInt32,			asLong,			asLong)		\


// ========================================
// ===== SysPktRPCType2 =====
// ========================================

#define FOR_SysPktRPCType2_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktRPCType2, sysPktMaxBodySize, FOR_SysPktRPCType2_FIELDS)

#define FOR_SysPktRPCType2_FIELDS(DO_TO_FIELD)												\
	DO_TO_FIELD (0,		SysPktRPCType2,			UInt8,			command,		command)	\
	DO_TO_FIELD (1,		SysPktRPCType2,			UInt8,			_filler,		_filler)	\
	DO_TO_FIELD (2,		SysPktRPCType2,			UInt16,			trapWord,		trapWord)	\
	DO_TO_FIELD (4,		SysPktRPCType2,			UInt32,			resultD0,		resultD0)	\
	DO_TO_FIELD (8,		SysPktRPCType2,			UInt32,			resultA0,		resultA0)	\
	DO_TO_FIELD (12,	SysPktRPCType2,			UInt16,			numParams,		numParams)	\
	DO_TO_FIELD (14,	SysPktRPCType2,			char,			param,			param)		\


// ========================================
// ===== SysPktRtnNameCmdType =====
// ========================================

#define FOR_SysPktRtnNameCmdType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktRtnNameCmdType, 6, FOR_SysPktRtnNameCmdType_FIELDS)

#define FOR_SysPktRtnNameCmdType_FIELDS(DO_TO_FIELD)										\
	DO_TO_FIELD (0,		SysPktRtnNameCmdType,	UInt8,			command,		command)	\
	DO_TO_FIELD (1,		SysPktRtnNameCmdType,	UInt8,			_filler,		_filler)	\
	DO_TO_FIELD (2,		SysPktRtnNameCmdType,	voidPtr,		address,		address)	\


// ========================================
// ===== SysPktRtnNameRspType =====
// ========================================

#define FOR_SysPktRtnNameRspType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktRtnNameRspType, (14 + sysPktMaxNameLen), FOR_SysPktRtnNameRspType_FIELDS)

#define FOR_SysPktRtnNameRspType_FIELDS(DO_TO_FIELD)										\
	DO_TO_FIELD (0,		SysPktRtnNameRspType,	UInt8,			command,		command)	\
	DO_TO_FIELD (1,		SysPktRtnNameRspType,	UInt8,			_filler,		_filler)	\
	DO_TO_FIELD (2,		SysPktRtnNameRspType,	voidPtr,		address,		address)	\
	DO_TO_FIELD (6,		SysPktRtnNameRspType,	voidPtr,		startAddr,		startAddr)	\
	DO_TO_FIELD (10,	SysPktRtnNameRspType,	voidPtr,		endAddr,		endAddr)	\
	DO_TO_FIELD (14,	SysPktRtnNameRspType,	char,			name,			name)		\


// ========================================
// ===== SysPktSetBreakpointsCmdType2 =====
// ========================================

#define FOR_SysPktSetBreakpointsCmdType2_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktSetBreakpointsCmdType2, 38, FOR_SysPktSetBreakpointsCmdType2_FIELDS)

#define FOR_SysPktSetBreakpointsCmdType2_FIELDS(DO_TO_FIELD)								\
	DO_TO_FIELD (0,		SysPktSetBreakpointsCmdType2,	UInt8,			command,	command)\
	DO_TO_FIELD (1,		SysPktSetBreakpointsCmdType2,	UInt8,			_filler,	_filler)\
	DO_TO_FIELD (2,		SysPktSetBreakpointsCmdType2,	BreakpointType,	bp,			bp)		\


// ========================================
// ===== SysPktSetBreakpointsRspType =====
// ========================================

#define FOR_SysPktSetBreakpointsRspType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktSetBreakpointsRspType, 2, FOR_SysPktSetBreakpointsRspType_FIELDS)

#define FOR_SysPktSetBreakpointsRspType_FIELDS(DO_TO_FIELD)									\
	DO_TO_FIELD (0,		SysPktSetBreakpointsRspType,	UInt8,	command,		command)	\
	DO_TO_FIELD (1,		SysPktSetBreakpointsRspType,	UInt8,	_filler,		_filler)	\


// ========================================
// ===== SysPktSetTrapBreaksCmdType2 =====
// ========================================

#define FOR_SysPktSetTrapBreaksCmdType2_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktSetTrapBreaksCmdType2, 12, FOR_SysPktSetTrapBreaksCmdType2_FIELDS)

#define FOR_SysPktSetTrapBreaksCmdType2_FIELDS(DO_TO_FIELD)									\
	DO_TO_FIELD (0,		SysPktSetTrapBreaksCmdType2,	UInt8,	command,		command)	\
	DO_TO_FIELD (1,		SysPktSetTrapBreaksCmdType2,	UInt8,	_filler,		_filler)	\
	DO_TO_FIELD (2,		SysPktSetTrapBreaksCmdType2,	UInt16,	trapBP,			trapBP)		\


// ========================================
// ===== SysPktSetTrapConditionsCmdType2 =====
// ========================================

#define FOR_SysPktSetTrapConditionsCmdType2_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktSetTrapConditionsCmdType2, 12, FOR_SysPktSetTrapConditionsCmdType2_FIELDS)

#define FOR_SysPktSetTrapConditionsCmdType2_FIELDS(DO_TO_FIELD)									\
	DO_TO_FIELD (0,		SysPktSetTrapConditionsCmdType2,	UInt8,	command,		command)	\
	DO_TO_FIELD (1,		SysPktSetTrapConditionsCmdType2,	UInt8,	_filler,		_filler)	\
	DO_TO_FIELD (2,		SysPktSetTrapConditionsCmdType2,	UInt16,	trapParam,			trapParam)		\


// ========================================
// ===== SysPktSetTrapConditionsRspType =====
// ========================================

#define FOR_SysPktSetTrapConditionsRspType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktSetTrapConditionsRspType, 2, FOR_SysPktSetTrapConditionsRspType_FIELDS)

#define FOR_SysPktSetTrapConditionsRspType_FIELDS(DO_TO_FIELD)									\
	DO_TO_FIELD (0,		SysPktSetTrapConditionsRspType,	UInt8,	command,		command)	\
	DO_TO_FIELD (1,		SysPktSetTrapConditionsRspType,	UInt8,	_filler,		_filler)	\


// ========================================
// ===== SysPktStateCmdType =====
// ========================================

#define FOR_SysPktStateCmdType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktStateCmdType, 2, FOR_SysPktStateCmdType_FIELDS)

#define FOR_SysPktStateCmdType_FIELDS(DO_TO_FIELD)											\
	DO_TO_FIELD (0,		SysPktStateCmdType,		UInt8,			command,		command)	\
	DO_TO_FIELD (1,		SysPktStateCmdType,		UInt8,			_filler,		_filler)	\


// ========================================
// ===== SysPktStateRspType =====
// ========================================

#define FOR_SysPktStateRspType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktStateRspType, 188, FOR_SysPktStateRspType_FIELDS)

#define FOR_SysPktStateRspType_FIELDS(DO_TO_FIELD)											\
	DO_TO_FIELD (0,		SysPktStateRspType,		UInt8,			command,		command)	\
	DO_TO_FIELD (1,		SysPktStateRspType,		UInt8,			_filler,		_filler)	\
	DO_TO_FIELD (2,		SysPktStateRspType,		Boolean,		resetted,		resetted)	\
	DO_TO_FIELD (3,		SysPktStateRspType,		UInt8,			reserved1,		reserved1)	\
	DO_TO_FIELD (4,		SysPktStateRspType,		UInt16,			exceptionId,	exceptionId)\
	DO_TO_FIELD (6,		SysPktStateRspType,		M68KRegsType,	reg,			reg)		\
	DO_TO_FIELD (80,	SysPktStateRspType,		UInt16,			inst,			inst)		\
	DO_TO_FIELD (110,	SysPktStateRspType,		BreakpointType,	bp,				bp)			\
	DO_TO_FIELD (146,	SysPktStateRspType,		voidPtr,		startAddr,		startAddr)	\
	DO_TO_FIELD (150,	SysPktStateRspType,		voidPtr,		endAddr,		endAddr)	\
	DO_TO_FIELD (154,	SysPktStateRspType,		char,			name,			name)		\
	DO_TO_FIELD (186,	SysPktStateRspType,		UInt8,			trapTableRev,	trapTableRev)\
	DO_TO_FIELD (187,	SysPktStateRspType,		UInt8,			reserved2,		reserved2)	\


// ========================================
// ===== SysPktWriteMemCmdType =====
// ========================================

#define FOR_SysPktWriteMemCmdType2_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktWriteMemCmdType2, sysPktMaxBodySize, FOR_SysPktWriteMemCmdType2_FIELDS)

#define FOR_SysPktWriteMemCmdType2_FIELDS(DO_TO_FIELD)										\
	DO_TO_FIELD (0,		SysPktWriteMemCmdType2,	UInt8,			command,		command)	\
	DO_TO_FIELD (1,		SysPktWriteMemCmdType2,	UInt8,			_filler,		_filler)	\
	DO_TO_FIELD (2,		SysPktWriteMemCmdType2,	voidPtr,		address,		address)	\
	DO_TO_FIELD (6,		SysPktWriteMemCmdType2,	UInt16,			numBytes,		numBytes)	\
	DO_TO_FIELD (8,		SysPktWriteMemCmdType2,	char,			data,			data)		\


// ========================================
// ===== SysPktWriteMemRspType =====
// ========================================

#define FOR_SysPktWriteMemRspType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktWriteMemRspType, 2, FOR_SysPktWriteMemRspType_FIELDS)

#define FOR_SysPktWriteMemRspType_FIELDS(DO_TO_FIELD)										\
	DO_TO_FIELD (0,		SysPktWriteMemRspType,	UInt8,			command,		command)	\
	DO_TO_FIELD (1,		SysPktWriteMemRspType,	UInt8,			_filler,		_filler)	\


// ========================================
// ===== SysPktWriteRegsCmdType =====
// ========================================

#define FOR_SysPktWriteRegsCmdType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktWriteRegsCmdType, 76, FOR_SysPktWriteRegsCmdType_FIELDS)

#define FOR_SysPktWriteRegsCmdType_FIELDS(DO_TO_FIELD)										\
	DO_TO_FIELD (0,		SysPktWriteRegsCmdType,	UInt8,			command,		command)	\
	DO_TO_FIELD (1,		SysPktWriteRegsCmdType,	UInt8,			_filler,		_filler)	\
	DO_TO_FIELD (2,		SysPktWriteRegsCmdType,	M68KRegsType,	reg,			reg)		\


// ========================================
// ===== SysPktWriteRegsRspType =====
// ========================================

#define FOR_SysPktWriteRegsRspType_STRUCT(DO_TO_STRUCT)	\
	DO_TO_STRUCT(SysPktWriteRegsRspType, 2, FOR_SysPktWriteRegsRspType_FIELDS)

#define FOR_SysPktWriteRegsRspType_FIELDS(DO_TO_FIELD)										\
	DO_TO_FIELD (0,		SysPktWriteRegsRspType,	UInt8,			command,		command)	\
	DO_TO_FIELD (1,		SysPktWriteRegsRspType,	UInt8,			_filler,		_filler)	\


// ================================================================================
// FOR_EACH_STRUCT allows us to iterate over all the macros that we've defined to
// describe the struct layout.  Whenever a new struct-defining macro is created, it
// should be added here.
//
// Note: breaking this up into multiple lists.  If kept as one list, the CW Pro 5.2
// compiler complains that the macros are too complex.
// ================================================================================

#define FOR_EACH_STRUCT1(DO_TO_STRUCT)							\
																\
	FOR_BreakpointType_STRUCT					(DO_TO_STRUCT)	\
	FOR_FixedGlobalsType_STRUCT					(DO_TO_STRUCT)	\
	FOR_M68KExcTableType_STRUCT					(DO_TO_STRUCT)	\
	FOR_M68KRegsType_STRUCT						(DO_TO_STRUCT)	\
																\
	FOR_ctlEnter_STRUCT							(DO_TO_STRUCT)	\
	FOR_EventTypeData_STRUCT					(DO_TO_STRUCT)	\
	FOR_EventType_STRUCT						(DO_TO_STRUCT)	\
	FOR_LowMemHdrType_STRUCT					(DO_TO_STRUCT)	\
	FOR_SlkPktFooterType_STRUCT					(DO_TO_STRUCT)	\
	FOR_SlkPktHeaderType_STRUCT					(DO_TO_STRUCT)	\
	FOR_SysPktBodyType_STRUCT					(DO_TO_STRUCT)	\


#define FOR_EACH_STRUCT2(DO_TO_STRUCT)							\
																\
	FOR_SysPktContinueCmdType_STRUCT			(DO_TO_STRUCT)	\
	FOR_SysPktDbgBreakToggleCmdType_STRUCT		(DO_TO_STRUCT)	\
	FOR_SysPktDbgBreakToggleRspType_STRUCT		(DO_TO_STRUCT)	\
	FOR_SysPktEmptyRspType_STRUCT				(DO_TO_STRUCT)	\
	FOR_SysPktFindCmdType2_STRUCT				(DO_TO_STRUCT)	\
	FOR_SysPktFindRspType_STRUCT				(DO_TO_STRUCT)	\
	FOR_SysPktGetBreakpointsCmdType_STRUCT		(DO_TO_STRUCT)	\
	FOR_SysPktGetBreakpointsRspType2_STRUCT		(DO_TO_STRUCT)	\
	FOR_SysPktGetTrapBreaksCmdType_STRUCT		(DO_TO_STRUCT)	\
	FOR_SysPktGetTrapBreaksRspType2_STRUCT		(DO_TO_STRUCT)	\


#define FOR_EACH_STRUCT3(DO_TO_STRUCT)							\
																\
	FOR_SysPktGetTrapConditionsCmdType_STRUCT	(DO_TO_STRUCT)	\
	FOR_SysPktGetTrapConditionsRspType2_STRUCT	(DO_TO_STRUCT)	\
	FOR_SysPktReadMemCmdType_STRUCT				(DO_TO_STRUCT)	\
	FOR_SysPktReadMemRspType2_STRUCT			(DO_TO_STRUCT)	\
	FOR_SysPktReadRegsCmdType_STRUCT			(DO_TO_STRUCT)	\
	FOR_SysPktReadRegsRspType2_STRUCT			(DO_TO_STRUCT)	\
	FOR_SysPktRPC2Type_STRUCT					(DO_TO_STRUCT)	\
	FOR_SysPktRPCParamType2_STRUCT				(DO_TO_STRUCT)	\
	FOR_SysPktRPCType2_STRUCT					(DO_TO_STRUCT)	\
	FOR_SysPktRtnNameCmdType_STRUCT				(DO_TO_STRUCT)	\
	FOR_SysPktRtnNameRspType_STRUCT				(DO_TO_STRUCT)	\


#define FOR_EACH_STRUCT4(DO_TO_STRUCT)							\
																\
	FOR_SysPktSetBreakpointsCmdType2_STRUCT		(DO_TO_STRUCT)	\
	FOR_SysPktSetBreakpointsRspType_STRUCT		(DO_TO_STRUCT)	\
	FOR_SysPktSetTrapBreaksCmdType2_STRUCT		(DO_TO_STRUCT)	\
	FOR_SysPktSetTrapConditionsCmdType2_STRUCT	(DO_TO_STRUCT)	\
	FOR_SysPktSetTrapConditionsRspType_STRUCT	(DO_TO_STRUCT)	\
	FOR_SysPktStateCmdType_STRUCT				(DO_TO_STRUCT)	\
	FOR_SysPktStateRspType_STRUCT				(DO_TO_STRUCT)	\
	FOR_SysPktWriteMemCmdType2_STRUCT			(DO_TO_STRUCT)	\
	FOR_SysPktWriteMemRspType_STRUCT			(DO_TO_STRUCT)	\
	FOR_SysPktWriteRegsCmdType_STRUCT			(DO_TO_STRUCT)	\
	FOR_SysPktWriteRegsRspType_STRUCT			(DO_TO_STRUCT)	\

