/* -*- mode: C++; tab-width: 4 -*- */
/* ===================================================================== *\
	Copyright (c) 1998-2000 Palm, Inc. or its subsidiaries.
	All rights reserved.

	This file is part of the Palm OS Emulator.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
\* ===================================================================== */

#ifndef EmRegs_h
#define EmRegs_h

class SessionFile;
struct RGBType;

typedef uint32	(*ReadFunction)(emuptr address, int size);
typedef void	(*WriteFunction)(emuptr address, int size, uint32 value);

const uint32	kMemoryStart	= 0xFFFFF000;
const uint32	kMemorySize		= 0x00001000;

// Inline functions for reading/writing Dragonball registers.

inline int8		_get_reg (int8* a)				{ return do_get_mem_byte (a); }
inline int16	_get_reg (int16* a)				{ return do_get_mem_word (a); }
inline int32	_get_reg (int32* a)				{ return do_get_mem_long (a); }

inline uint8	_get_reg (uint8* a)				{ return do_get_mem_byte (a); }
inline uint16	_get_reg (uint16* a)			{ return do_get_mem_word (a); }
inline uint32	_get_reg (uint32* a)			{ return do_get_mem_long (a); }

inline void		_put_reg (int8* a, int8 v)		{ do_put_mem_byte (a, v); }
inline void		_put_reg (int16* a, int16 v)	{ do_put_mem_word (a, v); }
inline void		_put_reg (int32* a, int32 v)	{ do_put_mem_long (a, v); }

inline void		_put_reg (uint8* a, uint8 v)	{ do_put_mem_byte (a, v); }
inline void		_put_reg (uint16* a, uint16 v)	{ do_put_mem_word (a, v); }
inline void		_put_reg (uint32* a, uint32 v)	{ do_put_mem_long (a, v); }


class EmRegs
{
	public:
		virtual void			Initialize				(void);
		virtual void			Reset					(void);
		virtual void			Save					(SessionFile&);
		virtual void			Load					(SessionFile&);
		virtual void			Dispose					(void);

		uint32					GetLong					(emuptr address);
		uint32					GetWord					(emuptr address);
		uint32					GetByte					(emuptr address);
		void					SetLong					(emuptr address, uint32 value);
		void					SetWord					(emuptr address, uint32 value);
		void					SetByte					(emuptr address, uint32 value);

		int						ValidAddress			(emuptr address, uint32 iSize);
		virtual uint8*			GetRealAddress			(emuptr address) = 0;

		virtual void			SetSubHandlers			(void);
		void					SetHandlers				(ReadFunction,
														 WriteFunction,
														 emuptr start,
														 int32 count);
		ReadFunction			GetReadHandler			(emuptr address) const;
		WriteFunction			GetWriteHandler			(emuptr address) const;

		virtual void			Cycle					(Bool sleeping) = 0;
		virtual void			CycleSlowly				(Bool sleeping) = 0;

		virtual void			ButtonEvent				(Bool buttonIsDown, int32 button) = 0;
		virtual void			HotSyncEvent			(Bool buttonIsDown) = 0;

		virtual void			TurnSoundOff			(void) = 0;
		virtual void			ResetTimer				(void) = 0;
		virtual void			ResetRTC				(void) = 0;

		virtual int32			GetInterruptLevel		(void) = 0;
		virtual int32			GetInterruptBase		(void) = 0;
		virtual int32			GetLCDDepth				(void) = 0;
		virtual int32			GetLCDRowBytes			(void) = 0;
		virtual int32			GetLCDWidth				(void) = 0;
		virtual int32			GetLCDHeight			(void) = 0;
		virtual emuptr			GetLCDStartAddr			(void) = 0;
		virtual int32			GetLCDPalette			(RGBType*) = 0;
		virtual int32			GetLCDOffset			(void) = 0;
		virtual int32			GetDynamicHeapSize		(void) = 0;
		virtual int32			GetROMSize				(void) = 0;
		virtual emuptr			GetROMBaseAddress		(void) = 0;
		virtual Bool			ChipSelectsConfigured	(void) = 0;
		virtual int32			GetSystemClockFrequency	(void) = 0;
		virtual Bool			GetLCDOn				(void) = 0;
		virtual Bool			GetBacklightOn			(void) = 0;
		virtual Bool			GetSerialPortOn			(void) = 0;
		virtual Bool			GetCanStop				(void) = 0;

		virtual uint8			GetPortInputValue		(int) = 0;
		virtual uint8			GetPortInternalValue	(int) = 0;
		virtual void			PortDataChanged			(int, uint8, uint8) = 0;

	protected:
		static uint32			UnsupportedRead			(emuptr address, int size);
		static void				UnsupportedWrite		(emuptr address, int size, uint32 value);
		static void				NullWrite				(emuptr address, int size, uint32 value);	// For read-only registers

		static void				InvalidAccess			(emuptr iAddress, int size, Bool forRead);
};

#endif	/* EmRegs_h */
