/* -*- mode: C++; tab-width: 4 -*- */
/* ===================================================================== *\
	Copyright (c) 1998-2000 Palm, Inc. or its subsidiaries.
	All rights reserved.

	This file is part of the Palm OS Emulator.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
\* ===================================================================== */

#ifndef EmRegs328_h
#define EmRegs328_h

#include "EmRegs.h"				// EmRegs
#include "UART.h"				// UART::State

class EmRegs328 : public EmRegs
{
	public:
		virtual void			Initialize				(void);
		virtual void			Reset					(void);
		virtual void			Save					(SessionFile&);
		virtual void			Load					(SessionFile&);
		virtual void			Dispose					(void);

		virtual void			SetSubHandlers			(void);
		virtual uint8*			GetRealAddress			(emuptr address);

		virtual void			Cycle					(Bool sleeping);
		virtual void			CycleSlowly				(Bool sleeping);

		virtual void			ButtonEvent				(Bool buttonIsDown, int32 button);
		virtual void			HotSyncEvent			(Bool buttonIsDown);

		virtual void			TurnSoundOff			(void);
		virtual void			ResetTimer				(void);
		virtual void			ResetRTC				(void);

		virtual int32			GetInterruptLevel		(void);
		virtual int32			GetInterruptBase		(void);
		virtual int32			GetLCDDepth				(void);
		virtual int32			GetLCDRowBytes			(void);
		virtual int32			GetLCDWidth				(void);
		virtual int32			GetLCDHeight			(void);
		virtual emuptr			GetLCDStartAddr			(void);
		virtual int32			GetLCDPalette			(RGBType*);
		virtual int32			GetLCDOffset			(void);
		virtual Bool			GetLCDOn				(void) = 0;
		virtual Bool			GetBacklightOn			(void) = 0;
		virtual Bool			GetSerialPortOn			(void) = 0;
		virtual int32			GetDynamicHeapSize		(void);
		virtual int32			GetROMSize				(void);
		virtual emuptr			GetROMBaseAddress		(void);
		virtual Bool			ChipSelectsConfigured	(void);
		virtual int32			GetSystemClockFrequency	(void);
		virtual Bool			GetCanStop				(void);

		virtual uint8			GetPortInputValue		(int);
		virtual uint8			GetPortInternalValue	(int);
		virtual void			PortDataChanged			(int, uint8, uint8);

	protected:
		static uint32			StdRead					(emuptr address, int size);
		static void				StdWrite				(emuptr address, int size, uint32 value);

	private:
		static uint32			pllFreqSelRead			(emuptr address, int size);
		static uint32			portXDataRead			(emuptr address, int size);
		static uint32			tmr1StatusRead			(emuptr address, int size);
		static uint32			tmr2StatusRead			(emuptr address, int size);
		static uint32			uartRead				(emuptr address, int size);
		static uint32			rtcHourMinSecRead		(emuptr address, int size);

		static void				csASelect1Write			(emuptr address, int size, uint32 value);
		static void				csCSelect0Write			(emuptr address, int size, uint32 value);
		static void				csCSelect1Write			(emuptr address, int size, uint32 value);
		static void				intMaskHiWrite			(emuptr address, int size, uint32 value);
		static void				intMaskLoWrite			(emuptr address, int size, uint32 value);
		static void				intStatusHiWrite		(emuptr address, int size, uint32 value);
		static void				portXDataWrite			(emuptr address, int size, uint32 value);
		static void				portDIntReqEnWrite		(emuptr address, int size, uint32 value);
		static void				tmr1StatusWrite			(emuptr address, int size, uint32 value);
		static void				tmr2StatusWrite			(emuptr address, int size, uint32 value);
		static void				wdCounterWrite			(emuptr address, int size, uint32 value);
		static void				spiMasterControlWrite	(emuptr address, int size, uint32 value);
		static void				uartWrite				(emuptr address, int size, uint32 value);
		static void				lcdRegisterWrite		(emuptr address, int size, uint32 value);
		static void				rtcControlWrite			(emuptr address, int size, uint32 value);
		static void				rtcIntStatusWrite		(emuptr address, int size, uint32 value);
		static void				rtcIntEnableWrite		(emuptr address, int size, uint32 value);

	protected:
		virtual uint8			GetKeyBits				(void);

	protected:
		static void				UpdateInterrupts		(void);
		static void				UpdatePortDInterrupts	(void);
		static void				UpdateRTCInterrupts		(void);

	protected:
		static Bool				IDDetectAsserted		(void);
		static UInt8			GetHardwareID			(void);

	protected:
		static void				UARTStateChanged		(Bool sendTxData);
		static void				UpdateUARTState			(Bool refreshRxData);
		static void				UpdateUARTInterrupts	(const UART::State& state);
		static void				MarshalUARTState		(UART::State& state);
		static void				UnmarshalUARTState		(const UART::State& state);

	protected:
		static int				GetPort					(emuptr address);

	protected:
		static HwrM68328Type	fg68328Regs;
		static bool				fgHotSyncButtonDown;
		static uint32			fgTmr2CurrentMilliseconds;
		static uint32			fgTmr2StartMilliseconds;
		static uint16			fgKeyBits;
		static uint16			fgLastTmr1Status;
		static uint16			fgLastTmr2Status;
		static uint8			fgPortDEdge;
};

#endif	/* EmRegs328_h */
