/* -*- mode: C++; tab-width: 4 -*- */
/* ===================================================================== *\
	Copyright (c) 1998-2000 Palm, Inc. or its subsidiaries.
	All rights reserved.

	This file is part of the Palm OS Emulator.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
\* ===================================================================== */

#include "EmCommon.h"
#include "EmRegsEZPalmIIIc.h"
#include "EmRegsEZPrv.h"

#include "Bank_SED1375.h"		// SED1375 video controller register access

#include "PalmPack.h"
#define NON_PORTABLE
	#include "EZAustin/IncsPrv/HardwareAustin.h"	// hwrEZPortCKbdRow0, hwrEZPortBRS232Enable, etc.
#undef NON_PORTABLE
#include "PalmPackPop.h"

const int		kNumButtonRows = 3;
const int		kNumButtonCols = 4;

const uint16	kButtonMap[kNumButtonRows][kNumButtonCols] =
{
	{ keyBitHard1,	keyBitHard2,	keyBitHard3,	keyBitHard4 },
	{ keyBitPageUp,	keyBitPageDown,	0,				0 },
	{ keyBitPower,	keyBitContrast,	0,				0 }
};


// ---------------------------------------------------------------------------
//		 EmRegsEZPalmIIIc::GetLCDDepth
// ---------------------------------------------------------------------------

int32 EmRegsEZPalmIIIc::GetLCDDepth (void)
{
	return SED1375Bank::GetLCDDepth ();
}


// ---------------------------------------------------------------------------
//		 EmRegsEZPalmIIIc::GetLCDRowBytes
// ---------------------------------------------------------------------------

int32 EmRegsEZPalmIIIc::GetLCDRowBytes (void)
{
	return SED1375Bank::GetLCDRowBytes ();
}


// ---------------------------------------------------------------------------
//		 EmRegsEZPalmIIIc::GetLCDWidth
// ---------------------------------------------------------------------------

int32 EmRegsEZPalmIIIc::GetLCDWidth (void)
{
	return SED1375Bank::GetLCDWidth ();
}


// ---------------------------------------------------------------------------
//		 EmRegsEZPalmIIIc::GetLCDHeight
// ---------------------------------------------------------------------------

int32 EmRegsEZPalmIIIc::GetLCDHeight (void)
{
	return SED1375Bank::GetLCDHeight ();
}


// ---------------------------------------------------------------------------
//		 EmRegsEZPalmIIIc::GetLCDStartAddr
// ---------------------------------------------------------------------------

emuptr EmRegsEZPalmIIIc::GetLCDStartAddr (void)
{
	return SED1375Bank::GetLCDStartAddr ();
}


// ---------------------------------------------------------------------------
//		 EmRegsEZPalmIIIc::GetLCDPalette
// ---------------------------------------------------------------------------

int32 EmRegsEZPalmIIIc::GetLCDPalette (RGBType* thePalette)
{
	return SED1375Bank::GetLCDPalette (thePalette);
}


// ---------------------------------------------------------------------------
//		 EmRegsEZPalmIIIc::GetLCDOffset
// ---------------------------------------------------------------------------

int32 EmRegsEZPalmIIIc::GetLCDOffset (void)
{
	return SED1375Bank::GetLCDOffset ();
}


// ---------------------------------------------------------------------------
//		 EmRegsEZPalmIIIc::GetLCDOn
// ---------------------------------------------------------------------------

Bool EmRegsEZPalmIIIc::GetLCDOn (void)
{
	return SED1375Bank::GetLCDOn ();
}


// ---------------------------------------------------------------------------
//		 EmRegsEZPalmIIIc::GetBacklightOn
// ---------------------------------------------------------------------------

Bool EmRegsEZPalmIIIc::GetBacklightOn (void)
{
	return SED1375Bank::GetBacklightOn ();
}


// ---------------------------------------------------------------------------
//		 EmRegsEZPalmIIIc::GetSerialPortOn
// ---------------------------------------------------------------------------

Bool EmRegsEZPalmIIIc::GetSerialPortOn (void)
{
	return (READ_REGISTER (portBData) & hwrEZPortBRS232Enable) != 0;
}


// ---------------------------------------------------------------------------
//		 EmRegsEZPalmIIIc::GetPortInputValue
// ---------------------------------------------------------------------------

uint8 EmRegsEZPalmIIIc::GetPortInputValue (int port)
{
	uint8	result = EmRegsEZ::GetPortInputValue (port);

	if (port == 'F')
	{
		// Make sure this is always set, or HwrDisplayWake will hang

		result |= hwrEZPortFLCDPowered;
	}

	return result;
}


// ---------------------------------------------------------------------------
//		 EmRegsEZPalmIIIc::GetPortInternalValue
// ---------------------------------------------------------------------------

uint8 EmRegsEZPalmIIIc::GetPortInternalValue (int port)
{
	uint8	result = EmRegsEZ::GetPortInternalValue (port);

	if (port == 'D')
	{
		// Ensure that bit hwrEZPortDDockButton is set.  If it's clear, HotSync
		// will sync via the modem instead of the serial port.
		//
		// Also make sure that hwrEZPortDPowerFail is set.  If it's clear,
		// the battery code will make the device go to sleep immediately.

		result |= hwrEZPortDDockButton | hwrEZPortDPowerFail;
	}

	return result;
}


// ---------------------------------------------------------------------------
//		 EmRegsEZPalmIIIc::GetKeyInfo
// ---------------------------------------------------------------------------

void EmRegsEZPalmIIIc::GetKeyInfo (int* numRows, int* numCols,
								uint16* keyMap, Bool* rows)
{
	*numRows = kNumButtonRows;
	*numCols = kNumButtonCols;

	memcpy (keyMap, kButtonMap, sizeof (kButtonMap));

	// Determine what row is being asked for.

	UInt8	portCDir	= READ_REGISTER (portCDir);
	UInt8	portCData	= READ_REGISTER (portCData);

	rows[0]	= (portCDir & hwrEZPortCKbdRow0) != 0 && (portCData & hwrEZPortCKbdRow0) == 0;
	rows[1]	= (portCDir & hwrEZPortCKbdRow1) != 0 && (portCData & hwrEZPortCKbdRow1) == 0;
	rows[2]	= (portCDir & hwrEZPortCKbdRow2) != 0 && (portCData & hwrEZPortCKbdRow2) == 0;
}
