/* -*- mode: C++; tab-width: 4 -*- */
/* ===================================================================== *\
	Copyright (c) 1998-2000 Palm, Inc. or its subsidiaries.
	All rights reserved.

	This file is part of the Palm OS Emulator.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
\* ===================================================================== */

#include "EmCommon.h"
#include "EmUAEGlue.h"

#include "CPU_REG.h"


// ---------------------------------------------------------------------------
// Glue functions to bridge from UAE-generated code to the implemenations
// we define.
// ---------------------------------------------------------------------------

void customreset (void)
{
	Emulator::Reset ();
}


void Exception (int nr, emuptr oldpc)
{
	emuptr curpc = m68k_getpc ();

	// Some opcode handlers (like op_4e40 for the TRAP #xx functions)
	// call Exception with oldpc == 0. We really need the PC so that
	// we can look up dispatch numbers, so provide that here.

	if (oldpc == 0)
	{
		if (nr >= kException_Trap0 && nr <= kException_Trap15)
		{
			// pc was bumped by 2 already in op_4e40 handler.

			oldpc = curpc - 2;
		}
		else
		{
			assert (false);
		}
	}

	Software::ProcessException (nr, oldpc, curpc);
}


unsigned long op_illg (uint32 iOpcode)
{
	Software::ProcessIllegalInstruction (iOpcode);
	return 0;
}


void Software_ProcessLINK (int32 linkSize)
{
	Software::ProcessLINK (linkSize);
}


int Software_ProcessRTS (emuptr dest)
{
#if CHECK_STACK_INFO
	if (0)
	{
		StackAddress	stackAddr = m68k_areg(regs, 7);
		ReturnMap::iterator	iter = gReturnMap.find (stackAddr);
		if (iter != gReturnMap.end())
		{
			gReturnMap.erase (iter);
		}
		else
		{
			LogAppendMsg ("Failed to pop return info: stackAddr = 0x%08X, ", stackAddr);
			LogDump ();
		}
	}
#endif

	Software::CheckNewPC (dest);
	return Software::ProcessRTS ();
}


int Software_ProcessRTE (emuptr dest)
{
#if CHECK_STACK_INFO
	if (1)
	{
		StackAddress	stackAddr = m68k_areg(regs, 7) - 6;
		ExceptionMap::iterator	iter = gExceptionMap.find (stackAddr);
		if (iter != gExceptionMap.end())
		{
			LogAppendMsg ("Popping exception info: stack = 0x%08X, retAddr = 0x%08X, sr = 0x%04X",
				stackAddr, iter->second.first, iter->second.second);
			LogDump ();

			gExceptionMap.erase (iter);
		}
		else
		{
			LogAppendMsg ("Failed to pop exception info: stackAddr = 0x%08X, ", stackAddr);
			LogDump ();
		}
	}
#endif

	Software::CheckNewPC (dest);
	return Software::ProcessRTE (dest);
}


int Software_ProcessJSR (emuptr oldpc, emuptr dest)
{
#if CHECK_STACK_INFO
	if (0)
	{
		gReturnMap[m68k_areg(regs, 7) - 4] = oldpc;
	}
#endif

	Software::CheckNewPC (dest);
	return Software::ProcessJSR (oldpc, dest);
}


int Software_ProcessJSR_Ind (emuptr oldpc, emuptr dest)
{
	return Software::ProcessJSR_Ind (oldpc, dest);
}


void Software_CheckNewPC (emuptr dest)
{
	Software::CheckNewPC (dest);
}


void Software_CheckStackPointerAssignment ()
{
	Software::CheckStackPointerAssignment ();
}


void Software_CheckStackPointerDecrement ()
{
	Software::CheckStackPointerDecrement ();
}


void Software_CheckStackPointerIncrement ()
{
//	Software::CheckStackPointerIncrement ();
}


void MakeSR (void)
{
	Registers::UpdateSRFromRegisters ();
}


void MakeFromSR (void)
{
	Registers::UpdateRegistersFromSR ();
}


void* xmalloc (size_t size)
{
	return Platform::AllocateMemory (size);
}

uint32 get_disp_ea_000 (uint32 base, uint32 dp)
{
    int reg = (dp >> 12) & 15;
    uae_s32 regd = regs.regs[reg];

    if ((dp & 0x800) == 0)
	regd = (uae_s32)(uae_s16)regd;
    return base + (uae_s8)dp + regd;
}
