/******************************************************************************
 *
 * Copyright (c) 1994-1998 Palm Computing, Inc. or its subsidiaries.
 * All rights reserved.
 *
 * File: KeyMgr328JerryPrv.h
 *
 * Description:
 *		Private Include file for Key manager
 *
 * History:
 *   	9/13/95 		Created by Ron Marianetti
 *    2/04/98 srj	Added modifications to keymgr globals (keyState and 
 *						associated variables are now words) 
 *						added private routine prototypes for keyboard scanning
 *						on Razor.
 *		3/31/98 		Added support for the Antenna key on Jerry
 *		8/30/98 scl Reordered KeyGlobalsType to match 3.1, for better
 *						alignment, and to more closely match older OS revs
 *
 *****************************************************************************/

#ifdef	NON_PORTABLE
#ifndef __KEYMGR328JERRYPRV_H
#define __KEYMGR328JERRYPRV_H

#include <HAL.h>


/********************************************************************
 * Key Manager Constants
 ********************************************************************/
#define	keyMinUpTicks					(sysTicksPerSecond / 20)// Min# of ticks key must be up to recognize it

#define	keyMinAntennaTicks			(sysTicksPerSecond / 2)	// Min# of ticks key antenna must be up to
																				// generate antenna event

// The following is obsolete in EZ, as port D isn't a key map anymore.

#define	keyPortDMaskPower			0x01	// Power key
#define	keyPortDMaskPageUp		0x02	// Page-up
#define	keyPortDMaskPageDown		0x04	// Page-down
#define	keyPortDMaskHard1			0x08	// App #1
#define	keyPortDMaskHard2			0x10	// App #2
#define	keyPortDMaskHard3			0x20	// App #3
#define	keyPortDMaskHard4			0x40	// App #4
#define	keyPortDMaskAntenna		0x80	// The bit in port D for the antenna. 

#define	keyPortDMaskStd			0x7F	// Which bits in port D we use for standard keys
#define	keyBitMaskStd				0x7F  // These map 1 to 1 to the keyBitXXX
													//  equates returned by KeyCurrentState defined
													//  in KeyMgr.h

#define	keyBitAllKeys					0x01FF	// Which bits in keyState are 
															// actually used for keys

/********************************************************************
 * Key Manager Globals
 ********************************************************************/
typedef struct KeyGlobalsType {
	UInt32	lastKeyTicks;					// GHwrCurTicks last time we enqueued a key event;
													//  cleared as soon as key is released.

	UInt8		keyStateV30;					// OBSOLETE: current state of hardware keys
													//  (kept for SW compatibility with old apps)
	UInt8		keyStateMaskV30;				// OBSOLETE: which keys generate an event


	UInt8		keyUpCount;						// # of consecutive times we've detected
													// no keys down - used for debouncing
	UInt8		initDelay;						// key repeat init delay in ticks


	UInt8		period;							// key repeat period in ticks
	Boolean	queueAhead;						// if true, enqueue auto-repeat keys even if 
													// queue is not empty. (Moved in 2.0 and 3.1)

	UInt32	turnOnTicks;					// Tickcount for key that turned device on.  Used
													// for detecting when we should turn on backlight.
	
	// These fields are for sensing double-taps
	UInt32	doubleTapTicks;				// GHwrCurTicks last time we enqueued
													//  a key event - never cleared. 
	UInt16	doubleTapDelay;				// double-tap timeout in ticks

	UInt8		doubleTapStateV30;			// OBSOLETE: state of hardware keys last time (for compat.)

	UInt8		unused;							// filler for alignment; feel free to use
	UInt8		usedPortDBits;					// 3.1: set to keyBitMaskStd if no antenna key
													//      set to (keyBitMaskStd | keyBitMaskAntenna) if 
													//      antenna key is available. (new in 3.2)
	UInt8		antKeyPendCount;				// used as a counter and flag to signal pending antenna up event

	// PalmOS 3.1: These fields are new, wider fields that can handle more than eight keys.
	// The byte fields (labeled "V30" above) are shadowed for compatibility w/old apps.
	// However, apps should *NOT* be reading these fields directly!!
	UInt32	keyState;						// 3.1: current state of hardware keys
													//      returned by KeyCurrentState
	UInt32	keyStateMask;					// 3.1: which keys generate an event
	UInt32	doubleTapState;				// 3.1: state of hardware keys last time
	
	} KeyGlobalsType;
	
typedef KeyGlobalsType*	KeyGlobalsPtr;





/********************************************************************
 * Key manager Routines
 ********************************************************************/
#ifdef __cplusplus
extern "C" {
#endif


/********************************************************************
 * Private Routines
 ********************************************************************/

// 2 uS delay routine
void	prvDelay2us(void);

// Enable/Disable keyboard interrupts
void	prvEnableKeyInterrupts(Boolean);
void	prvDisableKeyInterrupts(void);

// Scan Keyboard
UInt32 prvKeyScan(void);
UInt32 PrvScanKeyStatus(void);

#ifdef __cplusplus
}
#endif

	
#endif	//__KEYMGR328JERRYPRV_H
#endif 	// NON_PORTABLE
