/* -*- mode: C++; tab-width: 4 -*- */
/* ===================================================================== *\
	Copyright (c) 1998-2000 Palm, Inc. or its subsidiaries.
	All rights reserved.

	This file is part of the Palm OS Emulator.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
\* ===================================================================== */

#ifndef _SKINS_H_
#define _SKINS_H_

#include "EmulatorTypes.h"		// DeviceType, ScaleType

class EmStream;

enum SkinElementType
{
	kElement_First,

	kElement_PowerButton = kElement_First,
	kElement_UpButton,
	kElement_DownButton,
	kElement_App1Button,
	kElement_App2Button,
	kElement_App3Button,
	kElement_App4Button,
	kElement_CradleButton,
	kElement_Antenna,
	kElement_ContrastButton,

		// Symbol-specific
	kElement_TriggerLeft,
	kElement_TriggerCenter,
	kElement_TriggerRight,
	kElement_UpButtonLeft,
	kElement_UpButtonRight,
	kElement_DownButtonLeft,
	kElement_DownButtonRight,

	kElement_Touchscreen,
	kElement_LCD,

	kElement_NumElements,

	kElement_Frame	= -1,
	kElement_None	= -2
};

	// Pre-increment operator
inline void	operator++(SkinElementType& x)
{
	x = SkinElementType (x + 1);
}

	// Post-increment operator
inline void	operator++(SkinElementType& x, int)
{
	x = SkinElementType (x + 1);
}

	// Pre-decrement operator
inline void	operator--(SkinElementType& x)
{
	x = SkinElementType (x - 1);
}

	// Post-decrement operator
inline void	operator--(SkinElementType& x, int)
{
	x = SkinElementType (x - 1);
}


typedef string				SkinName;
typedef vector<SkinName>	SkinNameList;


SkinName		SkinGetSkinName			(DeviceType);
SkinName		SkinGetDefaultSkinName	(DeviceType);
void			SkinGetSkinNames		(DeviceType, SkinNameList&);

void			SkinSetSkin				(void);
void			SkinSetSkin				(DeviceType, ScaleType scale, const SkinName&);

EmFileRef		SkinGetSkinfoFile		(void);
EmFileRef		SkinGetSkinFile			(void);
EmFileRef		SkinGetSkinFile			(ScaleType scale);

EmStream*		SkinGetSkinStream		(void);
EmStream*		SkinGetSkinStream		(ScaleType scale);

Bool			SkinValidSkin			(DeviceType, const SkinName&);

RGBType			SkinGetBackgroundColor	(void);
RGBType			SkinGetHighlightColor	(void);

SkinElementType	SkinTestPoint			(const PointType& windowPt);
PointType		SkinWindowToTouchscreen	(const PointType& windowPt);
PointType		SkinTouchscreenToWindow	(const PointType& lcdPt);
Bool			SkinGetElementInfo		(int index, SkinElementType&, RectangleType&);
Bool			SkinGetElementInfo		(int index, SkinElementType&, AbsRectType&);

PointType		SkinScaleDown			(const PointType&);
RectangleType	SkinScaleDown			(const RectangleType&);
AbsRectType		SkinScaleDown			(const AbsRectType&);

PointType		SkinScaleUp				(const PointType&);
RectangleType	SkinScaleUp				(const RectangleType&);
AbsRectType		SkinScaleUp				(const AbsRectType&);

#endif	// _SKINS_H_
