/* -*- mode: C++; tab-width: 4 -*- */
/* ===================================================================== *\
	Copyright (c) 1998-2000 Palm, Inc. or its subsidiaries.
	All rights reserved.

	This file is part of the Palm OS Emulator.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
\* ===================================================================== */

#include "EmCommon.h"
#include "TrapPatches.h"

#include "Logging.h"			// LogAppendMsg
#include "Marshal.h"			// PARAM_VAL, etc.
#include "Platform_FfsLib.h"	// Platform_NetLib
#include "RAM_ROM.h"			// CEnableFullAccess


/***************************************************************************
 * Ffs library function trap ID's. Each library call gets a trap number:
 *   FfsLibTrapXXXX which serves as an index into the library's dispatch
 *   table. The constant sysLibTrapCustom is the first available trap number
 *   after the system predefined library traps Open,Close,Sleep & Wake.
 *
 * WARNING!!! The order of these traps MUST match the order of the dispatch
 *  table in FfsLibDispatch.c!!!
 ****************************************************************************/

typedef enum
{
	FfsLibTrapGetLibAPIVersion = sysLibTrapCustom,
	FfsLibTrapGetdiskfree,
	FfsLibTrapFindfirst,
	FfsLibTrapFindnext,
	FfsLibTrapFinddone,
	FfsLibTrapFileOpen,
	FfsLibTrapFileClose,
	FfsLibTrapRead,
	FfsLibTrapWrite,
	FfsLibTrapRemove,
	FfsLibTrapChdir,
	FfsLibTrapGetcwd,
	FfsLibTrapMkdir,
	FfsLibTrapRmdir,
	FfsLibTrapFormat,
	FfsLibTrapRename,
	FfsLibTrapGetfileattr,
	FfsLibTrapFlush,
	FfsLibTrapFlushDisk,
	FfsLibTrapSetfileattr,
	FfsLibTrapStat,
	FfsLibTrapFstat,
	FfsLibTrapIsDir,
	FfsLibTrapLseek,
	FfsLibTrapGetErrno,
	FfsLibTrapTell,
	FfsLibTrapGetdrive,
	FfsLibTrapSetdrive,
	FfsLibTrapUnlink,
	FfsLibTrapEof,
	FfsLibTrapCreat,
	FfsLibTrapInstErrHandle,
	FfsLibTrapUnInstErrHandle,
	FfsLibTrapSetDebuggingOn,
	FfsLibTrapSetDebuggingOff,
	FfsLibTrapCardIsInserted,
	FfsLibTrapExerciseFAT,
	FfsLibTrapCardIsATA,
	FfsLibTrapChangeOccurred,
	FfsLibTrapLast
} FfsLibTrapNumberEnum;


// ======================================================================
//	Patches for FfsLib functions
// ======================================================================

class FfsLibHeadpatch
{
	public:
		static CallROMType		FfsLibOpen(void);
		static CallROMType		FfsLibClose(void);
		static CallROMType		FfsLibSleep(void);
		static CallROMType		FfsLibWake(void);

		static CallROMType		FfsLibTrapGetLibAPIVersion(void);
		static CallROMType		FfsLibTrapGetdiskfree(void);
		static CallROMType		FfsLibTrapFindfirst(void);
		static CallROMType		FfsLibTrapFindnext(void);
		static CallROMType		FfsLibTrapFinddone(void);
		static CallROMType		FfsLibTrapOpen(void);
		static CallROMType		FfsLibTrapClose(void);
		static CallROMType		FfsLibTrapRead(void);
		static CallROMType		FfsLibTrapWrite(void);
		static CallROMType		FfsLibTrapRemove(void);
		static CallROMType		FfsLibTrapChdir(void);
		static CallROMType		FfsLibTrapGetcwd(void);
		static CallROMType		FfsLibTrapMkdir(void);
		static CallROMType		FfsLibTrapRmdir(void);
		static CallROMType		FfsLibTrapFormat(void);
		static CallROMType		FfsLibTrapRename(void);
		static CallROMType		FfsLibTrapGetfileattr(void);
		static CallROMType		FfsLibTrapFlush(void);
		static CallROMType		FfsLibTrapFlushDisk(void);
		static CallROMType		FfsLibTrapSetfileattr(void);
		static CallROMType		FfsLibTrapStat(void);
		static CallROMType		FfsLibTrapFstat(void);
		static CallROMType		FfsLibTrapIsDir(void);
		static CallROMType		FfsLibTrapLseek(void);
		static CallROMType		FfsLibTrapGetErrno(void);
		static CallROMType		FfsLibTrapTell(void);
		static CallROMType		FfsLibTrapGetdrive(void);
		static CallROMType		FfsLibTrapSetdrive(void);
		static CallROMType		FfsLibTrapUnlink(void);
		static CallROMType		FfsLibTrapEof(void);
		static CallROMType		FfsLibTrapCreat(void);
		static CallROMType		FfsLibTrapInstErrHandle(void);
		static CallROMType		FfsLibTrapUnInstErrHandle(void);
		static CallROMType		FfsLibTrapSetDebuggingOn(void);
		static CallROMType		FfsLibTrapSetDebuggingOff(void);
		static CallROMType		FfsLibTrapCardIsInserted(void);
		static CallROMType		FfsLibTrapCardIsATA(void);
};


// ======================================================================
//	Proto patch table for FfsLib functions.  This array will be used
//	to create a sparse array at runtime.
// ======================================================================

ProtoPatchTableEntry	gProtoFfsLibPatchTable[] =
{
	{sysLibTrapOpen,				FfsLibHeadpatch::FfsLibOpen,					NULL},
	{sysLibTrapClose,				FfsLibHeadpatch::FfsLibClose,					NULL},
	{sysLibTrapSleep,				FfsLibHeadpatch::FfsLibSleep,					NULL},
	{sysLibTrapWake,				FfsLibHeadpatch::FfsLibWake,					NULL},

	{FfsLibTrapGetLibAPIVersion,	FfsLibHeadpatch::FfsLibTrapGetLibAPIVersion,	NULL},
	{FfsLibTrapGetdiskfree,			FfsLibHeadpatch::FfsLibTrapGetdiskfree,			NULL},
	{FfsLibTrapFindfirst,			FfsLibHeadpatch::FfsLibTrapFindfirst,			NULL},
	{FfsLibTrapFindnext,			FfsLibHeadpatch::FfsLibTrapFindnext,			NULL},
	{FfsLibTrapFinddone,			FfsLibHeadpatch::FfsLibTrapFinddone,			NULL},
	{FfsLibTrapFileOpen,			FfsLibHeadpatch::FfsLibTrapOpen,				NULL},
	{FfsLibTrapFileClose,			FfsLibHeadpatch::FfsLibTrapClose,				NULL},
	{FfsLibTrapRead,				FfsLibHeadpatch::FfsLibTrapRead,				NULL},
	{FfsLibTrapWrite,				FfsLibHeadpatch::FfsLibTrapWrite,				NULL},
	{FfsLibTrapRemove,				FfsLibHeadpatch::FfsLibTrapRemove,				NULL},
	{FfsLibTrapChdir,				FfsLibHeadpatch::FfsLibTrapChdir,				NULL},
	{FfsLibTrapGetcwd,				FfsLibHeadpatch::FfsLibTrapGetcwd,				NULL},
	{FfsLibTrapMkdir,				FfsLibHeadpatch::FfsLibTrapMkdir,				NULL},
	{FfsLibTrapRmdir,				FfsLibHeadpatch::FfsLibTrapRmdir,				NULL},
	{FfsLibTrapFormat,				FfsLibHeadpatch::FfsLibTrapFormat,				NULL},
	{FfsLibTrapRename,				FfsLibHeadpatch::FfsLibTrapRename,				NULL},
	{FfsLibTrapGetfileattr,			FfsLibHeadpatch::FfsLibTrapGetfileattr,			NULL},
	{FfsLibTrapFlush,				FfsLibHeadpatch::FfsLibTrapFlush,				NULL},
	{FfsLibTrapFlushDisk,			FfsLibHeadpatch::FfsLibTrapFlushDisk,			NULL},
	{FfsLibTrapSetfileattr,			FfsLibHeadpatch::FfsLibTrapSetfileattr,			NULL},
	{FfsLibTrapStat,				FfsLibHeadpatch::FfsLibTrapStat,				NULL},
	{FfsLibTrapFstat,				FfsLibHeadpatch::FfsLibTrapFstat,				NULL},
	{FfsLibTrapIsDir,				FfsLibHeadpatch::FfsLibTrapIsDir,				NULL},
	{FfsLibTrapLseek,				FfsLibHeadpatch::FfsLibTrapLseek,				NULL},
	{FfsLibTrapGetErrno,			FfsLibHeadpatch::FfsLibTrapGetErrno,			NULL},
	{FfsLibTrapTell,				FfsLibHeadpatch::FfsLibTrapTell,				NULL},
	{FfsLibTrapGetdrive,			FfsLibHeadpatch::FfsLibTrapGetdrive,			NULL},
	{FfsLibTrapSetdrive,			FfsLibHeadpatch::FfsLibTrapSetdrive,			NULL},
	{FfsLibTrapUnlink,				FfsLibHeadpatch::FfsLibTrapUnlink,				NULL},
	{FfsLibTrapEof,					FfsLibHeadpatch::FfsLibTrapEof,					NULL},
	{FfsLibTrapCreat,				FfsLibHeadpatch::FfsLibTrapCreat,				NULL},
	{FfsLibTrapInstErrHandle,		FfsLibHeadpatch::FfsLibTrapInstErrHandle,		NULL},
	{FfsLibTrapUnInstErrHandle,		FfsLibHeadpatch::FfsLibTrapUnInstErrHandle,		NULL},
	{FfsLibTrapSetDebuggingOn,		FfsLibHeadpatch::FfsLibTrapSetDebuggingOn,		NULL},
	{FfsLibTrapSetDebuggingOff,		FfsLibHeadpatch::FfsLibTrapSetDebuggingOff,		NULL},
	{FfsLibTrapCardIsInserted,		FfsLibHeadpatch::FfsLibTrapCardIsInserted,		NULL},
	{FfsLibTrapCardIsATA,			FfsLibHeadpatch::FfsLibTrapCardIsATA,			NULL},
	{0,								NULL,											NULL}
};


#define PARAMETER_SIZE(x)	\
	(sizeof (((StackFrame*) 0)->x))

#define PARAMETER_OFFSET(x) \
	(m68k_areg (regs, 7) + offsetof (StackFrame, x))

#define GET_PARAMETER(x)		\
	((PARAMETER_SIZE(x) == sizeof (char)) ? get_byte (PARAMETER_OFFSET(x)) :	\
	 (PARAMETER_SIZE(x) == sizeof (short)) ? get_word (PARAMETER_OFFSET(x)) :	\
											get_long (PARAMETER_OFFSET(x)))
#define SET_PARAMETER(x, v) 	\
	((PARAMETER_SIZE(x) == sizeof (char)) ? put_byte (PARAMETER_OFFSET(x), v) : \
	 (PARAMETER_SIZE(x) == sizeof (short)) ? put_word (PARAMETER_OFFSET(x), v) :	\
											put_long (PARAMETER_OFFSET(x), v))

// ======================================================================
//	Private functions
// ======================================================================


#define PRINTF	if (1) ; else LogAppendMsg


// ========================================================================
// The following functions define a bunch of StackFrame structs.
// These structs need to mirror the format of parameters pushed
// onto the stack by the emulated code, and so need to be packed
// to 2-byte boundaries.
//
// The pragmas are reversed at the end of the file.
// ========================================================================

#include "PalmPack.h"


#pragma mark -


/***********************************************************************
 *
 * FUNCTION:	FfsLibOpen()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibOpen (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibOpen");

	struct StackFrame
	{
		UInt16	libRefNum;
	};

	PARAM_VAL(UInt16, libRefNum);

	m68k_dreg (regs, 0) = Platform_FfsLib::LibOpen (libRefNum);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibClose()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibClose (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibClose");

	struct StackFrame
	{
		UInt16	libRefNum;
	};

	PARAM_VAL(UInt16, libRefNum);

	m68k_dreg (regs, 0) = Platform_FfsLib::LibClose (libRefNum);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibSleep()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibSleep (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibSleep");

	struct StackFrame
	{
		UInt16	libRefNum;
	};

	PARAM_VAL(UInt16, libRefNum);

	m68k_dreg (regs, 0) = Platform_FfsLib::LibSleep (libRefNum);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibWake()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibWake (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibWake");

	struct StackFrame
	{
		UInt16	libRefNum;
	};

	PARAM_VAL(UInt16, libRefNum);

	m68k_dreg (regs, 0) = Platform_FfsLib::LibWake (libRefNum);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapGetLibAPIVersion()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapGetLibAPIVersion (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapGetLibAPIVersion");

	struct StackFrame
	{
		UInt16	libRefNum;
		UInt32*	dwVerP;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_REF(UInt32, dwVerP, Marshal::kOutput);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsGetLibAPIVersion (libRefNum, dwVerP);

	dwVerP.Put();

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapGetdiskfree()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapGetdiskfree (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapGetdiskfree");

	struct StackFrame
	{
		UInt16			libRefNum;
		UInt8			drive;
		ffs_diskfree_t*	dtable;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_VAL(UInt8,  drive);
	PARAM_REF(ffs_diskfree_t,  dtable, Marshal::kOutput);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsGetdiskfree (libRefNum, drive, dtable);

	dtable.Put();

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapFindfirst()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapFindfirst (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapFindfirst");

	struct StackFrame
	{
		UInt16		libRefNum;
		char*		path;
		Int16		attrib;
		ffs_ffblk*	ff_blk;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_STR(Char, path);
	PARAM_VAL(Int16, attrib);
	PARAM_REF(ffs_ffblk, ff_blk, Marshal::kOutput);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsFindfirst (libRefNum, path, attrib & 0xFFFF, ff_blk);

	ff_blk.Put();

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapFindnext()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapFindnext (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapFindnext");

	struct StackFrame
	{
		UInt16		libRefNum;
		ffs_ffblk*	ff_blk;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_REF(ffs_ffblk, ff_blk, Marshal::kOutput);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsFindnext (libRefNum, ff_blk);

	ff_blk.Put();

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapFinddone()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapFinddone (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapFinddone");

	struct StackFrame
	{
		UInt16		libRefNum;
		ffs_ffblk*	ff_blk;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_REF(ffs_ffblk, ff_blk, Marshal::kInOut);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsFinddone (libRefNum, ff_blk);

	ff_blk.Put();

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapOpen()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapOpen (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapOpen");

	struct StackFrame
	{
		UInt16	libRefNum;
		char*	path;
		Int16	flags;
		Int16	mode;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_STR(Char,   path);
	PARAM_VAL(Int16,  flags);
	PARAM_VAL(Int16,  mode);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsOpen (libRefNum, path, flags & 0xFFFF, mode & 0xFFFF);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapFileClose()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapClose (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapClose");

	struct StackFrame
	{
		UInt16	libRefNum;
		Int16	handle;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_VAL(Int16, handle);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsClose (libRefNum, handle);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapRead()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapRead (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapRead");

	struct StackFrame
	{
		UInt16	libRefNum;
		Int16	handle;
		void*	buffer;
		Int16	num_bytes;
	};


	UInt16	libRefNum	= GET_PARAMETER(libRefNum);
	Int16	handle		= GET_PARAMETER(handle);
	emuptr	buffer		= GET_PARAMETER(buffer);
	Int16	num_bytes	= GET_PARAMETER(num_bytes) & 0xFFFF;
	Int16	retval;
	void*	tmp;


	if ((tmp = Platform::AllocateMemory ((UInt16) num_bytes)) == NULL)
	{
		m68k_dreg (regs, 0) = -1;
	}
	else
	{
		if ((retval = Platform_FfsLib::FfsRead (libRefNum, handle, tmp, (UInt16) num_bytes)) == -1)
			m68k_dreg (regs, 0) = -1;
		else
			m68k_dreg (regs, 0) = retval & 0xFFFF;

		uae_memcpy (buffer, tmp, (UInt16) num_bytes);
		Platform::DisposeMemory (tmp);
	}

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapWrite()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapWrite (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapWrite");

	struct StackFrame
	{
		UInt16	libRefNum;
		Int16	handle;
		void*	buffer;
		Int16	num_bytes;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_VAL(Int16,  handle);
	PARAM_VAL(Int16,  num_bytes);
	PARAM_PTR(void,   buffer, num_bytes & 0xFFFF, Marshal::kInOut);

	Int16 retval;

	if ((retval = Platform_FfsLib::FfsWrite (libRefNum, handle, buffer, num_bytes & 0xFFFF)) == -1)
		m68k_dreg (regs, 0) = -1;
	else
		m68k_dreg (regs, 0) = retval & 0xFFFF;

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapRemove()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapRemove (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapRemove");

	struct StackFrame
	{
		UInt16	libRefNum;
		char*	path;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_STR(Char,   path);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsRemove (libRefNum, path);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapChdir()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROMnothing
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapChdir (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapChdir");

	struct StackFrame
	{
		UInt16	libRefNum;
		char*	path;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_STR(Char, path);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsChdir (libRefNum, path);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapGetcwd()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapGetcwd (void)
{
	char   tmpbuf[256];
	char   *tmptr;
	emuptr emutmp;

	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapGetcwd");

	struct StackFrame
	{
		UInt16	libRefNum;
		char*	path;
		Int16	numchars;
	};


	UInt16 libRefNum	= GET_PARAMETER(libRefNum);
	emuptr path			= GET_PARAMETER(path);
	Int16  numchars		= GET_PARAMETER(numchars) & 0xFFFF;

/*---------------------------------------------------------------------------
 * If path is NULL, Getcwd() will allocate space for the path.
 *--------------------------------------------------------------------------*/
	if (path == UAE_NULL)
	{
		tmptr  = Platform_FfsLib::FfsGetcwd(libRefNum, NULL, numchars);
		emutmp = (emuptr) MemPtrNew (strlen (tmptr) + 1);
		uae_strcpy (emutmp, tmptr);
		m68k_areg (regs, 0) = emutmp;
		Platform::DisposeMemory (tmptr);
	}
	else
	{
		uae_strcpy (tmpbuf, path);
		Platform_FfsLib::FfsGetcwd (libRefNum, tmpbuf, numchars);
		uae_strcpy (path, tmpbuf);
		m68k_areg (regs, 0) = path;
	}

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapMkdir()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapMkdir (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapMkdir");

	struct StackFrame
	{
		UInt16	libRefNum;
		char*	dirname;
	};

	PARAM_VAL (UInt16, libRefNum);
	PARAM_STR (Char, dirname);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsMkdir (libRefNum, dirname);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapRmdir()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapRmdir (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapRmdir");

	struct StackFrame
	{
		UInt16	libRefNum;
		char*	dirname;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_STR(Char, dirname);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsRmdir (libRefNum, dirname);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapFormat()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROMnothing
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapFormat (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapFormat");

	struct StackFrame
	{
		UInt16	libRefNum;
		UInt16	drive;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_VAL(UInt16, drive);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsFormat (libRefNum, drive);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapRename()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapRename (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapRename");

	struct StackFrame
	{
		UInt16	libRefNum;
		char*	path;
		char*	new_name;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_STR(Char, path);
	PARAM_STR(Char, new_name);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsRename (libRefNum, path, new_name);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapGetfileattr()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapGetfileattr (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapGetfileattr");

	struct StackFrame
	{
		UInt16	libRefNum;
		char*	name;
		UInt16*	attr;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_STR(Char,   name);
	PARAM_REF(UInt16, attr, Marshal::kOutput);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsGetfileattr (libRefNum, name, attr);

	attr.Put();

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapFlush()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROMnothing
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapFlush (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapFlush");

	struct StackFrame
	{
		UInt16	libRefNum;
		Int16	handle;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_VAL(Int16, handle);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsFlush (libRefNum, handle);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapFlushDisk()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapFlushDisk (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapFlushDisk");

	struct StackFrame
	{
		UInt16	libRefNum;
		UInt16	drive;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_VAL(UInt16, drive);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsFlushDisk (libRefNum, drive);

	return kSkipROM;
}

/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapSetfileattr()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapSetfileattr (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapSetfileattr");

	struct StackFrame
	{
		UInt16	libRefNum;
		char*	name;
		UInt16	attr;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_STR(Char, name);
	PARAM_VAL(UInt16, attr);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsSetfileattr (libRefNum, name, attr & 0xFFFF);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapStat()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapStat (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapStat");

	struct StackFrame
	{
		UInt16		libRefNum;
		char*		path;
		ffs_stat*	pstat;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_STR(Char, path);
	PARAM_REF(ffs_stat, pstat, Marshal::kOutput);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsStat (libRefNum, path, pstat);

	pstat.Put();

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapFstat()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROMnothing
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapFstat (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapFstat");

	struct StackFrame
	{
		UInt16		libRefNum;
		Int16		handle;
		ffs_stat*	pstat;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_VAL(Int16,  handle);
	PARAM_REF(ffs_stat, pstat, Marshal::kOutput);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsFstat (libRefNum, handle, pstat);

	pstat.Put();

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapIsDir()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapIsDir (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapIsDir");

	struct StackFrame
	{
		UInt16		libRefNum;
		char*		path;
		Boolean*	is_dir;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_STR(char, path);
	PARAM_REF(Boolean, is_dir, Marshal::kOutput);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsIsDir (libRefNum, path, is_dir);

	is_dir.Put();

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapLseek()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapLseek (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapLseek");

	struct StackFrame
	{
		UInt16	libRefNum;
		Int16	handle;
		Int32	offset;
		Int16	origin;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_VAL(Int16, handle);
	PARAM_VAL(Int32, offset);
	PARAM_VAL(Int16, origin);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsLseek (libRefNum, handle, offset, origin & 0xFFFF);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapGetErrno()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapGetErrno (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapGetErrno");

	struct StackFrame
	{
		UInt16	libRefNum;
	};

	PARAM_VAL(UInt16, libRefNum);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsGetErrno (libRefNum);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapTell()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROMnothing
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapTell (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapTell");

	struct StackFrame
	{
		UInt16	libRefNum;
		Int16	handle;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_VAL(Int16,  handle);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsTell (libRefNum, handle);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapGetdrive()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapGetdrive (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapGetdrive");

	struct StackFrame
	{
		UInt16	libRefNum;
		UInt16*	drive;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_REF(UInt16, drive, Marshal::kOutput);

	Platform_FfsLib::FfsGetdrive (libRefNum, drive);

	drive.Put();

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapSetdrive()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapSetdrive (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapSetdrive");

	struct StackFrame
	{
		UInt16	libRefNum;
		UInt16	drive;
		UInt16*	ndrives;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_VAL(UInt16, drive);
	PARAM_REF(UInt16, ndrives, Marshal::kOutput);

	Platform_FfsLib::FfsSetdrive (libRefNum, drive, ndrives);

	ndrives.Put();

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapUnlink()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROMnothing
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapUnlink (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapUnlink");

	struct StackFrame
	{
		UInt16	libRefNum;
		char*	path;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_STR(Char, path);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsUnlink (libRefNum, path);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapEof()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapEof (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapEof");

	struct StackFrame
	{
		UInt16	libRefNum;
		Int16	handle;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_VAL(Int16, handle);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsEof (libRefNum, handle);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapCreat()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapCreat (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapCreat");

	struct StackFrame
	{
		UInt16	libRefNum;
		char*	path;
		Int16	mode;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_STR(Char, path);
	PARAM_VAL(Int16, mode);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsCreat (libRefNum, path, mode & 0xFFFF);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapInstErrHandle()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapInstErrHandle (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapInstErrHandle");

/*---------------------------------------------------------------------------
 * Not implemented.
 *--------------------------------------------------------------------------*/

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapUnInstErrHandle()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapUnInstErrHandle (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapUnInstErrHandle");

/*---------------------------------------------------------------------------
 * Not Implelemented
 *--------------------------------------------------------------------------*/
	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapSetDebuggingOn()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapSetDebuggingOn (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapSetDebuggingOn");

	struct StackFrame
	{
		UInt16	libRefNum;
		UInt16	s_port;
		UInt16	level;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_VAL(UInt16, s_port);
	PARAM_VAL(UInt16, level);

	Platform_FfsLib::FfsSetDebuggingOn (libRefNum, s_port, level);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapSetDebuggingOff()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapSetDebuggingOff (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapSetDebuggingOff");

	struct StackFrame
	{
		UInt16	libRefNum;
	};

	PARAM_VAL(UInt16, libRefNum);

	Platform_FfsLib::FfsSetDebuggingOff (libRefNum);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapCardIsInserted()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapCardIsInserted (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapCardIsInserted");

	struct StackFrame
	{
		UInt16	libRefNum;
		UInt8	drive_num;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_VAL(UInt8, drive_num);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsCardIsInserted (libRefNum, drive_num);

	return kSkipROM;
}


/***********************************************************************
 *
 * FUNCTION:	FfsLibTrapCardIsATA()
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	none
 *
 * RETURNED:	kSkipROM
 *
 ***********************************************************************/

CallROMType FfsLibHeadpatch::FfsLibTrapCardIsATA (void)
{
	PRINTF ("----------------------------------------");
	PRINTF ("FfsLibTrapCardIsATA");

	struct StackFrame
	{
		UInt16	libRefNum;
		UInt8	drive_num;
	};

	PARAM_VAL(UInt16, libRefNum);
	PARAM_VAL(UInt8,  drive_num);

	m68k_dreg (regs, 0) = Platform_FfsLib::FfsCardIsATA (libRefNum, drive_num);

	return kSkipROM;
}


#include "PalmPackPop.h"
