/* -*- mode: C++; tab-width: 4 -*- */
/* ===================================================================== *\
	Copyright (c) 1998-2000 Palm, Inc. or its subsidiaries.
	All rights reserved.

	This file is part of the Palm OS Emulator.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
\* ===================================================================== */

#include "EmCommon.h"
#include "EmJPEGUnix.h"

#include "Platform.h"			// AllocateMemory

class EmJPEGDecompressBitsMemSink : public EmJPEGDecompressSink
{
	public:
								EmJPEGDecompressBitsMemSink	(void);
		virtual					~EmJPEGDecompressBitsMemSink	(void);

		virtual void			post_jpeg_read_header	(j_decompress_ptr cinfo);

		virtual void			start_output	(j_decompress_ptr cinfo);
		virtual void			put_pixel_rows	(j_decompress_ptr cinfo,
												 JDIMENSION rows_supplied);
		virtual void			finish_output	(j_decompress_ptr cinfo);

	public:
		void*					fBits;
		int						fWidth;
		int						fHeight;
		int						fRowBytes;

	private:
		JDIMENSION				fDataWidth;		/* JSAMPLEs per row */
		int						fPadBytes;		/* number of padding bytes needed per row */
		JDIMENSION				fCurOutputRow;	/* next row# to write to virtual array */
};



/***********************************************************************
 *
 * FUNCTION:	JPEGToBits
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

void JPEGToBits (const void* image, long imageLen,
				 const void*& bits, int& width, int& height, int& rowBytes)
{
	EmJPEGDecompressMemSource	src_data (image, imageLen);
	EmJPEGDecompressBitsMemSink	dest_data;

	::ConvertJPEG (src_data, dest_data);

	bits = dest_data.fBits;
	width = dest_data.fWidth;
	height = dest_data.fHeight;
	rowBytes = dest_data.fRowBytes;
}


/***********************************************************************
 *
 * FUNCTION:	JPEGToBits
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

void JPEGToBits (EmStream& stream,
				 const void*& bits, int& width, int& height, int& rowBytes)
{
	EmJPEGDecompressStreamSource	src_data (stream);
	EmJPEGDecompressBitsMemSink		dest_data;

	::ConvertJPEG (src_data, dest_data);

	bits = dest_data.fBits;
	width = dest_data.fWidth;
	height = dest_data.fHeight;
	rowBytes = dest_data.fRowBytes;
}


#pragma mark -

/***********************************************************************
 *
 * FUNCTION:	EmJPEGDecompressBitsMemSink c'tor
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

EmJPEGDecompressBitsMemSink::EmJPEGDecompressBitsMemSink (void) :
	EmJPEGDecompressSink (),
	fBits (NULL),
	fWidth (0),
	fHeight (0),
	fRowBytes (0),
	fDataWidth (0),
	fPadBytes (0),
	fCurOutputRow (0)
{
}


/***********************************************************************
 *
 * FUNCTION:	EmJPEGDecompressBitsMemSink d'tor
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

EmJPEGDecompressBitsMemSink::~EmJPEGDecompressBitsMemSink (void)
{
}


/***********************************************************************
 *
 * FUNCTION:	
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

void
EmJPEGDecompressBitsMemSink::post_jpeg_read_header (j_decompress_ptr cinfo)
{
	// Calculate output image dimensions so we can allocate space

	jpeg_calc_output_dimensions (cinfo);

	// Determine width of rows.

	fDataWidth = cinfo->output_width * cinfo->output_components;
	fRowBytes = fDataWidth;
	fPadBytes = fRowBytes - fDataWidth;

	// Prepare for write pass

	fCurOutputRow = 0;

	// Create decompressor output buffer.

	fBuffer = (*cinfo->mem->alloc_sarray)
		((j_common_ptr) cinfo, JPOOL_IMAGE, fDataWidth, 1);
	fBufferHeight = 1;

	long bitmap_size = fRowBytes * cinfo->output_height;
	fBits = Platform::AllocateMemory (bitmap_size);
}


/***********************************************************************
 *
 * FUNCTION:	EmJPEGDecompressBitsMemSink::start_output_bmp
 *
 * DESCRIPTION:	 * Startup: normally writes the file header.
 * In this module we may as well postpone everything until finish_output.
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

void EmJPEGDecompressBitsMemSink::start_output (j_decompress_ptr cinfo)
{
	fWidth		= cinfo->output_width;
	fHeight		= cinfo->output_height;
}


/***********************************************************************
 *
 * FUNCTION:	EmJPEGDecompressBitsMemSink::put_pixel_rows
 *
 * DESCRIPTION:	Write some pixel data.  In this module rows_supplied
 *				will always be 1.
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

void EmJPEGDecompressBitsMemSink::put_pixel_rows (j_decompress_ptr cinfo, JDIMENSION)
{
	int		line	= fCurOutputRow++;
	char*	outptr	= ((char*) fBits) + line * fRowBytes;

	// Transfer data.

	JSAMPROW	inptr = fBuffer[0];

	for (JDIMENSION col = 0; col < cinfo->output_width; ++col)
	{
		*outptr++ = *inptr++;
		*outptr++ = *inptr++;
		*outptr++ = *inptr++;
	}

	// Zero out the pad bytes.

	int pad = fPadBytes;
	while (--pad >= 0)
		*outptr++ = 0;
}


/***********************************************************************
 *
 * FUNCTION:	finish_output_bmp
 *
 * DESCRIPTION:	
 *
 * PARAMETERS:	None
 *
 * RETURNED:	Nothing
 *
 ***********************************************************************/

void EmJPEGDecompressBitsMemSink::finish_output (j_decompress_ptr)
{
}
